﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.ComponentModel.Design;
    using System.Runtime.InteropServices;
    using VSLangProj;

    internal class AddReferenceCommand : OleMenuCommand, IVsComponentUser, IServiceProvider
    {
        private string assemblyLocations;
        private IXnaCodeProject xnaProject;

        internal AddReferenceCommand(IXnaCodeProject xnaProject) : this(xnaProject, CmdIDConstants.AddReferenceCmdID)
        {
        }

        protected AddReferenceCommand(IXnaCodeProject xnaProject, CommandID commandID) : base(new EventHandler(AddReferenceCommand.OnAddReference), new EventHandler(AddReferenceCommand.OnCommandModified), new EventHandler(AddReferenceCommand.OnBeforeQueryStatus), commandID)
        {
            if (xnaProject == null)
            {
                throw new ArgumentNullException("xnaProject");
            }
            this.xnaProject = xnaProject;
            string[] sdkPaths = this.xnaProject.SdkPaths;
            this.assemblyLocations = string.Join(";", sdkPaths);
        }

        public int AddComponent(VSADDCOMPOPERATION dwAddCompOperation, uint cComponents, IntPtr[] rgpcsdComponents, IntPtr hwndPickerDlg, VSADDCOMPRESULT[] pResult)
        {
            if (rgpcsdComponents == null)
            {
                throw new ArgumentNullException("rgpcsdComponents");
            }
            if (cComponents > rgpcsdComponents.Length)
            {
                throw new ArgumentOutOfRangeException("cComponents");
            }
            VSADDCOMPRESULT vsaddcompresult = VSADDCOMPRESULT.ADDCOMPRESULT_Success;
            References references = Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(this.xnaProject.GetVsHierarchy()).References;
            for (int i = 0; i < cComponents; i++)
            {
                VSADDCOMPRESULT vsaddcompresult2 = VSADDCOMPRESULT.ADDCOMPRESULT_Failure;
                VSCOMPONENTSELECTORDATA vscomponentselectordata = new VSCOMPONENTSELECTORDATA();
                IntPtr ptr = rgpcsdComponents[i];
                if (ptr != IntPtr.Zero)
                {
                    vscomponentselectordata = (VSCOMPONENTSELECTORDATA) Marshal.PtrToStructure(ptr, typeof(VSCOMPONENTSELECTORDATA));
                }
                else
                {
                    vsaddcompresult = VSADDCOMPRESULT.ADDCOMPRESULT_Failure;
                    continue;
                }
                string message = null;
                OLEMSGICON icon = OLEMSGICON.OLEMSGICON_CRITICAL;
                OLEMSGBUTTON msgButton = OLEMSGBUTTON.OLEMSGBUTTON_OK;
                OLEMSGDEFBUTTON defaultButton = OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST;
                try
                {
                    Reference reference = null;
                    switch (vscomponentselectordata.type)
                    {
                        case VSCOMPONENTTYPE.VSCOMPONENTTYPE_File:
                        case VSCOMPONENTTYPE.VSCOMPONENTTYPE_ComPlus:
                        case VSCOMPONENTTYPE.VSCOMPONENTTYPE_Path:
                            reference = references.Add(vscomponentselectordata.bstrFile);
                            vsaddcompresult2 = VSADDCOMPRESULT.ADDCOMPRESULT_Success;
                            break;

                        case VSCOMPONENTTYPE.VSCOMPONENTTYPE_Com2:
                            message = XnaProjectResource.CannotAddCom;
                            break;

                        case VSCOMPONENTTYPE.VSCOMPONENTTYPE_Project:
                        {
                            IVsHierarchy hierarchy;
                            object obj2;
                            IVsSolution requiredService = Microsoft.Xna.GameStudio.ProjectUtilities.GetRequiredService<IVsSolution>(this);
                            string pbstrUpdatedProjref = null;
                            ErrorHandler.ThrowOnFailure(requiredService.GetProjectOfProjref(vscomponentselectordata.bstrProjRef, out hierarchy, out pbstrUpdatedProjref, null));
                            ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(0xfffffffe, -2027, out obj2));
                            Project pProject = obj2 as Project;
                            if (pProject != null)
                            {
                                reference = references.AddProject(pProject);
                                vsaddcompresult2 = VSADDCOMPRESULT.ADDCOMPRESULT_Success;
                            }
                            break;
                        }
                        case VSCOMPONENTTYPE.VSCOMPONENTTYPE_Custom:
                            throw new InvalidOperationException();
                    }
                    if (reference != null)
                    {
                        this.OnAfterAddReference(reference);
                    }
                }
                catch (BadImageFormatException exception)
                {
                    vsaddcompresult2 = VSADDCOMPRESULT.ADDCOMPRESULT_Failure;
                    message = exception.Message;
                }
                catch (COMException exception2)
                {
                    if (exception2.ErrorCode == -2147221492)
                    {
                        vsaddcompresult2 = VSADDCOMPRESULT.ADDCOMPRESULT_Cancel;
                    }
                    else
                    {
                        vsaddcompresult2 = VSADDCOMPRESULT.ADDCOMPRESULT_Failure;
                        message = exception2.Message;
                    }
                }
                if (vsaddcompresult2 == VSADDCOMPRESULT.ADDCOMPRESULT_Failure)
                {
                    if (message == null)
                    {
                        message = XnaProjectResource.FailureToAddProjectReference;
                    }
                    VsShellUtilities.ShowMessageBox(this, message, vscomponentselectordata.bstrTitle, icon, msgButton, defaultButton);
                }
                if ((vsaddcompresult == VSADDCOMPRESULT.ADDCOMPRESULT_Success) || (vsaddcompresult2 == VSADDCOMPRESULT.ADDCOMPRESULT_Failure))
                {
                    vsaddcompresult = vsaddcompresult2;
                }
            }
            if ((pResult != null) && (pResult.Length > 0))
            {
                pResult[0] = vsaddcompresult;
            }
            return 0;
        }

        public object GetService(Type serviceType)
        {
            return this.xnaProject.GetVsService(serviceType);
        }

        protected virtual void OnAddReference()
        {
            this.xnaProject.GetVsService(typeof(SVsComponentSelectorDlg));
            VSCOMPONENTSELECTORTABINIT[] tabInit = new VSCOMPONENTSELECTORTABINIT[4];
            tabInit[0].dwSize = (uint) Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[0].varTabInitInfo = this.assemblyLocations;
            tabInit[0].guidTab = VSConstants.GUID_COMPlusPage;
            tabInit[1].dwSize = (uint) Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[1].varTabInitInfo = -2055;
            tabInit[1].guidTab = VSConstants.GUID_SolutionPage;
            tabInit[2].dwSize = (uint) Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[2].guidTab = VSConstants.GUID_BrowseFilePage;
            tabInit[2].varTabInitInfo = 0;
            Guid guid = new Guid("{19B97F03-9594-4c1c-BE28-25FF030113B3}");
            tabInit[3].dwSize = (uint) Marshal.SizeOf(typeof(VSCOMPONENTSELECTORTABINIT));
            tabInit[3].guidTab = guid;
            tabInit[3].varTabInitInfo = 0;
            string browseFilter = XnaProjectResource.XboxAddReferenceFilterName + "\0" + XnaProjectResource.XboxAddReferenceFilter + "\0";
            string solutionDir = this.xnaProject.GetSolutionDir();
            string helpTopic = "VS.AddReference";
            this.ShowComponentSelectorDialog(tabInit, XnaProjectResource.AddRefTitle, browseFilter, solutionDir, helpTopic);
        }

        private static void OnAddReference(object sender, EventArgs args)
        {
            AddReferenceCommand command = sender as AddReferenceCommand;
            if (command != null)
            {
                command.OnAddReference();
            }
        }

        protected virtual void OnAfterAddReference(Reference reference)
        {
        }

        protected virtual void OnBeforeQueryStatus()
        {
        }

        private static void OnBeforeQueryStatus(object sender, EventArgs args)
        {
            AddReferenceCommand command = sender as AddReferenceCommand;
            if (command != null)
            {
                command.OnBeforeQueryStatus();
            }
        }

        protected virtual void OnCommandModified()
        {
        }

        private static void OnCommandModified(object sender, EventArgs args)
        {
            AddReferenceCommand command = sender as AddReferenceCommand;
            if (command != null)
            {
                command.OnCommandModified();
            }
        }

        protected int ShowComponentSelectorDialog(VSCOMPONENTSELECTORTABINIT[] tabInit, string title, string browseFilter, string browseLocations, string helpTopic)
        {
            if (title == null)
            {
                throw new ArgumentNullException("title");
            }
            int hr = 0;
            IVsComponentSelectorDlg2 vsService = this.xnaProject.GetVsService(typeof(SVsComponentSelectorDlg)) as IVsComponentSelectorDlg2;
            if (vsService != null)
            {
                uint pxDlgSize = 0;
                uint pyDlgSize = 0;
                Guid empty = Guid.Empty;
                if (browseFilter == null)
                {
                    browseFilter = string.Empty;
                }
                if (browseLocations == null)
                {
                    browseLocations = string.Empty;
                }
                try
                {
                    this.xnaProject.ShowProjectInSolutionPage = false;
                    uint grfFlags = 0x49;
                    hr = vsService.ComponentSelectorDlg2(grfFlags, this, 0, null, title, helpTopic, ref pxDlgSize, ref pyDlgSize, (uint) tabInit.Length, tabInit, ref empty, browseFilter, ref browseLocations);
                    ErrorHandler.ThrowOnFailure(hr, new int[] { -2147221492 });
                }
                finally
                {
                    this.xnaProject.ShowProjectInSolutionPage = true;
                }
            }
            return hr;
        }

        protected string AssemblyLocations
        {
            get
            {
                return this.assemblyLocations;
            }
        }

        protected IXnaCodeProject XnaProject
        {
            get
            {
                return this.xnaProject;
            }
        }
    }
}

