﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.Build.Construction;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Win32;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Diagnostics;
    using System.Drawing;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Threading;
    using System.Xml;

    public abstract class BasePlatformAdapter : IProjectPlatformAdapter
    {
        private string languageName;
        private IServiceProvider serviceProvider;

        internal BasePlatformAdapter()
        {
        }

        internal virtual void AddFilesToProject(XmlNodeList files, string configPath, ProjectRootElement project, List<string> errors)
        {
            string str = string.Empty;
            foreach (ProjectPropertyElement element in project.Properties)
            {
                if (element.Name.Equals("XnaOutputType"))
                {
                    str = element.Value;
                }
            }
            foreach (XmlNode node in files)
            {
                if ((node.Attributes["ConstrainXnaOutputType"] == null) || node.Attributes["ConstrainXnaOutputType"].Value.Equals(str))
                {
                    string path = Path.Combine(configPath, node.InnerText);
                    string innerText = node.InnerText;
                    if (node.Attributes["RelativePath"] != null)
                    {
                        innerText = Path.Combine(node.Attributes["RelativePath"].Value, node.InnerText);
                    }
                    string destFileName = Path.Combine(project.DirectoryPath, innerText);
                    if (File.Exists(path))
                    {
                        try
                        {
                            File.Copy(path, destFileName, true);
                            this.ProcessFile(path, destFileName, project);
                        }
                        catch (Exception exception)
                        {
                            errors.Add(exception.Message);
                        }
                    }
                    project.AddItem(node.Attributes["IncludeType"].Value, innerText).AddMetadata("XnaPlatformSpecific", "true");
                }
            }
        }

        public virtual void AddPlatformSpecificComponents(ProjectRootElement project, List<string> errors)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            if (errors == null)
            {
                throw new ArgumentNullException("errors");
            }
            string baseConfigPath = this.GetBaseConfigPath();
            if (!string.IsNullOrEmpty(baseConfigPath))
            {
                string path = Path.Combine(baseConfigPath, this.languageName, this.XnaPlatform);
                if (Directory.Exists(path))
                {
                    string str3 = Path.Combine(path, "_xnadefault.config");
                    if (File.Exists(str3))
                    {
                        XmlDocument propertiesConfigFile = new XmlDocument();
                        propertiesConfigFile.Load(str3);
                        this.AddPropertiesToProject(propertiesConfigFile, project);
                        this.AddFilesToProject(propertiesConfigFile.SelectSingleNode("//Files").ChildNodes, path, project, errors);
                    }
                }
            }
        }

        internal virtual void AddPropertiesToProject(XmlDocument propertiesConfigFile, ProjectRootElement project)
        {
            if (propertiesConfigFile == null)
            {
                throw new ArgumentNullException("propertiesConfigFile");
            }
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            IEnumerable<ProjectPropertyGroupElement> propertyGroups = project.PropertyGroups;
            List<ProjectPropertyGroupElement> list = new List<ProjectPropertyGroupElement>();
            List<ProjectPropertyGroupElement> list2 = new List<ProjectPropertyGroupElement>();
            foreach (ProjectPropertyGroupElement element in propertyGroups)
            {
                if (element.Condition.Contains("Release"))
                {
                    list2.Add(element);
                }
                if (element.Condition.Contains("Debug"))
                {
                    list.Add(element);
                }
            }
            this.AddPropertiesToPropertyGroups(propertiesConfigFile.SelectSingleNode("//Debug").ChildNodes, list);
            this.AddPropertiesToPropertyGroups(propertiesConfigFile.SelectSingleNode("//Release").ChildNodes, list2);
            foreach (XmlNode node in propertiesConfigFile.SelectSingleNode("//Global").ChildNodes)
            {
                if (!this.ShouldPreserveAttribute(node, project))
                {
                    ProjectPropertyElement element2 = project.AddProperty(node.Name, node.InnerText);
                    if (node.Attributes["Condition"] != null)
                    {
                        element2.Condition = node.Attributes["Condition"].Value;
                    }
                }
            }
        }

        internal virtual void AddPropertiesToPropertyGroups(XmlNodeList nodes, List<ProjectPropertyGroupElement> propertyGroups)
        {
            if (nodes == null)
            {
                throw new ArgumentNullException("nodes");
            }
            if (propertyGroups == null)
            {
                throw new ArgumentNullException("propertyGroups");
            }
            foreach (XmlNode node in nodes)
            {
                string name = node.Name;
                string innerText = node.InnerText;
                foreach (ProjectPropertyGroupElement element in propertyGroups)
                {
                    ProjectPropertyElement element2 = element.AddProperty(node.Name, node.InnerText);
                    if (node.Attributes["Condition"] != null)
                    {
                        element2.Condition = node.Attributes["Condition"].Value;
                    }
                }
            }
        }

        public virtual IVsProjectFlavorCfg CreateProjectFlavorCfg(IServiceProvider serviceProvider, IVsCfg baseProjectCfg, IVsProjectFlavorCfg innerFlavorCfg)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (baseProjectCfg == null)
            {
                throw new ArgumentNullException("baseProjectCfg");
            }
            return innerFlavorCfg;
        }

        public virtual IEnumerable<string> FilteredPropertyPages()
        {
            if (!this.IsDevicePlatform)
            {
                yield break;
            }
            yield return "{DF8F7042-0BB1-47D1-8E6D-DEB3D07698BD}";
            yield return "{CC4014F5-B18D-439C-9352-F99D984CCA85}";
            yield return "{43E38D2E-43B8-4204-8225-9357316137A4}";
        }

        internal virtual string GetBaseConfigPath()
        {
            string str = Registry.GetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\XNA\Game Studio\v4.0", "InstallPath", null) as string;
            if (string.IsNullOrEmpty(str))
            {
                return null;
            }
            return Path.Combine(str, "Config");
        }

        public virtual string[] GetDeviceSdkPaths()
        {
            return new string[0];
        }

        public virtual string[] GetFilterProperties()
        {
            return new string[0];
        }

        public virtual object GetHierarchyObject(Type objectType)
        {
            return null;
        }

        internal virtual bool ProcessFile(string sourcePath, string destinationPath, ProjectRootElement project)
        {
            return false;
        }

        public virtual void SetLanguageName(string languageName)
        {
            this.languageName = languageName;
        }

        public virtual void SetSite(IServiceProvider serviceProvider)
        {
            this.serviceProvider = serviceProvider;
        }

        internal virtual bool ShouldPreserveAttribute(XmlNode attribute, ProjectRootElement project)
        {
            if ((attribute != null) && ((attribute.Attributes["Preserve"] != null) && bool.TrueString.Equals(attribute.Attributes["Preserve"].Value, StringComparison.OrdinalIgnoreCase)))
            {
                foreach (ProjectPropertyElement element in project.Properties)
                {
                    if (element.Name.Equals(attribute.Name))
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        public virtual Guid AddItemTemplatesGuid
        {
            get
            {
                return base.GetType().GUID;
            }
        }

        public virtual IList<INamedExtenderProvider> ExtenderProviders
        {
            get
            {
                return new ReadOnlyCollection<INamedExtenderProvider>(new List<INamedExtenderProvider>(0));
            }
        }

        public virtual string[] GetDisabledPropertyList
        {
            get
            {
                return new string[0];
            }
        }

        public abstract Microsoft.Xna.GameStudio.CodeProject.HelpContextFilters HelpContextFilters { get; }

        public abstract bool IsDevicePlatform { get; }

        public virtual string NestedContentXnaPlatform
        {
            get
            {
                return this.XnaPlatform;
            }
        }

        public virtual string PlatformConfigurationName
        {
            get
            {
                return this.XnaPlatform;
            }
        }

        public abstract string PlatformConstants { get; }

        public abstract Bitmap ProjectNodeIcon { get; }

        public virtual Guid ProjectTypeGuid
        {
            get
            {
                return base.GetType().GUID;
            }
        }

        protected IServiceProvider ServiceProvider
        {
            get
            {
                return this.serviceProvider;
            }
        }

        public virtual bool SupportsContentCompression
        {
            get
            {
                return true;
            }
        }

        public virtual bool SupportsCrossPlatform
        {
            get
            {
                return true;
            }
        }

        public virtual bool SupportsDeviceCenter
        {
            get
            {
                return true;
            }
        }

        public virtual bool SupportsPackaging
        {
            get
            {
                return true;
            }
        }

        public virtual bool SupportsServiceReferences
        {
            get
            {
                return false;
            }
        }

        public abstract string XnaFrameworkRuntimeProfile { get; }

        public abstract string XnaFrameworkVersion { get; }

        public abstract string XnaPlatform { get; }

    }
}

