﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Debugger.Interop;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.Xna.GameStudio.CodeProject.Debugging.Interop;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal class DebugPort : IDebugPort2, IDebugPortNotify2, IConnectionPointContainer, IConnectionPoint, IRemoteDevicePort, IRemoteDeviceConnection, IEmbeddedRemoteCorDebugPort
    {
        private const int ConnectTimeout = 0x7530;
        private EventSinkCollection eventSinks;
        private Guid guid;
        private Dictionary<Guid, IDebugProcess2> mapGuidToProcess;
        private Dictionary<uint, IDebugProcess2> mapPidToProcess;
        private List<IntPtr> nativeStrings;
        private IPlatform platform;
        private string platformName;
        private IDebugPortRequest2 portRequest;
        private XnaPortSupplier portSupplier;
        private string runtimeToken;
        private string targetName;

        private DebugPort(IDebugPortRequest2 portRequest)
        {
            if (portRequest == null)
            {
                throw new ArgumentNullException("portRequest");
            }
            this.portRequest = portRequest;
            this.guid = Guid.NewGuid();
            this.mapPidToProcess = new Dictionary<uint, IDebugProcess2>();
            this.mapGuidToProcess = new Dictionary<Guid, IDebugProcess2>();
            this.nativeStrings = new List<IntPtr>();
            string pbstrPortName = null;
            ErrorHandler.ThrowOnFailure(this.portRequest.GetPortName(out pbstrPortName));
            if (!XnaPortSupplier.TryParsePortName(pbstrPortName, out this.platformName, out this.runtimeToken, out this.targetName))
            {
                throw new ArgumentException();
            }
        }

        internal DebugPort(string portName) : this(new PrivateRequest(portName))
        {
        }

        internal DebugPort(IDebugPortRequest2 portRequest, XnaPortSupplier portSupplier) : this(portRequest)
        {
            if (portSupplier == null)
            {
                throw new ArgumentNullException("portSupplier");
            }
            this.portSupplier = portSupplier;
        }

        public void Advise(object pUnkSink, out uint pdwCookie)
        {
            if (pUnkSink == null)
            {
                throw new ArgumentNullException("pUnkSink");
            }
            if (this.eventSinks == null)
            {
                this.eventSinks = new EventSinkCollection();
            }
            pdwCookie = this.eventSinks.Add(pUnkSink);
        }

        public void EnumConnectionPoints(out IEnumConnectionPoints ppEnum)
        {
            throw new NotImplementedException();
        }

        public void EnumConnections(out IEnumConnections ppEnum)
        {
            throw new NotImplementedException();
        }

        public void FindConnectionPoint(ref Guid riid, out IConnectionPoint ppCP)
        {
            ppCP = null;
            if (riid == typeof(IDebugPortEvents2).GUID)
            {
                ppCP = this;
            }
        }

        public void GetConnectionInterface(out Guid pIID)
        {
            throw new NotImplementedException();
        }

        public void GetConnectionPointContainer(out IConnectionPointContainer ppCPC)
        {
            throw new NotImplementedException();
        }

        internal virtual void InitializePlatform()
        {
            if (this.platform == null)
            {
                this.platform = PlatformManager.GetPlatform(this.platformName, this.runtimeToken);
            }
            if (this.platform == null)
            {
                throw new InvalidOperationException(XnaProjectResource.DebugNoPlatformAvailable);
            }
        }

        int IDebugPort2.EnumProcesses(out IEnumDebugProcesses2 ppEnum)
        {
            ppEnum = new EnumDebugProcesses(this.mapPidToProcess.Values);
            return 0;
        }

        int IDebugPort2.GetPortId(out Guid pguidPort)
        {
            pguidPort = this.guid;
            return 0;
        }

        int IDebugPort2.GetPortName(out string pbstrName)
        {
            return this.portRequest.GetPortName(out pbstrName);
        }

        int IDebugPort2.GetPortRequest(out IDebugPortRequest2 ppRequest)
        {
            ppRequest = this.portRequest;
            return 0;
        }

        int IDebugPort2.GetPortSupplier(out IDebugPortSupplier2 ppSupplier)
        {
            ppSupplier = this.portSupplier;
            return 0;
        }

        int IDebugPort2.GetProcess(AD_PROCESS_ID ProcessId, out IDebugProcess2 ppProcess)
        {
            ppProcess = null;
            if (ProcessId.ProcessIdType == 0)
            {
                if (!this.mapPidToProcess.TryGetValue(ProcessId.dwProcessId, out ppProcess))
                {
                    throw new ArgumentException(XnaProjectResource.InvalidProcessId, "ProcessId");
                }
            }
            else
            {
                if (ProcessId.ProcessIdType != 1)
                {
                    throw new ArgumentException(XnaProjectResource.InvalidProcessIdType, "ProcessId");
                }
                if (!this.mapGuidToProcess.TryGetValue(ProcessId.guidProcessId, out ppProcess))
                {
                    throw new ArgumentException(XnaProjectResource.InvalidProcessId, "ProcessId");
                }
            }
            return 0;
        }

        int IDebugPortNotify2.AddProgramNode(IDebugProgramNode2 pProgramNode)
        {
            IDebugProcess2 process;
            if (pProgramNode == null)
            {
                throw new ArgumentNullException("pProgramNode");
            }
            AD_PROCESS_ID[] pHostProcessId = new AD_PROCESS_ID[1];
            ErrorHandler.ThrowOnFailure(pProgramNode.GetHostPid(pHostProcessId));
            ErrorHandler.ThrowOnFailure(((IDebugPort2) this).GetProcess(pHostProcessId[0], out process));
            IDebugProgram2 program = ((DebugProcess) process).Program;
            IDebugEvent2 portEvent = new PortEvent();
            Guid gUID = typeof(IDebugProgramCreateEvent2).GUID;
            this.SendPortEvent(null, process, program, portEvent, ref gUID);
            return 0;
        }

        int IDebugPortNotify2.RemoveProgramNode(IDebugProgramNode2 pProgramNode)
        {
            throw new NotImplementedException();
        }

        void IEmbeddedRemoteCorDebugPort.NotifyProcessDestroy(uint in_dwProcessId)
        {
            IDebugProcess2 process;
            if (!this.mapPidToProcess.TryGetValue(in_dwProcessId, out process))
            {
                throw new ArgumentOutOfRangeException("in_dwProcessId");
            }
            DebugProcess process2 = (DebugProcess) process;
            IDebugProgram2 program = process2.Program;
            IDebugEvent2 portEvent = new PortEvent();
            Guid gUID = typeof(IDebugProgramDestroyEvent2).GUID;
            this.SendPortEvent(null, process, program, portEvent, ref gUID);
            this.mapPidToProcess.Remove(in_dwProcessId);
            this.mapGuidToProcess.Remove(process2.Guid);
        }

        void IEmbeddedRemoteCorDebugPort.TerminateProcess(uint in_dwProcessId, uint in_dwExitCode)
        {
            throw new NotImplementedException();
        }

        void IEmbeddedRemoteCorDebugPort.Unused_0()
        {
            throw new NotImplementedException();
        }

        void IEmbeddedRemoteCorDebugPort.Unused_1()
        {
            throw new NotImplementedException();
        }

        void IEmbeddedRemoteCorDebugPort.Unused_2()
        {
            throw new NotImplementedException();
        }

        void IRemoteDeviceConnection.CreateStream(string in_szTransportParameters, out ICcTransportStream out_ppConnection)
        {
            out_ppConnection = null;
            if (in_szTransportParameters == null)
            {
                throw new ArgumentNullException("in_szTransportParameters");
            }
            this.InitializePlatform();
            TransportStreamAdapter adapter = new TransportStreamAdapter(this.platform, this.targetName, in_szTransportParameters);
            try
            {
                adapter.Connect(0x7530);
                out_ppConnection = adapter;
                adapter = null;
            }
            finally
            {
                IDisposable disposable = adapter;
                if (disposable != null)
                {
                    disposable.Dispose();
                }
            }
        }

        void IRemoteDevicePort.EnumServiceProviders(string in_szServiceId, out IEnumServiceState ppEnum)
        {
            throw new NotImplementedException();
        }

        void IRemoteDevicePort.FreeString(IntPtr in_szString)
        {
            if (this.nativeStrings.Contains(in_szString))
            {
                try
                {
                    Marshal.FreeCoTaskMem(in_szString);
                }
                finally
                {
                    this.nativeStrings.Remove(in_szString);
                }
            }
        }

        void IRemoteDevicePort.GetEndPointInformation(uint in_dwTimeOut, string in_szServiceId, uint in_dwProcessId, IntPtr out_pszTransportParameters)
        {
            if (in_szServiceId == null)
            {
                throw new ArgumentNullException("in_szServiceId");
            }
            if (!this.mapPidToProcess.ContainsKey(in_dwProcessId))
            {
                throw new ArgumentOutOfRangeException("in_dwProcessId");
            }
            if (out_pszTransportParameters == IntPtr.Zero)
            {
                throw new ArgumentNullException("out_pszTransportParameters");
            }
            IntPtr val = Marshal.StringToCoTaskMemUni(in_szServiceId);
            try
            {
                Marshal.WriteIntPtr(out_pszTransportParameters, val);
            }
            catch
            {
                Marshal.FreeCoTaskMem(val);
                throw;
            }
            this.nativeStrings.Add(val);
        }

        void IRemoteDevicePort.GetServiceState(string in_szServiceId, uint in_dwProcessId, out bool out_pfCanConnect)
        {
            throw new NotImplementedException();
        }

        void IRemoteDevicePort.Launch(string in_szHostParameters, uint in_dwTimeOut, string in_szApplicationName, string in_szCommandLine, string in_szCurrentDirectory, uint in_EnvSize, IntPtr in_wchEnvironment, uint in_dwCreationFlags, IntPtr out_pdwProcessId, IntPtr out_pszHostParameters)
        {
            this.InitializePlatform();
            ITitlePlayer titlePlayer = this.platform.GetTitlePlayer(this.targetName);
            if (titlePlayer == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DebugNoTargetAvailable, new object[] { this.targetName }));
            }
            ILaunchService service = titlePlayer.GetService(typeof(ILaunchService)) as ILaunchService;
            try
            {
                if (string.IsNullOrEmpty(in_szApplicationName))
                {
                    throw new InvalidOperationException(XnaProjectResource.Xbox360LaunchFailedMissingTitleID);
                }
                new Guid(in_szApplicationName);
            }
            catch (FormatException exception)
            {
                throw new InvalidOperationException(XnaProjectResource.Xbox360LaunchFailedMissingTitleID, exception);
            }
            ITitleProcess process = service.LaunchTitle(in_szApplicationName, in_szCommandLine, in_szHostParameters);
            DebugProcess process2 = new DebugProcess((uint) process.Id, process.Name, process.StartupAssembly, this);
            this.mapGuidToProcess[process2.Guid] = process2;
            this.mapPidToProcess[process2.Pid] = process2;
            if (out_pdwProcessId != IntPtr.Zero)
            {
                Marshal.WriteInt32(out_pdwProcessId, (int) process2.Pid);
            }
            if (out_pszHostParameters != IntPtr.Zero)
            {
                IntPtr val = Marshal.StringToCoTaskMemUni(process.StartupHostParameters);
                bool flag = false;
                try
                {
                    Marshal.WriteIntPtr(out_pszHostParameters, val);
                    flag = true;
                }
                finally
                {
                    if (!flag)
                    {
                        Marshal.FreeCoTaskMem(val);
                    }
                }
                this.nativeStrings.Add(val);
            }
        }

        void IRemoteDevicePort.ShutdownHost(uint in_dwTimeOut, string in_szServiceId)
        {
            throw new NotImplementedException();
        }

        private void SendPortEvent(IDebugCoreServer2 server, IDebugProcess2 process, IDebugProgram2 program, IDebugEvent2 portEvent, ref Guid eventIID)
        {
            foreach (object obj2 in (IEnumerable) this.eventSinks)
            {
                IDebugPortEvents2 events = obj2 as IDebugPortEvents2;
                if (events != null)
                {
                    ErrorHandler.ThrowOnFailure(events.Event(server, this, process, program, portEvent, ref eventIID));
                }
            }
        }

        public void Unadvise(uint dwCookie)
        {
            if (this.eventSinks == null)
            {
                throw new InvalidOperationException();
            }
            this.eventSinks.RemoveAt(dwCookie);
        }

        private class PrivateRequest : IDebugPortRequest2
        {
            private string name;

            internal PrivateRequest(string name)
            {
                this.name = name;
            }

            public int GetPortName(out string pbstrPortName)
            {
                pbstrPortName = this.name;
                return 0;
            }
        }
    }
}

