﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.VisualStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.DeviceManagement;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Runtime.CompilerServices;

    internal class DeployManager
    {
        private DeviceDeployTransaction currentTransaction;
        internal const string defaultThumbnail = "Default Thumbnail";
        private FileStream deployCacheFile;
        private Dictionary<string, DateTime> deployedFiles;
        private string lastDeviceId;
        private string lastOutputPath;
        private string lastTitleId;
        private long lastUsedContainer;
        private IXnaCodeProject project;
        private DateTime thumbDeployTime;
        private string thumbnail;

        internal DeployManager(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.project = project;
        }

        internal DeployTransaction CreateTransaction(PlatformFlavorCfg configuration, string targetConsole, DeployParams deployParams)
        {
            if (configuration == null)
            {
                throw new ArgumentNullException("configuration");
            }
            if (deployParams == null)
            {
                throw new ArgumentNullException("deployParams");
            }
            string titleID = deployParams.TitleID;
            string outputDirectory = deployParams.OutputDirectory;
            if (this.currentTransaction != null)
            {
                throw new InvalidOperationException();
            }
            this.deployCacheFile = this.RefreshFromDeployCache(targetConsole);
            string str3 = targetConsole;
            bool thumbnailChanged = false;
            if (this.thumbnail != deployParams.Thumbnail)
            {
                thumbnailChanged = true;
                this.thumbnail = deployParams.Thumbnail;
                this.thumbDeployTime = DateTime.MinValue;
            }
            Dictionary<string, DateTime> alreadyDeployedFiles = null;
            if (((str3 == this.lastDeviceId) && (titleID == this.lastTitleId)) && (outputDirectory == this.lastOutputPath))
            {
                alreadyDeployedFiles = this.deployedFiles;
            }
            this.currentTransaction = new DeviceDeployTransaction(targetConsole, titleID, thumbnailChanged, this.thumbnail, this.thumbDeployTime, new DeployCompleteCallback(this.OnDeployTransactionComplete), outputDirectory, alreadyDeployedFiles);
            this.currentTransaction.DeployTargetCookie = this.lastUsedContainer;
            return this.currentTransaction;
        }

        internal static string GetDeployCacheDirectory(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string buildProperty = project.GetBuildProperty("MSBuildProjectDirectory", string.Empty);
            string str2 = project.GetBuildProperty("BaseIntermediateOutputPath", string.Empty);
            return Path.Combine(buildProperty, str2);
        }

        internal static string GetDeployCachePath(IXnaCodeProject project, string targetName)
        {
            string str2;
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string deployCacheDirectory = GetDeployCacheDirectory(project);
            ErrorHandler.ThrowOnFailure(project.InnerVsProject.GetMkDocument(0xfffffffe, out str2));
            return Path.Combine(deployCacheDirectory, Path.GetFileName(str2) + "." + DeviceInfo.GetSafeDeviceName(targetName) + ".DeployCache.txt");
        }

        private void OnDeployTransactionComplete(DeviceDeployTransaction transaction)
        {
            if ((transaction != null) && (this.currentTransaction != null))
            {
                if (!object.ReferenceEquals(this.currentTransaction, transaction))
                {
                    throw new InvalidOperationException();
                }
                this.currentTransaction = null;
                try
                {
                    if (transaction.Committed)
                    {
                        this.lastTitleId = transaction.DeployTitleId;
                        this.lastDeviceId = transaction.DeployTarget;
                        this.lastOutputPath = transaction.SourceDirectory;
                        this.thumbnail = transaction.Thumbnail;
                        this.thumbDeployTime = transaction.ThumbnailDeployTime;
                        this.lastUsedContainer = transaction.DeployTargetCookie;
                        this.deployedFiles = transaction.UpdatedFileList;
                        if (this.deployCacheFile == null)
                        {
                            return;
                        }
                        this.deployCacheFile.SetLength(0L);
                        using (StreamWriter writer = new StreamWriter(new NonDisposingStream(this.deployCacheFile)))
                        {
                            writer.WriteLine(this.lastTitleId);
                            writer.WriteLine(0);
                            writer.WriteLine(this.lastDeviceId);
                            writer.WriteLine(this.lastOutputPath);
                            writer.WriteLine(this.lastUsedContainer.ToString(CultureInfo.InvariantCulture));
                            writer.WriteLine(this.thumbnail);
                            writer.WriteLine(this.thumbDeployTime.ToString("o", CultureInfo.InvariantCulture));
                            foreach (KeyValuePair<string, DateTime> pair in this.deployedFiles)
                            {
                                writer.WriteLine(pair.Key);
                                writer.WriteLine(pair.Value.ToString("o", CultureInfo.InvariantCulture));
                            }
                            writer.Flush();
                            return;
                        }
                    }
                    if (!transaction.Cancelled)
                    {
                        this.Reset(transaction.DeployTarget);
                    }
                }
                finally
                {
                    if (this.deployCacheFile != null)
                    {
                        this.deployCacheFile.Dispose();
                        this.deployCacheFile = null;
                    }
                }
            }
        }

        private FileStream RefreshFromDeployCache(string targetName)
        {
            string deployCachePath = GetDeployCachePath(this.project, targetName);
            FileStream innerStream = null;
            this.lastTitleId = null;
            this.deployedFiles = null;
            this.lastDeviceId = null;
            try
            {
                string directoryName = Path.GetDirectoryName(deployCachePath);
                if (!Directory.Exists(directoryName))
                {
                    Directory.CreateDirectory(directoryName);
                }
                innerStream = new FileStream(deployCachePath, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.None);
            }
            catch (IOException)
            {
            }
            catch (UnauthorizedAccessException)
            {
            }
            if (innerStream != null)
            {
                using (StreamReader reader = new StreamReader(new NonDisposingStream(innerStream)))
                {
                    string str5;
                    this.lastTitleId = reader.ReadLine();
                    reader.ReadLine();
                    this.lastDeviceId = reader.ReadLine();
                    this.lastOutputPath = reader.ReadLine();
                    this.lastUsedContainer = 0L;
                    if (((this.lastTitleId == null) || (this.lastDeviceId == null)) || (this.lastOutputPath == null))
                    {
                        return innerStream;
                    }
                    string s = reader.ReadLine();
                    this.thumbnail = reader.ReadLine();
                    string str4 = reader.ReadLine();
                    if (string.IsNullOrEmpty(str4))
                    {
                        this.thumbDeployTime = DateTime.MinValue;
                    }
                    else
                    {
                        try
                        {
                            this.thumbDeployTime = DateTime.ParseExact(str4, "o", CultureInfo.InvariantCulture, DateTimeStyles.RoundtripKind);
                        }
                        catch (FormatException)
                        {
                            this.thumbDeployTime = DateTime.MinValue;
                        }
                    }
                    if (!(this.thumbDeployTime != DateTime.MinValue) || !long.TryParse(s, NumberStyles.AllowLeadingSign, CultureInfo.InvariantCulture, out this.lastUsedContainer))
                    {
                        return innerStream;
                    }
                    this.deployedFiles = new Dictionary<string, DateTime>();
                Label_0148:
                    str5 = reader.ReadLine();
                    string str6 = reader.ReadLine();
                    if (str5 == null)
                    {
                        return innerStream;
                    }
                    if (string.IsNullOrEmpty(str6))
                    {
                        this.deployedFiles.Clear();
                        return innerStream;
                    }
                    if (!str5.StartsWith(this.lastOutputPath, StringComparison.InvariantCultureIgnoreCase) || (str6 == null))
                    {
                        this.deployedFiles.Clear();
                        return innerStream;
                    }
                    try
                    {
                        DateTime time = DateTime.ParseExact(str6, "o", CultureInfo.InvariantCulture, DateTimeStyles.RoundtripKind);
                        this.deployedFiles[str5] = time;
                        goto Label_0148;
                    }
                    catch (ArgumentException)
                    {
                        this.deployedFiles.Clear();
                    }
                    catch (FormatException)
                    {
                        this.deployedFiles.Clear();
                    }
                }
            }
            return innerStream;
        }

        internal void Reset(string targetName)
        {
            if (this.deployCacheFile != null)
            {
                this.deployCacheFile.Dispose();
                this.deployCacheFile = null;
            }
            string deployCachePath = GetDeployCachePath(this.project, targetName);
            if (File.Exists(deployCachePath))
            {
                try
                {
                    File.Delete(deployCachePath);
                }
                catch (IOException)
                {
                }
                catch (UnauthorizedAccessException)
                {
                }
            }
        }

        private delegate void DeployCompleteCallback(DeployManager.DeviceDeployTransaction transaction);

        private class DeviceDeployTransaction : DeployTransaction
        {
            private string deployTarget;
            private long deployTargetCookie;
            private string deployTitleId;
            private DeployManager.DeployCompleteCallback onCompleteCallback;
            private string outputDirectory;
            private DateTime thumbDeployTime;
            private string thumbnail;
            private Dictionary<string, DateTime> updatedFileList;

            internal DeviceDeployTransaction(string deployTarget, string deployTitleId, bool thumbnailChanged, string thumbnail, DateTime thumbDeployTime, DeployManager.DeployCompleteCallback callback, string outputDirectory, Dictionary<string, DateTime> alreadyDeployedFiles) : base((alreadyDeployedFiles == null) || (alreadyDeployedFiles.Count == 0))
            {
                if (callback == null)
                {
                    throw new ArgumentNullException("callback");
                }
                this.deployTarget = deployTarget;
                this.deployTitleId = deployTitleId;
                this.thumbnail = thumbnail;
                this.thumbDeployTime = thumbDeployTime;
                this.onCompleteCallback = callback;
                this.outputDirectory = outputDirectory;
                this.updatedFileList = new Dictionary<string, DateTime>();
                this.ComputeUpdate(thumbnailChanged, alreadyDeployedFiles);
            }

            private void ComputeUpdate(bool thumbnailChanged, Dictionary<string, DateTime> alreadyDeployedFiles)
            {
                if (alreadyDeployedFiles == null)
                {
                    alreadyDeployedFiles = new Dictionary<string, DateTime>();
                }
                if (!string.IsNullOrEmpty(this.thumbnail))
                {
                    if (this.thumbnail == "Default Thumbnail")
                    {
                        if (thumbnailChanged)
                        {
                            base.FilesToDeploy.Add(this.thumbnail);
                            this.thumbDeployTime = DateTime.UtcNow;
                        }
                    }
                    else
                    {
                        FileInfo info = new FileInfo(this.thumbnail);
                        if (thumbnailChanged || (info.LastWriteTimeUtc > this.thumbDeployTime))
                        {
                            base.FilesToDeploy.Add(this.thumbnail);
                            this.thumbDeployTime = info.LastWriteTimeUtc;
                        }
                    }
                }
                if (this.outputDirectory != null)
                {
                    DirectoryInfo info2 = new DirectoryInfo(this.outputDirectory);
                    foreach (FileInfo info3 in info2.GetFiles("*", SearchOption.AllDirectories))
                    {
                        string[] strArray = new string[] { ".pdb", ".ccgame", ".lastcodeanalysissucceeded", ".codeanalysislog.xml" };
                        bool flag = false;
                        foreach (string str in strArray)
                        {
                            if (info3.FullName.EndsWith(str, StringComparison.InvariantCultureIgnoreCase))
                            {
                                flag = true;
                                break;
                            }
                        }
                        if (!flag)
                        {
                            DateTime time;
                            if (alreadyDeployedFiles.TryGetValue(info3.FullName, out time))
                            {
                                if (time < info3.LastWriteTimeUtc)
                                {
                                    base.FilesToDeploy.Add(info3.FullName);
                                    this.updatedFileList.Add(info3.FullName, info3.LastWriteTimeUtc);
                                }
                                else
                                {
                                    base.FilesUpToDate.Add(info3.FullName);
                                    this.updatedFileList.Add(info3.FullName, time);
                                }
                            }
                            else
                            {
                                base.FilesToDeploy.Add(info3.FullName);
                                this.updatedFileList.Add(info3.FullName, info3.LastWriteTimeUtc);
                            }
                        }
                    }
                }
                foreach (string str2 in alreadyDeployedFiles.Keys)
                {
                    if (!base.FilesUpToDate.Contains(str2) && !base.FilesToDeploy.Contains(str2))
                    {
                        base.FilesNotFound.Add(str2);
                    }
                }
            }

            protected override void OnCancel()
            {
                this.updatedFileList.Clear();
                this.onCompleteCallback(this);
            }

            protected override void OnCommit()
            {
                this.onCompleteCallback(this);
            }

            protected override void OnReset()
            {
                this.onCompleteCallback(this);
            }

            internal string DeployTarget
            {
                get
                {
                    return this.deployTarget;
                }
            }

            public long DeployTargetCookie
            {
                get
                {
                    return this.deployTargetCookie;
                }
                set
                {
                    this.deployTargetCookie = value;
                }
            }

            internal string DeployTitleId
            {
                get
                {
                    if (this.deployTitleId == null)
                    {
                        return string.Empty;
                    }
                    return this.deployTitleId;
                }
            }

            public string SourceDirectory
            {
                get
                {
                    if (this.outputDirectory == null)
                    {
                        return string.Empty;
                    }
                    return this.outputDirectory;
                }
            }

            internal override object Tag
            {
                get
                {
                    return this.DeployTargetCookie;
                }
                set
                {
                    this.DeployTargetCookie = (long) value;
                }
            }

            public string Thumbnail
            {
                get
                {
                    return this.thumbnail;
                }
            }

            public DateTime ThumbnailDeployTime
            {
                get
                {
                    return this.thumbDeployTime;
                }
            }

            internal Dictionary<string, DateTime> UpdatedFileList
            {
                get
                {
                    return this.updatedFileList;
                }
            }
        }
    }
}

