﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using Microsoft.Xna.GameStudio.CodeProject.Debugging.Interop;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.IO;
    using System.Runtime.InteropServices;

    internal class TransportStreamAdapter : ICcTransportStream, IDisposable
    {
        private bool connected;
        private string hostName;
        private byte[] nextByte;
        private IPlatform platform;
        private Stream serviceClient;
        private string serviceName;

        internal TransportStreamAdapter(IPlatform platform, string hostName, string serviceName)
        {
            if (platform == null)
            {
                throw new ArgumentNullException("platform");
            }
            if (hostName == null)
            {
                throw new ArgumentNullException("hostName");
            }
            if (serviceName == null)
            {
                throw new ArgumentNullException("serviceName");
            }
            this.platform = platform;
            this.hostName = hostName;
            this.serviceName = serviceName;
        }

        internal void Connect(int millisecondTimeout)
        {
            if (millisecondTimeout < -1)
            {
                throw new ArgumentOutOfRangeException("millisecondTimeout");
            }
            if (this.connected)
            {
                throw new InvalidOperationException(XnaProjectResource.StreamAlreadyConnected);
            }
            ITitlePlayer titlePlayer = this.platform.GetTitlePlayer(this.hostName);
            if (titlePlayer == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DebugUnableToConnectDebugStream, new object[] { this.hostName }));
            }
            try
            {
                this.serviceClient = titlePlayer.ConnectToService(this.serviceName, millisecondTimeout);
                this.connected = true;
            }
            catch (COMException exception)
            {
                if (exception.ErrorCode == -2146233083)
                {
                    throw new TimeoutException(exception.Message, exception);
                }
                throw;
            }
            finally
            {
                if (!this.connected)
                {
                    IDisposable serviceClient = this.serviceClient;
                    this.serviceClient = null;
                    if (serviceClient != null)
                    {
                        serviceClient.Dispose();
                    }
                }
            }
        }

        [SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId="disposing")]
        protected void Dispose(bool disposing)
        {
            try
            {
                IDisposable serviceClient = this.serviceClient;
                if (serviceClient != null)
                {
                    serviceClient.Dispose();
                }
            }
            catch (COMException)
            {
            }
            finally
            {
                this.serviceClient = null;
            }
        }

        ~TransportStreamAdapter()
        {
            this.Dispose(false);
        }

        void ICcTransportStream.Disconnect()
        {
            try
            {
                if (this.serviceClient != null)
                {
                    this.serviceClient.Close();
                    IDisposable serviceClient = this.serviceClient;
                    if (serviceClient != null)
                    {
                        serviceClient.Dispose();
                    }
                }
            }
            finally
            {
                this.serviceClient = null;
                this.connected = false;
            }
        }

        bool ICcTransportStream.IsConnected()
        {
            return this.connected;
        }

        bool ICcTransportStream.IsDataAvailable()
        {
            if ((this.nextByte == null) && this.connected)
            {
                try
                {
                    byte[] buffer = new byte[1];
                    if (this.serviceClient.Read(buffer, 1, 0) == 1)
                    {
                        this.nextByte = buffer;
                    }
                }
                catch (InvalidOperationException)
                {
                    this.connected = false;
                }
                catch (COMException)
                {
                    this.connected = false;
                }
            }
            return (this.nextByte != null);
        }

        void ICcTransportStream.Recv(uint dwTimeout, IntPtr pBuffer, IntPtr pcbBuffer)
        {
            if (pBuffer == IntPtr.Zero)
            {
                throw new ArgumentNullException("pBuffer");
            }
            if (pcbBuffer == IntPtr.Zero)
            {
                throw new ArgumentNullException("pcbBuffer");
            }
            int num = Marshal.ReadInt32(pcbBuffer);
            Marshal.WriteInt32(pcbBuffer, 0);
            if (num < 0)
            {
                throw new ArgumentOutOfRangeException("pcbBuffer");
            }
            if ((this.nextByte == null) && !this.connected)
            {
                throw new InvalidOperationException(XnaProjectResource.StreamNotConnected);
            }
            int num2 = num / 0x4000;
            List<byte[]> list = new List<byte[]>(num2 + 2);
            int length = 0;
            if (this.nextByte != null)
            {
                list.Add(this.nextByte);
                this.nextByte = null;
                length++;
            }
            int num4 = 0;
            int num5 = (dwTimeout == uint.MaxValue) ? -1 : ((int) Math.Min(dwTimeout, 0x7fffffff));
            this.serviceClient.ReadTimeout = num5;
            while (num > length)
            {
                num4 = Math.Min(0x4000, num - length);
                if ((num4 > 0) && this.connected)
                {
                    bool flag = false;
                    try
                    {
                        byte[] buffer = new byte[num4];
                        int num6 = this.serviceClient.Read(buffer, 0, buffer.Length);
                        flag = true;
                        if (num6 > 0)
                        {
                            if (num6 < length)
                            {
                                byte[] destinationArray = new byte[num6];
                                Array.Copy(buffer, destinationArray, num6);
                                buffer = destinationArray;
                            }
                            list.Add(buffer);
                            length += buffer.Length;
                        }
                        continue;
                    }
                    finally
                    {
                        this.connected = flag;
                    }
                }
            }
            byte[] array = new byte[num];
            int index = 0;
            foreach (byte[] buffer4 in list)
            {
                buffer4.CopyTo(array, index);
                index += buffer4.Length;
            }
            if (length > 0)
            {
                Marshal.Copy(array, 0, pBuffer, length);
            }
            Marshal.WriteInt32(pcbBuffer, length);
            if (length < num)
            {
                if (num5 != -1)
                {
                    throw new TimeoutException();
                }
                throw new InvalidOperationException(XnaProjectResource.StreamNotConnected);
            }
        }

        void ICcTransportStream.Send(uint dwTimeout, byte[] pData, uint cbData)
        {
            if (pData == null)
            {
                throw new ArgumentNullException("pData");
            }
            if (cbData != pData.Length)
            {
                throw new ArgumentException(XnaProjectResource.LengthDoesNotMatchBuffer, "cbData");
            }
            if (cbData > 0x7fffffff)
            {
                throw new ArgumentOutOfRangeException("cbData");
            }
            if (cbData != 0)
            {
                if (!this.connected)
                {
                    throw new InvalidOperationException(XnaProjectResource.StreamNotConnected);
                }
                int num = (dwTimeout == uint.MaxValue) ? -1 : ((int) Math.Max(0x7fffffff, dwTimeout));
                bool flag = false;
                try
                {
                    this.serviceClient.WriteTimeout = num;
                    this.serviceClient.Write(pData, 0, (int) cbData);
                    flag = true;
                }
                catch (COMException exception)
                {
                    if (exception.ErrorCode == -2146233083)
                    {
                        flag = true;
                        throw new TimeoutException(exception.Message, exception);
                    }
                    throw;
                }
                finally
                {
                    this.connected = flag;
                }
            }
        }

        void IDisposable.Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        bool ICcTransportStream.Buffering
        {
            get
            {
                return false;
            }
            set
            {
            }
        }

        string ICcTransportStream.ID
        {
            get
            {
                return this.serviceName;
            }
        }
    }
}

