﻿namespace Microsoft.Xna.GameStudio.CodeProject.Debugging
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Threading;

    internal class XnaDeployableProjectCfg : IVsDeployableProjectCfg
    {
        private IContainer activeDeployChannel;
        private volatile EventSinkCollection adviseSink;
        private object criticalSection;
        private volatile bool deployCancelled;
        private object deployChannelLock;
        private volatile System.Threading.Thread deploymentThread;
        private XnaErrorProvider errorProvider;
        private string f1HelpKeyword;
        private PlatformFlavorCfg parentCfg;

        internal XnaDeployableProjectCfg(PlatformFlavorCfg parentCfg, string f1HelpKeyword)
        {
            if (parentCfg == null)
            {
                throw new ArgumentNullException("parentCfg");
            }
            if (f1HelpKeyword == null)
            {
                throw new ArgumentNullException("f1HelpKeyword");
            }
            this.parentCfg = parentCfg;
            this.f1HelpKeyword = f1HelpKeyword;
            this.criticalSection = new object();
            this.errorProvider = this.parentCfg.Project.ErrorProvider;
            this.deployChannelLock = new object();
        }

        public int AdviseDeployStatusCallback(IVsDeployStatusCallback pIVsDeployStatusCallback, out uint pdwCookie)
        {
            if (pIVsDeployStatusCallback == null)
            {
                throw new ArgumentNullException("pIVsDeployStatusCallback");
            }
            lock (this.criticalSection)
            {
                if (this.adviseSink == null)
                {
                    this.adviseSink = new EventSinkCollection();
                }
                pdwCookie = this.adviseSink.Add(pIVsDeployStatusCallback);
            }
            return 0;
        }

        public int Commit(uint dwReserved)
        {
            return 0;
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void DeploymentThreadStart(object startParameter)
        {
            DeployParams deployParams = null;
            IVsOutputWindowPane outputWindow = null;
            bool succeeded = false;
            try
            {
                deployParams = (DeployParams) startParameter;
                outputWindow = deployParams.OutputWindow;
                string targetName = deployParams.TargetName;
                string titleID = deployParams.TitleID;
                string title = deployParams.Title;
                DeployTransaction deployTransaction = deployParams.DeployTransaction;
                IPlatform platform = PlatformManager.GetPlatform(this.parentCfg.Project.Adapter.XnaPlatform, this.parentCfg.XnaFrameworkRuntimeProfile);
                if (platform == null)
                {
                    throw new InvalidOperationException(XnaProjectResource.DeployNoPlatformAvailable);
                }
                ITitlePlayer titlePlayer = platform.GetTitlePlayer(targetName);
                if (titlePlayer == null)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DeployNoTargetAvailable, new object[] { targetName }));
                }
                ILaunchService service = titlePlayer.GetService(typeof(ILaunchService)) as ILaunchService;
                if (service.IsTitleRunning())
                {
                    string error = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360DeployErrorTitleAlreadyExecuting, new object[] { targetName });
                    this.parentCfg.Project.ErrorProvider.ReportError(error, this.f1HelpKeyword);
                    WriteLine(outputWindow, error, new object[0]);
                }
                else
                {
                    IRuntimeProfileService service2 = titlePlayer.GetService(typeof(IRuntimeProfileService)) as IRuntimeProfileService;
                    if (service2 != null)
                    {
                        try
                        {
                            WriteLine(outputWindow, XnaProjectResource.DeployingRuntime, new object[0]);
                            service2.EnsureRuntimeIsAvailable(deployParams.RuntimeProfile);
                        }
                        catch (Exception exception)
                        {
                            string str5 = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.RuntimeDeployFailedError, new object[] { targetName, exception.Message });
                            this.errorProvider.ReportError(str5, this.f1HelpKeyword);
                            WriteLine(outputWindow, str5, new object[0]);
                            return;
                        }
                    }
                    IDeployService service3 = titlePlayer.GetService(typeof(IDeployService)) as IDeployService;
                    string containerId = service3.GetContainerId(titleID);
                    if (deployTransaction.CleanDeploy)
                    {
                        service3.DeleteContainer(containerId);
                    }
                    long tag = (long) deployTransaction.Tag;
                    long token = 0L;
                    using (IContainer container = service3.OpenSpecificContainer(containerId, tag, title))
                    {
                        token = container.Token;
                        bool reopened = container.Reopened;
                        deployTransaction.Tag = token;
                        if (reopened)
                        {
                            WriteLine(outputWindow, XnaProjectResource.Xbox360DeployContainerReopened, new object[] { titleID });
                        }
                        else
                        {
                            WriteLine(outputWindow, XnaProjectResource.Xbox360DeployContainerCreated, new object[] { titleID });
                        }
                        if (!reopened && !deployTransaction.CleanDeploy)
                        {
                            deployTransaction.Reset();
                            deployTransaction.Dispose();
                            deployParams.DeployTransaction = this.parentCfg.DeployManager.CreateTransaction(this.parentCfg, targetName, deployParams);
                            deployTransaction = deployParams.DeployTransaction;
                            deployTransaction.Tag = token;
                        }
                        WriteLine(outputWindow, XnaProjectResource.Xbox360DeployOutputDirectory, new object[] { deployParams.OutputDirectory });
                        bool flag3 = deployTransaction.FilesToDeploy.Remove(deployParams.Thumbnail);
                        if ((deployTransaction.FilesToDeploy.Count > 0) || (deployTransaction.FilesNotFound.Count > 0))
                        {
                            if ((!this.deployCancelled && (deployTransaction.FilesNotFound.Count > 0)) && reopened)
                            {
                                RemoveFilesFromGameContainer(deployParams, outputWindow, container);
                            }
                            if (!this.deployCancelled && (deployTransaction.FilesToDeploy.Count > 0))
                            {
                                this.PutFilesInGameContainer(deployParams, container, outputWindow);
                            }
                        }
                        else
                        {
                            WriteLine(outputWindow, XnaProjectResource.Xbox360DeployAllFilesUpToDate, new object[0]);
                        }
                        if (!this.deployCancelled && ((!reopened || deployTransaction.CleanDeploy) || flag3))
                        {
                            string thumbnail = deployParams.Thumbnail;
                            try
                            {
                                if (thumbnail == "Default Thumbnail")
                                {
                                    WriteLine(outputWindow, XnaProjectResource.Xbox360DeploySetThumbnailNoPath, new object[0]);
                                    container.SetThumbnail(string.Empty);
                                }
                                else if (File.Exists(thumbnail))
                                {
                                    WriteLine(outputWindow, XnaProjectResource.Xbox360DeploySetThumbnail, new object[] { thumbnail });
                                    container.SetThumbnail(thumbnail);
                                }
                                else
                                {
                                    ArgumentException warning = new ArgumentException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.Xbox360DeploySetThumbnailInvalidPath, new object[] { thumbnail }));
                                    WriteLine(outputWindow, warning.Message, new object[0]);
                                    this.errorProvider.ReportWarning(warning, this.f1HelpKeyword);
                                    container.SetThumbnail(string.Empty);
                                }
                            }
                            catch (ArgumentException exception3)
                            {
                                WriteLine(outputWindow, XnaProjectResource.Xbox360DeploySetThumbnailFailed, new object[] { exception3.Message });
                                this.errorProvider.ReportWarning(exception3, this.f1HelpKeyword);
                            }
                            catch (PlatformToolsException exception4)
                            {
                                WriteLine(outputWindow, XnaProjectResource.Xbox360DeploySetThumbnailFailed, new object[] { exception4.Message });
                                this.errorProvider.ReportWarning(exception4, this.f1HelpKeyword);
                            }
                        }
                    }
                    if (this.deployCancelled)
                    {
                        WriteLine(outputWindow, XnaProjectResource.Xbox360DeployCancelled, new object[0]);
                    }
                    else
                    {
                        service3.SetTitleProperties(titleID, title, deployParams.Description, deployParams.StartupAssembly, deployParams.RuntimeProfile);
                        if (!this.deployCancelled)
                        {
                            deployTransaction.Commit();
                        }
                        succeeded = !this.deployCancelled;
                        if (succeeded)
                        {
                            TitlePropertiesHelper.DisplayTitleProperties(outputWindow, deployParams);
                        }
                    }
                }
            }
            catch (ThreadAbortException)
            {
                System.Threading.Thread.ResetAbort();
                WriteLine(outputWindow, XnaProjectResource.Xbox360DeployCancelled, new object[0]);
            }
            catch (PlatformToolsException exception5)
            {
                succeeded = false;
                WriteLine(outputWindow, XnaProjectResource.Xbox360DeployFailedError, new object[] { exception5.Message });
                this.errorProvider.ReportError(exception5, this.f1HelpKeyword);
            }
            catch (Exception exception6)
            {
                succeeded = false;
                WriteLine(outputWindow, XnaProjectResource.Xbox360DeployFailedError, new object[] { exception6.Message });
                this.errorProvider.ReportError(exception6, this.f1HelpKeyword);
            }
            finally
            {
                if ((deployParams != null) && (deployParams.DeployTransaction != null))
                {
                    deployParams.DeployTransaction.Dispose();
                    deployParams.DeployTransaction = null;
                }
                this.deploymentThread = null;
                this.OnEndDeploy(outputWindow, succeeded);
            }
        }

        private IEnumerable<IVsDeployStatusCallback> GetDeployStatusCallbacks()
        {
            List<IVsDeployStatusCallback> list;
            lock (this.criticalSection)
            {
                int capacity = (this.adviseSink == null) ? 0 : this.adviseSink.Count;
                list = new List<IVsDeployStatusCallback>(capacity);
                if (capacity <= 0)
                {
                    return list;
                }
                foreach (IVsDeployStatusCallback callback in this.adviseSink)
                {
                    list.Add(callback);
                }
            }
            return list;
        }

        private string GetOutputDirectory()
        {
            string buildProperty = this.parentCfg.Project.GetBuildProperty("MSBuildProjectDirectory", string.Empty);
            string str2 = this.parentCfg.Project.GetBuildProperty("OutputPath", this.parentCfg.GetConfigurationName());
            return Path.GetFullPath(Path.Combine(buildProperty, str2));
        }

        private void GetTitleProperties(DeployParams deployParams)
        {
            object obj2;
            if (deployParams == null)
            {
                throw new ArgumentNullException("deployParams");
            }
            string str = null;
            ErrorHandler.ThrowOnFailure(this.parentCfg.Project.GetVsHierarchy().GetProperty(0xfffffffe, -2027, out obj2));
            Project project = obj2 as Project;
            if ((project != null) && (project.Properties != null))
            {
                foreach (string str2 in DeployParams.ProjectProperties)
                {
                    try
                    {
                        Property property = project.Properties.Item(str2);
                        if (property != null)
                        {
                            deployParams.SetProperty(str2, property.Value as string);
                        }
                    }
                    catch (ArgumentException)
                    {
                    }
                }
                Guid assemblyGuid = Microsoft.Xna.GameStudio.ProjectUtilities.GetAssemblyGuid(project);
                if (assemblyGuid == Guid.Empty)
                {
                    throw new InvalidOperationException(XnaProjectResource.Xbox360DeployFailedMissingTitleID);
                }
                str = assemblyGuid.ToString("N", CultureInfo.InvariantCulture);
            }
            deployParams.RuntimeProfile = this.parentCfg.XnaFrameworkRuntimeProfile;
            if (string.IsNullOrEmpty(str))
            {
                throw new InvalidOperationException(XnaProjectResource.Xbox360DeployFailedMissingTitleID);
            }
            deployParams.TitleID = str;
            deployParams.Thumbnail = this.parentCfg.Project.GetFullThumbnailPath();
            if (string.IsNullOrEmpty(deployParams.Thumbnail))
            {
                deployParams.Thumbnail = "Default Thumbnail";
            }
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OnEndDeploy(IVsOutputWindowPane outputWindow, bool succeeded)
        {
            int fSuccessful = succeeded ? 1 : 0;
            foreach (IVsDeployStatusCallback callback in this.GetDeployStatusCallbacks())
            {
                try
                {
                    ErrorHandler.ThrowOnFailure(callback.OnEndDeploy(fSuccessful));
                }
                catch (Exception exception)
                {
                    WriteLine(outputWindow, XnaProjectResource.Xbox360DeployStatusNotifyError, new object[] { exception.ToString() });
                    this.errorProvider.ReportError(exception, this.f1HelpKeyword);
                }
            }
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private bool OnStartDeploy(IVsOutputWindowPane outputWindow)
        {
            bool flag = true;
            int pfContinue = 1;
            foreach (IVsDeployStatusCallback callback in this.GetDeployStatusCallbacks())
            {
                try
                {
                    ErrorHandler.ThrowOnFailure(callback.OnStartDeploy(ref pfContinue));
                    flag = flag && (pfContinue != 0);
                }
                catch (Exception exception)
                {
                    WriteLine(outputWindow, XnaProjectResource.Xbox360DeployStatusNotifyError, new object[] { exception.ToString() });
                    this.errorProvider.ReportError(exception, this.f1HelpKeyword);
                }
            }
            return flag;
        }

        private void PutFilesInGameContainer(DeployParams deployParams, IContainer openContainer, IVsOutputWindowPane outputWindow)
        {
            if (deployParams == null)
            {
                throw new ArgumentNullException("deployParams");
            }
            if (openContainer == null)
            {
                throw new ArgumentNullException("openContainer");
            }
            DeployTransaction deployTransaction = deployParams.DeployTransaction;
            int length = deployParams.OutputDirectory.Length;
            if (deployParams.OutputDirectory[length - 1] != Path.DirectorySeparatorChar)
            {
                length++;
            }
            lock (this.deployChannelLock)
            {
                this.activeDeployChannel = openContainer;
            }
            try
            {
                foreach (string str in deployTransaction.FilesToDeploy)
                {
                    string remoteFilePath = str.Substring(length);
                    WriteLine(outputWindow, XnaProjectResource.Xbox360DeploySendFile, new object[] { remoteFilePath });
                    openContainer.SendFile(str, remoteFilePath);
                    if (this.deployCancelled)
                    {
                        return;
                    }
                }
            }
            finally
            {
                lock (this.deployChannelLock)
                {
                    this.activeDeployChannel = null;
                }
            }
        }

        public int QueryStartDeploy(uint dwOptions, int[] pfSupported, int[] pfReady)
        {
            if ((pfSupported != null) && (pfSupported.Length > 0))
            {
                pfSupported[0] = this.parentCfg.Project.IsEntryProject ? 1 : 0;
            }
            if ((pfReady != null) && (pfReady.Length > 0))
            {
                pfReady[0] = (this.deploymentThread == null) ? 1 : 0;
            }
            return 0;
        }

        public int QueryStatusDeploy(out int pfDeployDone)
        {
            pfDeployDone = 1;
            if (this.deploymentThread != null)
            {
                pfDeployDone = 0;
            }
            return 0;
        }

        private static void RemoveFilesFromGameContainer(DeployParams deployParams, IVsOutputWindowPane outputWindow, IContainer openContainer)
        {
            if (deployParams == null)
            {
                throw new ArgumentNullException("deployParams");
            }
            if (openContainer == null)
            {
                throw new ArgumentNullException("openContainer");
            }
            DeployTransaction deployTransaction = deployParams.DeployTransaction;
            int length = deployParams.OutputDirectory.Length;
            if (deployParams.OutputDirectory[length - 1] != Path.DirectorySeparatorChar)
            {
                length++;
            }
            foreach (string str in deployTransaction.FilesNotFound)
            {
                string remoteFilePath = str.Substring(length);
                WriteLine(outputWindow, XnaProjectResource.Xbox360DeployFileNotFound, new object[] { remoteFilePath });
                openContainer.DeleteFile(remoteFilePath);
            }
        }

        public int Rollback(uint dwReserved)
        {
            return 0;
        }

        public int StartDeploy(IVsOutputWindowPane pIVsOutputWindowPane, uint dwOptions)
        {
            if (pIVsOutputWindowPane == null)
            {
                throw new ArgumentNullException("pIVsOutputWindowPane");
            }
            this.errorProvider.Tasks.Clear();
            System.Threading.Thread thread = new System.Threading.Thread(new ParameterizedThreadStart(this.DeploymentThreadStart));
            if (Interlocked.CompareExchange<System.Threading.Thread>(ref this.deploymentThread, thread, null) != null)
            {
                throw new InvalidOperationException();
            }
            this.deployCancelled = false;
            bool flag = this.OnStartDeploy(pIVsOutputWindowPane);
            DeployParams deployParams = new DeployParams(pIVsOutputWindowPane);
            bool flag2 = false;
            try
            {
                string targetConsole = this.parentCfg.TargetConsole;
                if (string.IsNullOrEmpty(targetConsole))
                {
                    string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(this.parentCfg.Project.GetVsHierarchy());
                    throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DeployNoDefaultDeviceRegistered, new object[] { projectName }));
                }
                if (flag && !this.deployCancelled)
                {
                    WriteLine(pIVsOutputWindowPane, XnaProjectResource.Xbox360DeployBegin, new object[] { targetConsole });
                    deployParams.TargetName = targetConsole;
                    deployParams.OutputDirectory = this.GetOutputDirectory();
                    this.GetTitleProperties(deployParams);
                    deployParams.DeployTransaction = this.parentCfg.DeployManager.CreateTransaction(this.parentCfg, targetConsole, deployParams);
                    this.deploymentThread.Start(deployParams);
                    flag2 = true;
                }
            }
            catch (Exception exception)
            {
                this.errorProvider.ReportError(exception, this.f1HelpKeyword);
                throw;
            }
            finally
            {
                if (!flag2)
                {
                    this.OnEndDeploy(pIVsOutputWindowPane, false);
                    this.deploymentThread = null;
                }
            }
            return 0;
        }

        public int StopDeploy(int fSync)
        {
            System.Threading.Thread deploymentThread = this.deploymentThread;
            if (deploymentThread != null)
            {
                this.deployCancelled = true;
                lock (this.deployChannelLock)
                {
                    if (this.activeDeployChannel != null)
                    {
                        try
                        {
                            this.activeDeployChannel.AbortSendFile();
                        }
                        catch (ObjectDisposedException)
                        {
                        }
                    }
                }
                try
                {
                    if ((fSync != 0) && !deploymentThread.Join(0x2710))
                    {
                        deploymentThread.Abort();
                        if (!deploymentThread.Join(0x1388))
                        {
                            return -2147467259;
                        }
                    }
                }
                finally
                {
                    this.deploymentThread = null;
                }
            }
            return 0;
        }

        public int UnadviseDeployStatusCallback(uint dwCookie)
        {
            lock (this.criticalSection)
            {
                if (this.adviseSink == null)
                {
                    throw new InvalidOperationException();
                }
                this.adviseSink.RemoveAt(dwCookie);
            }
            return 0;
        }

        public int WaitDeploy(uint dwMilliseconds, int fTickWhenMessageQNotEmpty)
        {
            return 0;
        }

        private static void WriteLine(IVsOutputWindowPane outputWindow, string format, params object[] args)
        {
            if (format == null)
            {
                throw new ArgumentNullException("format");
            }
            if (outputWindow != null)
            {
                string str = format + "\n";
                if (args != null)
                {
                    str = string.Format(CultureInfo.InvariantCulture, str, args);
                }
                ErrorHandler.ThrowOnFailure(outputWindow.OutputStringThreadSafe(str));
            }
        }
    }
}

