﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using VSLangProj;

    internal class HierarchyItemManager : IDisposable
    {
        private IXnaCodeProject codeProject;
        private Dictionary<uint, ContentReferenceNode> contentReferenceItems;
        private Microsoft.Xna.GameStudio.ContentReferencesFolderNode contentReferencesNode;
        private uint firstContentReferenceItem;
        private IVsHierarchy hierarchy;
        private uint itemBeforeContentReferences;
        private ContentReferenceNode lastDeletedItem;
        private uint lkgReferencesFolderId = uint.MaxValue;
        private bool needsRefresh;
        private XnaProjectNode projectNode;
        private Microsoft.Xna.GameStudio.CodeProject.ReferencesFolderNode referencesFolderNode;
        private bool visible;

        internal HierarchyItemManager(IVsHierarchy hierarchy, IXnaCodeProject codeProject)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            if (codeProject == null)
            {
                throw new ArgumentNullException("codeProject");
            }
            this.hierarchy = codeProject.InnerIVsHierarchy;
            this.codeProject = codeProject;
            this.needsRefresh = true;
            this.itemBeforeContentReferences = uint.MaxValue;
            this.firstContentReferenceItem = uint.MaxValue;
            this.projectNode = new XnaProjectNode(codeProject);
            this.contentReferenceItems = new Dictionary<uint, ContentReferenceNode>();
            this.contentReferencesNode = new Microsoft.Xna.GameStudio.ContentReferencesFolderNode(codeProject);
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.projectNode != null)
                {
                    this.projectNode.Dispose();
                }
                if (this.contentReferencesNode != null)
                {
                    this.contentReferencesNode.Dispose();
                }
            }
        }

        ~HierarchyItemManager()
        {
            this.Dispose(false);
        }

        internal ContentReferenceNode GetContentReferenceNode(uint itemid)
        {
            ContentReferenceNode node;
            this.Refresh();
            this.contentReferenceItems.TryGetValue(itemid, out node);
            return node;
        }

        internal HierarchyItem GetManagedItem(uint itemid)
        {
            ContentReferenceNode node;
            if (itemid == 0xfffffffe)
            {
                return this.projectNode;
            }
            this.Refresh();
            HierarchyItem lastDeletedItem = null;
            if ((this.referencesFolderNode != null) && (itemid == this.referencesFolderNode.Itemid))
            {
                return this.referencesFolderNode;
            }
            if ((this.contentReferencesNode != null) && (itemid == this.contentReferencesNode.GetItemId()))
            {
                return this.contentReferencesNode;
            }
            if (this.contentReferenceItems.TryGetValue(itemid, out node))
            {
                return node;
            }
            if ((this.lastDeletedItem != null) && (itemid == this.lastDeletedItem.Itemid))
            {
                lastDeletedItem = this.lastDeletedItem;
            }
            return lastDeletedItem;
        }

        internal uint GetNextNonContentReferenceItem(uint itemid, int propid)
        {
            this.Refresh();
            uint num = itemid;
            do
            {
                object obj2;
                ErrorHandler.ThrowOnFailure(this.codeProject.InnerIVsHierarchy.GetProperty(num, propid, out obj2));
                num = (uint) ((int) obj2);
            }
            while (this.IsManagedItem(num));
            return num;
        }

        private bool IsContentReferenceItem(uint itemid)
        {
            string str;
            if (!this.IsReferenceItem(itemid))
            {
                return false;
            }
            return (ErrorHandler.Succeeded(this.codeProject.InnerVsBuildPropertyStorage.GetItemAttribute(itemid, "XnaReferenceType", out str)) && (str == "Content"));
        }

        internal bool IsContentReferencesFolderNodeVisible()
        {
            this.Refresh();
            return this.visible;
        }

        internal bool IsManagedItem(uint itemid)
        {
            return (this.GetManagedItem(itemid) != null);
        }

        private bool IsReferenceItem(uint itemid)
        {
            object obj2;
            if (ErrorHandler.Succeeded(this.hierarchy.GetProperty(itemid, -2027, out obj2)))
            {
                Reference reference = obj2 as Reference;
                return (reference != null);
            }
            return false;
        }

        internal void OnHierarchyChanged()
        {
            this.needsRefresh = true;
            this.itemBeforeContentReferences = uint.MaxValue;
            this.firstContentReferenceItem = uint.MaxValue;
            this.contentReferenceItems.Clear();
        }

        internal void OnItemDeleted(uint itemid)
        {
            this.lastDeletedItem = this.GetManagedItem(itemid) as ContentReferenceNode;
            this.OnHierarchyChanged();
        }

        private void Refresh()
        {
            if (this.needsRefresh)
            {
                uint referencesFolderNode = this.codeProject.LanguageAdapter.GetReferencesFolderNode();
                if (((this.referencesFolderNode == null) || (this.referencesFolderNode.Itemid == uint.MaxValue)) || ((referencesFolderNode != uint.MaxValue) && (referencesFolderNode != this.lkgReferencesFolderId)))
                {
                    this.lkgReferencesFolderId = referencesFolderNode;
                    this.referencesFolderNode = new Microsoft.Xna.GameStudio.CodeProject.ReferencesFolderNode(this.lkgReferencesFolderId, this.codeProject);
                }
                this.RefreshContentReferenceNodes(this.contentReferencesNode.GetItemId(), this.lkgReferencesFolderId);
                IEnumerable<uint> virtualFolders = this.codeProject.LanguageAdapter.GetVirtualFolders();
                this.itemBeforeContentReferences = (virtualFolders.Count<uint>() == 0) ? uint.MaxValue : virtualFolders.Last<uint>();
                this.needsRefresh = false;
            }
        }

        private void RefreshContentReferenceNodes(uint contentReferencesFolderId, uint referencesFolderId)
        {
            if ((contentReferencesFolderId != uint.MaxValue) && (referencesFolderId != uint.MaxValue))
            {
                object obj2;
                uint maxValue = uint.MaxValue;
                ContentReferenceNode node = null;
                uint itemid = uint.MaxValue;
                if (!ErrorHandler.Failed(this.hierarchy.GetProperty(referencesFolderId, -1001, out obj2)))
                {
                    for (itemid = (uint) ((int) obj2); itemid != uint.MaxValue; itemid = (uint) ((int) obj2))
                    {
                        if (this.IsContentReferenceItem(itemid))
                        {
                            if (this.firstContentReferenceItem == uint.MaxValue)
                            {
                                this.firstContentReferenceItem = itemid;
                            }
                            uint prevContentReferenceItem = uint.MaxValue;
                            if (node != null)
                            {
                                node.NextContentReferenceItem = itemid;
                                prevContentReferenceItem = node.Itemid;
                            }
                            node = new ContentReferenceNode(itemid, contentReferencesFolderId, this.codeProject, maxValue, prevContentReferenceItem);
                            this.contentReferenceItems.Add(itemid, node);
                            this.visible = true;
                        }
                        else
                        {
                            maxValue = itemid;
                        }
                        if (ErrorHandler.Failed(this.hierarchy.GetProperty(itemid, -1002, out obj2)))
                        {
                            return;
                        }
                    }
                }
            }
        }

        internal Microsoft.Xna.GameStudio.ContentReferencesFolderNode ContentReferencesFolderNode
        {
            get
            {
                return this.contentReferencesNode;
            }
        }

        internal uint FirstContentReferenceItem
        {
            get
            {
                this.Refresh();
                return this.firstContentReferenceItem;
            }
        }

        internal bool IsNestedProjectFirstChildOfRoot
        {
            get
            {
                this.Refresh();
                return (this.itemBeforeContentReferences == uint.MaxValue);
            }
        }

        internal uint ItemBeforeContentReferencesFolder
        {
            get
            {
                this.Refresh();
                return this.itemBeforeContentReferences;
            }
        }

        internal XnaProjectNode ProjectNode
        {
            get
            {
                return this.projectNode;
            }
        }

        internal Microsoft.Xna.GameStudio.CodeProject.ReferencesFolderNode ReferencesFolderNode
        {
            get
            {
                return this.referencesFolderNode;
            }
        }
    }
}

