﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using EnvDTE;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal sealed class MenuCommandHandlers : IServiceProvider
    {
        private List<OleMenuCommand> addedCommands;
        private AddReferenceCommand addReferenceCommand;
        private bool isEnabled;
        private OleMenuCommandService menuService;
        private List<uint> rootItemCmdIDs;
        private OleMenuCommand xnaPackageCommand;
        private OleMenuCommand xnaPackageCtxCommand;
        private IXnaCodeProject xnaProject;

        internal MenuCommandHandlers(OleMenuCommandService menuService, IXnaCodeProject xnaProject)
        {
            if (menuService == null)
            {
                throw new ArgumentNullException("menuService");
            }
            if (xnaProject == null)
            {
                throw new ArgumentNullException("xnaProject");
            }
            this.menuService = menuService;
            this.xnaProject = xnaProject;
            this.xnaPackageCommand = new OleMenuCommand(new EventHandler(this.OnXnaPackageCommand), CmdIDConstants.XnaPackageCmdID);
            this.xnaPackageCommand.BeforeQueryStatus += new EventHandler(this.OnXnaPackageCommandBeforeQueryStatus);
            this.xnaPackageCtxCommand = new OleMenuCommand(new EventHandler(this.OnXnaPackageCommand), CmdIDConstants.XnaPackageCtxCmdID);
            this.xnaPackageCtxCommand.BeforeQueryStatus += new EventHandler(this.OnXnaPackageCommandBeforeQueryStatus);
            this.addedCommands = new List<OleMenuCommand>();
            this.rootItemCmdIDs = new List<uint>(9);
            if (this.xnaProject.Adapter.SupportsPackaging)
            {
                this.AddMenuCommand(this.xnaPackageCommand, true);
                this.AddMenuCommand(this.xnaPackageCtxCommand, true);
            }
            if (this.xnaProject.Adapter.SupportsCrossPlatform)
            {
                string xnaPlatform = this.xnaProject.Adapter.XnaPlatform;
                if (xnaPlatform != "Windows")
                {
                    this.AddMenuCommand(new CloneProjectCommand(this.xnaProject, CmdIDConstants.XnaCloneForWindowsCmdID, false), true);
                    this.AddMenuCommand(new CloneProjectCommand(this.xnaProject, CmdIDConstants.XnaCloneForWindowsCtxCmdID, true), true);
                }
                if (xnaPlatform != "Xbox 360")
                {
                    this.AddMenuCommand(new CloneProjectCommand(this.xnaProject, CmdIDConstants.XnaCloneForXbox360CmdID, false), true);
                    this.AddMenuCommand(new CloneProjectCommand(this.xnaProject, CmdIDConstants.XnaCloneForXbox360CtxCmdID, true), true);
                }
                if (xnaPlatform != "Windows Phone")
                {
                    this.AddMenuCommand(new CloneProjectCommand(this.xnaProject, CmdIDConstants.XnaCloneForWindowsPhoneCmdID, false), true);
                    this.AddMenuCommand(new CloneProjectCommand(this.xnaProject, CmdIDConstants.XnaCloneForWindowsPhoneCtxCmdID, true), true);
                }
            }
            this.addedCommands.Add(new AddContentReferenceCommand(this.xnaProject));
            if (this.xnaProject.Adapter.IsDevicePlatform)
            {
                this.addReferenceCommand = new AddReferenceCommand(this.xnaProject);
                this.addedCommands.Add(this.addReferenceCommand);
                this.addedCommands.Add(new AddReferenceContextCommand(this.xnaProject));
            }
        }

        private void AddMenuCommand(OleMenuCommand command, bool rootItemSpecific)
        {
            if (command == null)
            {
                throw new ArgumentNullException("command");
            }
            this.addedCommands.Add(command);
            if (rootItemSpecific)
            {
                this.rootItemCmdIDs.Add((uint) command.CommandID.ID);
            }
        }

        internal void Enable(bool enable)
        {
            if (this.isEnabled != enable)
            {
                if (enable)
                {
                    foreach (OleMenuCommand command in this.addedCommands)
                    {
                        this.menuService.AddCommand(command);
                    }
                }
                else
                {
                    foreach (OleMenuCommand command2 in this.addedCommands)
                    {
                        this.menuService.RemoveCommand(command2);
                    }
                }
                this.isEnabled = enable;
            }
        }

        public object GetService(Type serviceType)
        {
            return this.xnaProject.GetVsService(serviceType);
        }

        internal bool IsMyHierarchyItem(uint itemid, out IVsUIHierarchy actualOwner, out IVsMultiItemSelect multiSelect)
        {
            bool flag;
            actualOwner = null;
            multiSelect = null;
            if (itemid != 0xfffffffd)
            {
                return true;
            }
            IVsMonitorSelection requiredService = Microsoft.Xna.GameStudio.ProjectUtilities.GetRequiredService<IVsMonitorSelection>((IServiceProvider) this.xnaProject);
            IntPtr zero = IntPtr.Zero;
            uint maxValue = uint.MaxValue;
            IntPtr ppSC = IntPtr.Zero;
            try
            {
                ErrorHandler.ThrowOnFailure(requiredService.GetCurrentSelection(out zero, out maxValue, out multiSelect, out ppSC));
                if (zero == IntPtr.Zero)
                {
                    return false;
                }
                if (ComUtilities.IsSameComObject(zero, this.xnaProject.GetVsHierarchy()))
                {
                    return true;
                }
                actualOwner = (IVsUIHierarchy) Marshal.GetObjectForIUnknown(zero);
                flag = false;
            }
            finally
            {
                ComUtilities.SmartRelease(ref zero);
                ComUtilities.SmartRelease(ref ppSC);
            }
            return flag;
        }

        internal void OnAddReference()
        {
            if (this.addReferenceCommand != null)
            {
                this.addReferenceCommand.Invoke();
            }
        }

        private void OnXnaPackageCommand(object sender, EventArgs e)
        {
            object obj2;
            this.xnaProject.DoPackaging(true);
            ErrorHandler.ThrowOnFailure(this.xnaProject.GetVsHierarchy().GetProperty(0xfffffffe, -2027, out obj2));
            Project project = obj2 as Project;
            if (project != null)
            {
                SolutionBuild solutionBuild = project.DTE.Solution.SolutionBuild;
                solutionBuild.BuildProject(solutionBuild.ActiveConfiguration.Name, project.UniqueName, false);
            }
        }

        private void OnXnaPackageCommandBeforeQueryStatus(object sender, EventArgs e)
        {
            OleMenuCommand command = sender as OleMenuCommand;
            if (command != null)
            {
                bool isEntryProject = this.xnaProject.IsEntryProject;
                command.Supported = true;
                command.Visible = isEntryProject;
                command.Enabled = isEntryProject && !this.xnaProject.IsBuilding;
                if (command == this.xnaPackageCommand)
                {
                    object obj2;
                    ErrorHandler.ThrowOnFailure(this.xnaProject.GetVsHierarchy().GetProperty(0xfffffffe, -2027, out obj2));
                    Project project = obj2 as Project;
                    if (project != null)
                    {
                        command.Text = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XNAPackageMenuText, new object[] { project.Name });
                    }
                }
            }
        }

        internal bool ShouldFilterCommand(uint itemid, ref Guid guidCmdGroup, uint commandID)
        {
            if ((itemid != 0xfffffffe) && (guidCmdGroup == CmdIDConstants.GuidXnaPackageCmdSet))
            {
                foreach (uint num in this.rootItemCmdIDs)
                {
                    if (num == commandID)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }
}

