﻿namespace Microsoft.Xna.GameStudio.CodeProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Globalization;
    using System.IO;
    using System.Windows.Forms;

    internal class TilePicker : UserControl, IServiceProvider
    {
        private Button addTileImageButton;
        private IContainer components;
        private bool isDirty;
        private bool isUpdating;
        private Label labelTileTitle;
        private Dictionary<string, string> mapDisplayToMoniker;
        private ComboBox pickerComboTileImage;
        private Label pickerLabel;
        private IXnaCodeProject project;
        private TableLayoutPanel tableLayoutPanel1;
        private TextBox textBoxTileTitle;

        internal event EventHandler PropertyChanged;

        internal TilePicker(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            this.InitializeComponent();
            this.project = project;
            this.RefreshList();
            this.pickerComboTileImage.SelectedIndexChanged += new EventHandler(this.OnSelectedIndexChanged);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private bool DoesAddFileToProjectOverwriteAnExistingFile(string filePath)
        {
            if (filePath == null)
            {
                throw new ArgumentNullException("filePath");
            }
            string fileName = Path.GetFileName(filePath);
            if (this.mapDisplayToMoniker.ContainsKey(fileName))
            {
                return true;
            }
            object pvar = null;
            ErrorHandler.ThrowOnFailure(this.project.GetVsHierarchy().GetProperty(0xfffffffe, -2021, out pvar));
            string fullPath = Path.GetFullPath(Path.Combine((string) pvar, fileName));
            return (File.Exists(fullPath) && (string.Compare(fullPath, filePath, StringComparison.OrdinalIgnoreCase) != 0));
        }

        private string GetDisplayName(string filepath)
        {
            string str = ProjectAssemblyPath.MakeProjectRelativePath(filepath, this.project.GetVsHierarchy());
            if (str.StartsWith("..", StringComparison.OrdinalIgnoreCase))
            {
                return filepath;
            }
            return str;
        }

        private void InitializeComponent()
        {
            ComponentResourceManager manager = new ComponentResourceManager(typeof(TilePicker));
            this.tableLayoutPanel1 = new TableLayoutPanel();
            this.labelTileTitle = new Label();
            this.pickerComboTileImage = new ComboBox();
            this.addTileImageButton = new Button();
            this.pickerLabel = new Label();
            this.textBoxTileTitle = new TextBox();
            this.tableLayoutPanel1.SuspendLayout();
            base.SuspendLayout();
            manager.ApplyResources(this.tableLayoutPanel1, "tableLayoutPanel1");
            this.tableLayoutPanel1.Controls.Add(this.labelTileTitle, 0, 0);
            this.tableLayoutPanel1.Controls.Add(this.textBoxTileTitle, 0, 1);
            this.tableLayoutPanel1.Controls.Add(this.pickerLabel, 0, 2);
            this.tableLayoutPanel1.Controls.Add(this.pickerComboTileImage, 0, 3);
            this.tableLayoutPanel1.Controls.Add(this.addTileImageButton, 1, 3);
            this.tableLayoutPanel1.Name = "tableLayoutPanel1";
            manager.ApplyResources(this.labelTileTitle, "labelTileTitle");
            this.labelTileTitle.Name = "labelTileTitle";
            manager.ApplyResources(this.pickerComboTileImage, "pickerComboTileImage");
            this.pickerComboTileImage.AutoCompleteMode = AutoCompleteMode.Append;
            this.pickerComboTileImage.AutoCompleteSource = AutoCompleteSource.ListItems;
            this.pickerComboTileImage.DropDownStyle = ComboBoxStyle.DropDownList;
            this.pickerComboTileImage.FormattingEnabled = true;
            this.pickerComboTileImage.Name = "pickerComboTileImage";
            manager.ApplyResources(this.addTileImageButton, "addTileImageButton");
            this.addTileImageButton.Name = "addTileImageButton";
            this.addTileImageButton.UseVisualStyleBackColor = true;
            this.addTileImageButton.Click += new EventHandler(this.OnAddTileImageButtonClick);
            manager.ApplyResources(this.pickerLabel, "pickerLabel");
            this.pickerLabel.Name = "pickerLabel";
            manager.ApplyResources(this.textBoxTileTitle, "textBoxTileTitle");
            this.textBoxTileTitle.Name = "textBoxTileTitle";
            this.textBoxTileTitle.TextChanged += new EventHandler(this.OnTextBoxTileTitleTextChanged);
            manager.ApplyResources(this, "$this");
            base.AutoScaleMode = AutoScaleMode.Font;
            base.Controls.Add(this.tableLayoutPanel1);
            base.Name = "TilePicker";
            this.tableLayoutPanel1.ResumeLayout(false);
            this.tableLayoutPanel1.PerformLayout();
            base.ResumeLayout(false);
            base.PerformLayout();
        }

        private bool IsDocumentInProject(string filePath)
        {
            if (filePath == null)
            {
                throw new ArgumentNullException("filePath");
            }
            int pfFound = 0;
            uint maxValue = uint.MaxValue;
            VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
            ErrorHandler.ThrowOnFailure(this.project.InnerVsProject.IsDocumentInProject(filePath, out pfFound, pdwPriority, out maxValue));
            return (pfFound != 0);
        }

        private void OnAddTileImageButtonClick(object sender, EventArgs e)
        {
            IVsUIShell vsService = this.project.GetVsService(typeof(SVsUIShell)) as IVsUIShell;
            IVsAddProjectItemDlg2 dlg = this.project.GetVsService(typeof(SVsAddProjectItemDlg)) as IVsAddProjectItemDlg2;
            if ((vsService != null) && (dlg != null))
            {
                string selectedFile = this.SelectedFile;
                string filePathOut = null;
                if (VsBrowseDialog.ShowOpenFileDialog(this, base.Handle, selectedFile, XnaProjectResource.TileImageOpenFileDialogFilter, 0, XnaProjectResource.TileImageOpenFileDialogTitle, 0, out filePathOut) == DialogResult.OK)
                {
                    if (this.IsDocumentInProject(filePathOut))
                    {
                        this.pickerComboTileImage.Text = this.GetDisplayName(filePathOut);
                    }
                    else if (!this.DoesAddFileToProjectOverwriteAnExistingFile(filePathOut))
                    {
                        Guid empty = Guid.Empty;
                        Guid rguidEditorType = Guid.Empty;
                        Guid rguidLogicalView = Guid.Empty;
                        int hr = dlg.AddExistingProjectItems(0xfffffffe, ref empty, this.project.InnerVsProject, 0, ref rguidEditorType, null, ref rguidLogicalView, 1, new string[] { filePathOut });
                        if (ErrorHandler.Succeeded(hr))
                        {
                            this.pickerComboTileImage.Text = Path.GetFileName(filePathOut);
                        }
                        else if (hr != -2147221492)
                        {
                            vsService.ReportErrorInfo(hr);
                        }
                    }
                    else
                    {
                        string text = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.TilePickerCannotAddTileImage, new object[] { Path.GetFileName(filePathOut) }).Replace('|', '\n');
                        MessageBoxOptions options = 0;
                        Control parent = this;
                        while (parent.RightToLeft == RightToLeft.Inherit)
                        {
                            parent = parent.Parent;
                            if (parent == null)
                            {
                                break;
                            }
                        }
                        if ((parent != null) && (parent.RightToLeft == RightToLeft.Yes))
                        {
                            options = MessageBoxOptions.RtlReading | MessageBoxOptions.RightAlign;
                        }
                        MessageBox.Show(null, text, XnaProjectResource.ProjectDesignerTitle, MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1, options);
                    }
                }
            }
        }

        protected override void OnFontChanged(EventArgs e)
        {
            base.OnFontChanged(e);
            if (this.pickerComboTileImage != null)
            {
                ComboBoxUtils.SetComboBoxDropdownWidth(this.pickerComboTileImage);
            }
        }

        private void OnSelectedIndexChanged(object sender, EventArgs args)
        {
            if (!this.isUpdating)
            {
                this.Dirty = true;
            }
        }

        private void OnTextBoxTileTitleTextChanged(object sender, EventArgs e)
        {
            this.Dirty = true;
        }

        internal void RefreshList()
        {
            bool isUpdating = this.isUpdating;
            this.isUpdating = true;
            try
            {
                string text = this.pickerComboTileImage.Text;
                this.pickerComboTileImage.Items.Clear();
                this.pickerComboTileImage.Items.Add(XnaProjectResource.DefaultTileImageItem);
                IVsHierarchy vsHierarchy = this.project.GetVsHierarchy();
                this.mapDisplayToMoniker = new Dictionary<string, string>();
                foreach (uint num in this.project.GetVisibleItems())
                {
                    Guid empty = Guid.Empty;
                    ErrorHandler.ThrowOnFailure(vsHierarchy.GetGuidProperty(num, -1004, out empty));
                    if (empty == VSConstants.GUID_ItemType_PhysicalFile)
                    {
                        string str2;
                        this.project.InnerVsProject.GetMkDocument(num, out str2);
                        if ((str2 != null) && (string.Compare(Path.GetExtension(str2), ".png", StringComparison.OrdinalIgnoreCase) == 0))
                        {
                            uint num2;
                            int num3;
                            VSDOCUMENTPRIORITY[] pdwPriority = new VSDOCUMENTPRIORITY[1];
                            ErrorHandler.ThrowOnFailure(this.project.InnerVsProject.IsDocumentInProject(str2, out num3, pdwPriority, out num2));
                            if (num3 != 0)
                            {
                                string displayName = this.GetDisplayName(str2);
                                this.mapDisplayToMoniker[displayName] = str2;
                                this.pickerComboTileImage.Items.Add(displayName);
                            }
                        }
                    }
                }
                if (string.IsNullOrWhiteSpace(text))
                {
                    text = XnaProjectResource.DefaultTileImageItem;
                }
                this.pickerComboTileImage.Text = text;
                ComboBoxUtils.SetComboBoxDropdownWidth(this.pickerComboTileImage);
            }
            finally
            {
                this.isUpdating = isUpdating;
            }
        }

        object IServiceProvider.GetService(System.Type serviceType)
        {
            return this.project.GetVsService(serviceType);
        }

        internal bool Dirty
        {
            get
            {
                return this.isDirty;
            }
            set
            {
                this.isDirty = value;
                if (value)
                {
                    this.propertyChangedEvent(this, null);
                }
            }
        }

        internal string SelectedFile
        {
            get
            {
                string str2;
                string text = this.Text;
                if (this.mapDisplayToMoniker.TryGetValue(text, out str2))
                {
                    return str2;
                }
                return text;
            }
        }

        public override string Text
        {
            get
            {
                string text = this.pickerComboTileImage.Text;
                if (text == XnaProjectResource.DefaultTileImageItem)
                {
                    text = string.Empty;
                }
                return text;
            }
            set
            {
                bool isUpdating = this.isUpdating;
                this.isUpdating = true;
                try
                {
                    if (string.IsNullOrEmpty(value))
                    {
                        value = XnaProjectResource.DefaultTileImageItem;
                    }
                    this.pickerComboTileImage.Text = value;
                }
                finally
                {
                    this.isUpdating = isUpdating;
                }
            }
        }

        public string Title
        {
            get
            {
                return this.textBoxTileTitle.Text;
            }
            set
            {
                bool isUpdating = this.isUpdating;
                this.isUpdating = true;
                try
                {
                    if (!string.IsNullOrEmpty(value))
                    {
                        this.textBoxTileTitle.Text = value;
                        this.isDirty = true;
                    }
                }
                finally
                {
                    this.isUpdating = isUpdating;
                }
            }
        }
    }
}

