﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using Microsoft.Xna.PlatformTools.ContentPipeline.Views;
    using System;
    using System.AddIn.Hosting;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.IO;
    using System.Reflection;
    using VSLangProj;

    internal class ContentPipelineManager
    {
        private bool disableAssemblyLoad;
        private const string HostingDir = "PipelineHosting";
        private bool needsRefresh;
        private IComponentScanner pipelineScanner;
        private string[] referencedAssemblies;
        private const string xactExtension = ".xap";
        private const string xactImporterName = "XactImporter";
        private const string xactProcessorName = "XactProcessor";

        public ContentPipelineManager(References references, bool disableAssemblyLoad, string xnaFrameworkVersion)
        {
            if (references == null)
            {
                throw new ArgumentNullException("referenceContainer");
            }
            this.disableAssemblyLoad = disableAssemblyLoad;
            this.referencedAssemblies = new string[0];
            this.OnReferencesChanged(references);
            this.pipelineScanner = GetPipelineScanner(xnaFrameworkVersion);
        }

        internal void GetDefaultImporterAndProcessor(string extension, ref string importer, ref string processor)
        {
            if (this.pipelineScanner == null)
            {
                importer = null;
                processor = null;
            }
            else if (extension.Equals(".xap", StringComparison.InvariantCultureIgnoreCase))
            {
                importer = "XactImporter";
                processor = "XactProcessor";
            }
            else
            {
                IImporterDescriptor importerForExtension = this.GetImporterForExtension(extension);
                if (importerForExtension != null)
                {
                    importer = importerForExtension.Name;
                    processor = this.GetDefaultProcessor(importerForExtension);
                }
            }
        }

        private string GetDefaultProcessor(IImporterDescriptor importer)
        {
            if (importer == null)
            {
                throw new ArgumentNullException("importer");
            }
            if (this.pipelineScanner != null)
            {
                string defaultProcessorName = importer.DefaultProcessorName;
                if (!string.IsNullOrEmpty(defaultProcessorName))
                {
                    foreach (IProcessorDescriptor descriptor in this.pipelineScanner.Processors)
                    {
                        if (defaultProcessorName == descriptor.Name)
                        {
                            return defaultProcessorName;
                        }
                    }
                }
                foreach (IProcessorDescriptor descriptor2 in this.pipelineScanner.Processors)
                {
                    if (importer.OutputType == descriptor2.InputType)
                    {
                        return descriptor2.Name;
                    }
                }
            }
            return null;
        }

        internal string GetImporterDisplayName(string importerName)
        {
            if ((importerName != null) && (this.pipelineScanner != null))
            {
                if (importerName.Equals("XactImporter", StringComparison.InvariantCultureIgnoreCase))
                {
                    return XnaProjectResource.XactImporterDisplayName;
                }
                this.UpdatePipelineComponentScanner();
                foreach (IImporterDescriptor descriptor in this.pipelineScanner.Importers)
                {
                    if (importerName == descriptor.Name)
                    {
                        return descriptor.DisplayName;
                    }
                }
            }
            return importerName;
        }

        private IImporterDescriptor GetImporterForExtension(string extension)
        {
            if (extension == null)
            {
                throw new ArgumentNullException("extension");
            }
            if (this.pipelineScanner != null)
            {
                this.UpdatePipelineComponentScanner();
                if (this.pipelineScanner != null)
                {
                    foreach (IImporterDescriptor descriptor in this.pipelineScanner.Importers)
                    {
                        foreach (string str in descriptor.FileExtensions)
                        {
                            if (extension.Equals(str, StringComparison.OrdinalIgnoreCase))
                            {
                                return descriptor;
                            }
                        }
                    }
                }
            }
            return null;
        }

        internal string GetImporterName(string importerDisplayName)
        {
            if ((importerDisplayName != null) && (this.pipelineScanner != null))
            {
                if (importerDisplayName.Equals(XnaProjectResource.XactImporterDisplayName, StringComparison.InvariantCultureIgnoreCase))
                {
                    return "XactImporter";
                }
                this.UpdatePipelineComponentScanner();
                foreach (IImporterDescriptor descriptor in this.pipelineScanner.Importers)
                {
                    if (importerDisplayName == descriptor.DisplayName)
                    {
                        return descriptor.Name;
                    }
                }
            }
            return importerDisplayName;
        }

        internal ReadOnlyCollection<string> GetImporterNames()
        {
            if (this.pipelineScanner == null)
            {
                return new ReadOnlyCollection<string>(new List<string>(0));
            }
            this.UpdatePipelineComponentScanner();
            IList<IImporterDescriptor> importers = this.pipelineScanner.Importers;
            List<string> list = new List<string>(importers.Count);
            foreach (IImporterDescriptor descriptor in importers)
            {
                list.Add(descriptor.Name);
            }
            list.Add("XactImporter");
            list.Sort();
            return new ReadOnlyCollection<string>(list);
        }

        private static string GetPipelineAddInRootPath()
        {
            string str = null;
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\XNA\Game Studio\");
            if (key != null)
            {
                string str2 = key.GetValue("SharedComponentsPath", string.Empty) as string;
                str = Path.Combine(str2, "PipelineHosting");
            }
            if (str == "PipelineHosting")
            {
                str = null;
            }
            return str;
        }

        private static IComponentScanner GetPipelineScanner(string xnaFrameworkVersion)
        {
            if (!string.IsNullOrEmpty(xnaFrameworkVersion))
            {
                string pipelineAddInRootPath = GetPipelineAddInRootPath();
                if (!string.IsNullOrEmpty(pipelineAddInRootPath))
                {
                    try
                    {
                        foreach (AddInToken token in AddInStore.FindAddIns(typeof(IComponentScanner), pipelineAddInRootPath, new string[0]))
                        {
                            string str2;
                            if ((token.QualificationData[AddInSegmentType.AddIn].TryGetValue("XnaFrameworkVersion", out str2) && (str2 != null)) && (str2 == xnaFrameworkVersion))
                            {
                                try
                                {
                                    return token.Activate<IComponentScanner>(AppDomain.CurrentDomain);
                                }
                                catch (TargetInvocationException exception)
                                {
                                    string message = string.Empty;
                                    if (exception.InnerException != null)
                                    {
                                        message = exception.InnerException.Message;
                                    }
                                    GameStudioWindowPane.WriteLine(XnaProjectResource.PipelineAddInFailedInstantiation, new object[] { xnaFrameworkVersion, message });
                                }
                            }
                        }
                        GameStudioWindowPane.WriteLine(XnaProjectResource.PipelineAddInInvalidVersion, new object[] { xnaFrameworkVersion });
                    }
                    catch (InvalidOperationException exception2)
                    {
                        GameStudioWindowPane.WriteLine(XnaProjectResource.PipelineAddInInvalidPipeline, new object[] { exception2.Message });
                    }
                    catch (DirectoryNotFoundException exception3)
                    {
                        GameStudioWindowPane.WriteLine(XnaProjectResource.PipelineAddInInvalidPipeline, new object[] { exception3.Message });
                    }
                    catch (InvalidPipelineStoreException exception4)
                    {
                        GameStudioWindowPane.WriteLine(XnaProjectResource.PipelineAddInInvalidPipeline, new object[] { exception4.Message });
                    }
                }
            }
            else
            {
                GameStudioWindowPane.WriteLine(XnaProjectResource.PipelineAddInMissingVersionProperty, new object[0]);
            }
            return null;
        }

        internal string GetProcessorDisplayName(string processorName)
        {
            if ((processorName != null) && (this.pipelineScanner != null))
            {
                if (processorName.Equals("XactProcessor", StringComparison.InvariantCultureIgnoreCase))
                {
                    return XnaProjectResource.XactProcessorDisplayName;
                }
                this.UpdatePipelineComponentScanner();
                foreach (IProcessorDescriptor descriptor in this.pipelineScanner.Processors)
                {
                    if (processorName == descriptor.Name)
                    {
                        return descriptor.DisplayName;
                    }
                }
            }
            return processorName;
        }

        internal string GetProcessorName(string processorDisplayName)
        {
            if ((processorDisplayName != null) && (this.pipelineScanner != null))
            {
                if (processorDisplayName.Equals(XnaProjectResource.XactProcessorDisplayName, StringComparison.InvariantCultureIgnoreCase))
                {
                    return "XactProcessor";
                }
                this.UpdatePipelineComponentScanner();
                foreach (IProcessorDescriptor descriptor in this.pipelineScanner.Processors)
                {
                    if (processorDisplayName == descriptor.DisplayName)
                    {
                        return descriptor.Name;
                    }
                }
            }
            return processorDisplayName;
        }

        internal ReadOnlyCollection<string> GetProcessorNames()
        {
            if (this.pipelineScanner == null)
            {
                return new ReadOnlyCollection<string>(new List<string>(0));
            }
            this.UpdatePipelineComponentScanner();
            IList<IProcessorDescriptor> processors = this.pipelineScanner.Processors;
            List<string> list = new List<string>(processors.Count);
            foreach (IProcessorDescriptor descriptor in processors)
            {
                list.Add(descriptor.Name);
            }
            list.Add("XactProcessor");
            list.Sort();
            return new ReadOnlyCollection<string>(list);
        }

        internal IList<IParameterDescriptor> GetProcessorParameters(string processorName)
        {
            if (processorName == null)
            {
                throw new ArgumentNullException("processorName");
            }
            if (this.pipelineScanner != null)
            {
                this.UpdatePipelineComponentScanner();
                foreach (IProcessorDescriptor descriptor in this.pipelineScanner.Processors)
                {
                    if (processorName == descriptor.Name)
                    {
                        return descriptor.Parameters;
                    }
                }
            }
            return null;
        }

        internal void OnReferencesChanged(References references)
        {
            this.needsRefresh = true;
            if (references == null)
            {
                this.referencedAssemblies = new string[0];
            }
            else
            {
                this.referencedAssemblies = new string[references.Count];
                int num = 0;
                try
                {
                    foreach (Reference reference in references)
                    {
                        this.referencedAssemblies[num++] = reference.Path;
                    }
                }
                catch
                {
                    this.referencedAssemblies = new string[0];
                    throw;
                }
            }
        }

        private void UpdatePipelineComponentScanner()
        {
            if (this.needsRefresh && (this.pipelineScanner != null))
            {
                if (this.disableAssemblyLoad)
                {
                    this.referencedAssemblies = new string[0];
                }
                this.pipelineScanner.Scan(this.referencedAssemblies, new string[0]);
                foreach (string str in this.pipelineScanner.Errors)
                {
                    GameStudioWindowPane.WriteLine(str, new object[0]);
                }
                this.needsRefresh = false;
            }
        }
    }
}

