﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal class ContentProjectNode : ProjectHierarchyItem
    {
        private ProjectBrowseObject browseObject;
        private static readonly Guid[] cfgPropertyPages = new Guid[] { typeof(ContentBuildPropertyPage).GUID };
        private IXnaContentProject contentProject;
        private int projectIconIndex;
        private static readonly Guid[] propertyPages = new Guid[] { Microsoft.Xna.GameStudio.ProjectConstants.CLSID_BuildEventsPropertyPage, Microsoft.Xna.GameStudio.ProjectConstants.CLSID_ReferencesPathPropertyPage };
        private IServiceProvider serviceProvider;
        private static readonly Guid[] sortedPropertyPages = new Guid[] { typeof(ContentBuildPropertyPage).GUID, Microsoft.Xna.GameStudio.ProjectConstants.CLSID_BuildEventsPropertyPage, Microsoft.Xna.GameStudio.ProjectConstants.CLSID_ReferencesPathPropertyPage };
        private IVsUserContext userContext;

        internal ContentProjectNode(IVsUIHierarchy innerHierarchy, IServiceProvider serviceProvider, IXnaContentProject contentProject) : base(innerHierarchy)
        {
            this.projectIconIndex = -1;
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (contentProject == null)
            {
                throw new ArgumentNullException("contentProject");
            }
            this.serviceProvider = serviceProvider;
            this.contentProject = contentProject;
            base.OverrideProperty(-2055, false);
            base.OverrideProperty(-999000001, true);
            base.OverrideProperty(-2083, false);
            base.OverrideProperty(-2004, new HierarchyItem.GetPropertyDelegate(this.GetIconImgList));
            base.OverrideProperty(-2005, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2015, new HierarchyItem.GetPropertyDelegate(this.GetIconIndex));
            base.OverrideProperty(-2003, new HierarchyItem.GetPropertyDelegate(this.GetCaption), new HierarchyItem.SetPropertyDelegate(this.SetCaption));
            base.OverrideProperty(-2065, new HierarchyItem.GetPropertyDelegate(this.GetPropertyPagesCLSIDList));
            base.OverrideProperty(-2066, new HierarchyItem.GetPropertyDelegate(this.GetCfgPropertyPagesCLSIDList));
            base.OverrideProperty(-2089, new HierarchyItem.GetPropertyDelegate(this.GetPriorityPropertyPagesCLSIDList));
            Guid gUID = typeof(ContentProjectFactory).GUID;
            base.OverrideGuidProperty(-2070, ref gUID);
            base.OverrideProperty(-2023, new HierarchyItem.GetPropertyDelegate(this.GetUserContext));
            base.OverrideProperty(-2018, new HierarchyItem.GetPropertyDelegate(this.GetBrowseObject));
        }

        private void EnsureImageListInitialized()
        {
            if (this.projectIconIndex < 0)
            {
                this.projectIconIndex = base.AddImageListIcon(XnaProjectResource.NestedContentProject);
            }
        }

        private int GetBrowseObject(uint itemid, int propid, out object property)
        {
            if (this.browseObject == null)
            {
                object obj2;
                ErrorHandler.ThrowOnFailure(base.InnerHierarchy.GetProperty(0xfffffffe, -2018, out obj2));
                IVsGetCfgProvider provider = obj2 as IVsGetCfgProvider;
                if (provider != null)
                {
                    IVsCfgProvider provider2;
                    ErrorHandler.ThrowOnFailure(provider.GetCfgProvider(out provider2));
                    IVsProjectCfgProvider innerProjectCfgProvider = (IVsProjectCfgProvider) provider2;
                    this.browseObject = new ProjectBrowseObject(innerProjectCfgProvider, obj2, this.contentProject.GetVsHierarchy());
                }
            }
            if (this.browseObject != null)
            {
                property = this.browseObject;
                return 0;
            }
            return base.InnerHierarchy.GetProperty(itemid, propid, out property);
        }

        private int GetCaption(uint itemid, int propid, out object property)
        {
            int hr = base.InnerHierarchy.GetProperty(itemid, propid, out property);
            if (ErrorHandler.Succeeded(hr))
            {
                string y = (string) property;
                IVsBuildPropertyStorage innerHierarchy = base.InnerHierarchy as IVsBuildPropertyStorage;
                if (innerHierarchy != null)
                {
                    string pbstrPropValue = null;
                    uint storage = 1;
                    if ((ErrorHandler.Succeeded(innerHierarchy.GetPropertyValue("ContentRootDirectory", null, storage, out pbstrPropValue)) && !string.IsNullOrEmpty(pbstrPropValue)) && !StringComparer.OrdinalIgnoreCase.Equals(pbstrPropValue, y))
                    {
                        property = string.Format(CultureInfo.InvariantCulture, "{0} ({1})", new object[] { y, pbstrPropValue });
                    }
                }
            }
            return hr;
        }

        private int GetCfgPropertyPagesCLSIDList(uint itemid, int propid, out object property)
        {
            property = Microsoft.Xna.GameStudio.ProjectUtilities.CreateSemicolonDelimitedListOfStringFromGuids(cfgPropertyPages);
            return 0;
        }

        private int GetIconImgList(uint itemid, int propid, out object property)
        {
            this.EnsureImageListInitialized();
            property = base.IconImageList.Handle;
            return 0;
        }

        private int GetIconIndex(uint itemid, int propid, out object property)
        {
            this.EnsureImageListInitialized();
            property = this.projectIconIndex;
            return 0;
        }

        private int GetPriorityPropertyPagesCLSIDList(uint itemid, int propid, out object property)
        {
            property = Microsoft.Xna.GameStudio.ProjectUtilities.CreateSemicolonDelimitedListOfStringFromGuids(sortedPropertyPages);
            return 0;
        }

        private int GetPropertyPagesCLSIDList(uint itemid, int propid, out object property)
        {
            property = Microsoft.Xna.GameStudio.ProjectUtilities.CreateSemicolonDelimitedListOfStringFromGuids(propertyPages);
            return 0;
        }

        private int GetUserContext(uint itemid, int propid, out object property)
        {
            property = null;
            if (this.userContext == null)
            {
                IVsMonitorUserContext service = this.serviceProvider.GetService(typeof(SVsMonitorUserContext)) as IVsMonitorUserContext;
                if (service == null)
                {
                    throw new InvalidOperationException(XnaProjectResource.NoMonitorUserContext);
                }
                ErrorHandler.ThrowOnFailure(service.CreateEmptyContext(out this.userContext));
                ErrorHandler.ThrowOnFailure(this.userContext.AddAttribute(VSUSERCONTEXTATTRIBUTEUSAGE.VSUC_Usage_Filter, "Product", "XNA"));
                ErrorHandler.ThrowOnFailure(this.userContext.AddAttribute(VSUSERCONTEXTATTRIBUTEUSAGE.VSUC_Usage_Filter, "ProjType", "XNA_40"));
                ErrorHandler.ThrowOnFailure(this.userContext.AddAttribute(VSUSERCONTEXTATTRIBUTEUSAGE.VSUC_Usage_Filter, "Project", "XNA Content Project"));
            }
            property = this.userContext;
            return 0;
        }

        private int SetCaption(uint itemid, int propid, object property)
        {
            return base.InnerHierarchy.SetProperty(itemid, propid, property);
        }

        internal void SetProjectID(ref Guid projectID)
        {
            base.OverrideGuidProperty(-2059, ref projectID);
        }
    }
}

