﻿namespace Microsoft.Xna.GameStudio.ContentProject
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using Microsoft.Xna.PlatformTools.ContentPipeline.Views;
    using System;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;

    internal sealed class ParameterPropertyDescriptor : PropertyDescriptor
    {
        private StandardValuesExclusiveConverter customEnumConverter;
        private uint itemid;
        private IVsBuildPropertyStorage msbuild;
        private string parameterName;
        private IParameterDescriptor processorParameter;
        private ITypeResolutionService typeResolutionService;

        private ParameterPropertyDescriptor(string name, DescriptionAttribute description, IParameterDescriptor processorParameter, IVsBuildPropertyStorage buildPropertyStorage, uint itemid, ITypeResolutionService typeResolutionService) : base(name, new Attribute[] { description, new HelpKeywordAttribute("Parameterized Processors") })
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (description == null)
            {
                throw new ArgumentNullException("description");
            }
            if (processorParameter == null)
            {
                throw new ArgumentNullException("processorParameter");
            }
            if (buildPropertyStorage == null)
            {
                throw new ArgumentNullException("buildPropertyStorage");
            }
            if (typeResolutionService == null)
            {
                throw new ArgumentNullException("typeResolutionService");
            }
            this.processorParameter = processorParameter;
            this.msbuild = buildPropertyStorage;
            this.itemid = itemid;
            this.typeResolutionService = typeResolutionService;
            this.parameterName = "ProcessorParameters_" + processorParameter.PropertyName;
        }

        public override bool CanResetValue(object component)
        {
            return ((this.processorParameter.DefaultValue != null) && !this.processorParameter.DefaultValue.Equals(this.GetValue(component)));
        }

        internal static ParameterPropertyDescriptor Create(IParameterDescriptor processorParameter, IVsBrowseObject browseObject)
        {
            if (processorParameter == null)
            {
                throw new ArgumentNullException("processorParameter");
            }
            if (browseObject == null)
            {
                throw new ArgumentNullException("browseObject");
            }
            uint maxValue = uint.MaxValue;
            IVsHierarchy pHier = null;
            ErrorHandler.ThrowOnFailure(browseObject.GetProjectItem(out pHier, out maxValue));
            IVsBuildPropertyStorage buildPropertyStorage = new VsBuildPropertyStorageSccWrapper((IVsBuildPropertyStorage) pHier, (IVsProject) pHier);
            string displayName = processorParameter.DisplayName;
            string description = processorParameter.Description;
            if (string.IsNullOrEmpty(description))
            {
                description = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.ParameterDefaultDescription, new object[] { processorParameter.PropertyName });
            }
            IXnaContentProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaContentProject>(pHier as IHierarchyObjectProvider);
            if (hierarchyObject == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(IXnaContentProject).FullName }));
            }
            return new ParameterPropertyDescriptor(displayName, new DescriptionAttribute(description), processorParameter, buildPropertyStorage, maxValue, Microsoft.Xna.GameStudio.ProjectUtilities.GetRequiredService<DynamicTypeService>(hierarchyObject.GetServiceProvider()).GetTypeResolutionService(pHier, maxValue));
        }

        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public override object GetValue(object component)
        {
            string pbstrAttributeValue = null;
            int hr = this.msbuild.GetItemAttribute(this.itemid, this.parameterName, out pbstrAttributeValue);
            ErrorHandler.ThrowOnFailure(hr, new int[] { -2147170504 });
            if (hr == -2147170504)
            {
                return this.processorParameter.DefaultValue;
            }
            object obj2 = null;
            if (this.Converter.CanConvertFrom(null, typeof(string)))
            {
                try
                {
                    obj2 = this.Converter.ConvertFrom(null, CultureInfo.InvariantCulture, pbstrAttributeValue);
                }
                catch (Exception)
                {
                }
            }
            return obj2;
        }

        internal void RemoveMetadata()
        {
            ErrorHandler.ThrowOnFailure(this.msbuild.SetItemAttribute(this.itemid, this.parameterName, string.Empty));
        }

        public override void ResetValue(object component)
        {
            if (!this.CanResetValue(component))
            {
                throw new InvalidOperationException(XnaProjectResource.PropertyDescriptorResetNotSupported);
            }
            this.SetValue(component, this.processorParameter.DefaultValue);
        }

        public override void SetValue(object component, object value)
        {
            object obj2 = this.GetValue(component);
            if ((obj2 != value) && ((obj2 == null) || !obj2.Equals(value)))
            {
                string pszAttributeValue = this.Converter.ConvertTo(null, CultureInfo.InvariantCulture, value, typeof(string)) as string;
                if ((this.processorParameter.DefaultValue != null) && this.processorParameter.DefaultValue.Equals(value))
                {
                    pszAttributeValue = string.Empty;
                }
                ErrorHandler.ThrowOnFailure(this.msbuild.SetItemAttribute(this.itemid, this.parameterName, pszAttributeValue));
            }
        }

        public override bool ShouldSerializeValue(object component)
        {
            if (!this.CanResetValue(component))
            {
                return (this.processorParameter.DefaultValue == null);
            }
            return true;
        }

        public override Type ComponentType
        {
            get
            {
                return typeof(string);
            }
        }

        public override TypeConverter Converter
        {
            get
            {
                if (!this.processorParameter.IsEnum)
                {
                    return base.Converter;
                }
                if (this.customEnumConverter == null)
                {
                    this.customEnumConverter = new StandardValuesExclusiveConverter(new ReadOnlyCollection<string>(this.processorParameter.PossibleEnumValues));
                }
                return this.customEnumConverter;
            }
        }

        public override bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        public override Type PropertyType
        {
            get
            {
                if (this.processorParameter.IsEnum)
                {
                    return typeof(string);
                }
                return this.typeResolutionService.GetType(this.processorParameter.PropertyType, true);
            }
        }
    }
}

