﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal sealed class CrossPlatformGroupManager : IDisposable
    {
        private System.IServiceProvider serviceProvider;
        private Microsoft.Xna.GameStudio.SyncRenameComponent syncComponent;
        private uint syncComponentId;
        private Dictionary<string, Dictionary<string, IXnaCodeProject>> syncGroups;

        internal CrossPlatformGroupManager(System.IServiceProvider serviceProvider)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            this.serviceProvider = serviceProvider;
            this.syncGroups = new Dictionary<string, Dictionary<string, IXnaCodeProject>>();
        }

        public void Dispose()
        {
            if ((this.syncComponentId != 0) && (this.serviceProvider != null))
            {
                IOleComponentManager service = (IOleComponentManager) this.serviceProvider.GetService(typeof(IOleComponentManager));
                if (service != null)
                {
                    ErrorHandler.ThrowOnFailure(service.FRevokeComponent(this.syncComponentId));
                }
                this.syncComponent = null;
                this.syncComponentId = 0;
            }
        }

        internal IXnaCodeProject GetProject(string groupID, string platform)
        {
            if (groupID == null)
            {
                throw new ArgumentNullException("groupID");
            }
            if (platform == null)
            {
                throw new ArgumentNullException("platform");
            }
            IXnaCodeProject project = null;
            Dictionary<string, IXnaCodeProject> dictionary = null;
            if (this.syncGroups.TryGetValue(groupID, out dictionary))
            {
                dictionary.TryGetValue(platform, out project);
            }
            return project;
        }

        internal Dictionary<string, IXnaCodeProject> GetProjects(string groupID)
        {
            if (groupID == null)
            {
                throw new ArgumentNullException("groupID");
            }
            Dictionary<string, IXnaCodeProject> dictionary = null;
            if (this.syncGroups.TryGetValue(groupID, out dictionary))
            {
                return dictionary;
            }
            return null;
        }

        internal void RegisterProject(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string crossPlatformGroupID = project.CrossPlatformGroupID;
            if (!string.IsNullOrEmpty(crossPlatformGroupID))
            {
                string buildProperty = project.GetBuildProperty("XnaPlatform", null);
                if (!string.IsNullOrEmpty(buildProperty))
                {
                    Dictionary<string, IXnaCodeProject> dictionary = null;
                    if (!this.syncGroups.TryGetValue(crossPlatformGroupID, out dictionary))
                    {
                        dictionary = new Dictionary<string, IXnaCodeProject>();
                        this.syncGroups[crossPlatformGroupID] = dictionary;
                    }
                    if (dictionary.ContainsKey(buildProperty))
                    {
                        string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(project.GetVsHierarchy());
                        string error = string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.CrossPlatformProjectAlreadyRegisteredForThisPlatform, new object[] { projectName, buildProperty });
                        project.ErrorProvider.ReportError(error, "Microsoft.Xna.GameStudio.CrossPlatformGroupManager");
                        project.ErrorProvider.BringToFront();
                    }
                    else
                    {
                        dictionary[buildProperty] = project;
                    }
                }
            }
        }

        internal void UnregisterProject(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string crossPlatformGroupID = project.CrossPlatformGroupID;
            if (!string.IsNullOrEmpty(crossPlatformGroupID))
            {
                string buildProperty = project.GetBuildProperty("XnaPlatform", null);
                if (!string.IsNullOrEmpty(buildProperty))
                {
                    IXnaCodeProject project2;
                    Dictionary<string, IXnaCodeProject> dictionary = null;
                    if ((this.syncGroups.TryGetValue(crossPlatformGroupID, out dictionary) && dictionary.TryGetValue(buildProperty, out project2)) && object.ReferenceEquals(project, project2))
                    {
                        dictionary.Remove(buildProperty);
                        if (dictionary.Count == 0)
                        {
                            this.syncGroups.Remove(crossPlatformGroupID);
                        }
                    }
                }
            }
        }

        [DebuggerBrowsable(DebuggerBrowsableState.Never)]
        internal Microsoft.Xna.GameStudio.SyncRenameComponent SyncRenameComponent
        {
            get
            {
                if (this.syncComponent == null)
                {
                    this.syncComponent = new Microsoft.Xna.GameStudio.SyncRenameComponent(this);
                    IOleComponentManager service = (IOleComponentManager) this.serviceProvider.GetService(typeof(SOleComponentManager));
                    OLECRINFO[] pcrinfo = new OLECRINFO[1];
                    pcrinfo[0].cbSize = (uint) Marshal.SizeOf(typeof(OLECRINFO));
                    pcrinfo[0].grfcrf = 1;
                    pcrinfo[0].grfcadvf = 7;
                    ErrorHandler.ThrowOnFailure(service.FRegisterComponent(this.syncComponent, pcrinfo, out this.syncComponentId));
                }
                return this.syncComponent;
            }
        }
    }
}

