﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Runtime.InteropServices;

    internal class DeploymentRegistrationService : IXnaDeviceDeploymentRegistration, IVsSelectionEvents, IDisposable
    {
        private readonly Dictionary<KeyPair, string> currentDeviceSelection = new Dictionary<KeyPair, string>();
        private KeyPair currentPlatform;
        private object currentProject;
        private uint monitorCookie;
        private readonly Dictionary<KeyPair, IProjectPlatformAdapter> platformAdapters = new Dictionary<KeyPair, IProjectPlatformAdapter>();
        private readonly Dictionary<KeyPair, PlatformDevices> registeredPlatforms = new Dictionary<KeyPair, PlatformDevices>();
        private readonly Dictionary<object, List<KeyPair>> registeredProjects = new Dictionary<object, List<KeyPair>>(2);
        private IVsMonitorSelection selectionMonitorService;
        private uint visibilityCookie;

        internal DeploymentRegistrationService(IServiceProvider serviceProvider)
        {
            this.selectionMonitorService = (IVsMonitorSelection) serviceProvider.GetService(typeof(IVsMonitorSelection));
            ErrorHandler.ThrowOnFailure(this.selectionMonitorService.AdviseSelectionEvents(this, out this.monitorCookie));
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if ((disposing && (this.selectionMonitorService != null)) && (this.monitorCookie != 0))
            {
                this.selectionMonitorService.UnadviseSelectionEvents(this.monitorCookie);
            }
            this.selectionMonitorService = null;
            this.monitorCookie = 0;
        }

        ~DeploymentRegistrationService()
        {
            this.Dispose(false);
        }

        internal string[] GetAvailableDevices()
        {
            string str;
            return this.GetAvailableDevices(out str);
        }

        internal string[] GetAvailableDevices(out string defaultDevice)
        {
            if ((this.currentPlatform != null) && (this.currentProject != null))
            {
                PlatformDevices devices = this.registeredPlatforms[this.currentPlatform];
                IList<ITitlePlayer> titlePlayers = devices.Platform.GetTitlePlayers();
                if (titlePlayers.Count > 0)
                {
                    List<string> list2;
                    bool flag = true;
                    if (this.platformAdapters[this.currentPlatform] is IProjectPlatformAdapterCaps)
                    {
                        IProjectPlatformAdapterCaps caps = (IProjectPlatformAdapterCaps) this.platformAdapters[this.currentPlatform];
                        if (caps.GetCapabilityValue("listDefaultDevice").Equals("false", StringComparison.InvariantCultureIgnoreCase))
                        {
                            flag = false;
                        }
                    }
                    foreach (KeyPair pair in this.registeredProjects[this.currentProject])
                    {
                        string runtimeToken = pair.RuntimeToken;
                        Dictionary<string, string> dictionary = this.registeredPlatforms[pair].Devices;
                        dictionary.Clear();
                        foreach (ITitlePlayer player in titlePlayers)
                        {
                            if (player.GetRuntimeSupport(runtimeToken) == RuntimeSupport.Supported)
                            {
                                dictionary.Add(player.ToString(), player.Name);
                            }
                        }
                    }
                    if (flag)
                    {
                        defaultDevice = XnaProjectResource.DefaultDeviceDisplayName;
                        list2 = new List<string>(titlePlayers.Count + 1) {
                            defaultDevice
                        };
                    }
                    else
                    {
                        defaultDevice = devices.Platform.GetTitlePlayer(null).ToString();
                        list2 = new List<string>(titlePlayers.Count);
                    }
                    list2.AddRange(this.registeredPlatforms[this.currentPlatform].Devices.Keys);
                    return list2.ToArray();
                }
            }
            defaultDevice = null;
            return null;
        }

        internal string GetCurrentSelection()
        {
            if ((this.currentPlatform == null) || (this.currentProject == null))
            {
                return "";
            }
            if (this.currentDeviceSelection.ContainsKey(this.currentPlatform))
            {
                return this.currentDeviceSelection[this.currentPlatform];
            }
            string defaultDevice = null;
            if (this.currentProject is XnaProject)
            {
                defaultDevice = (this.currentProject as XnaProject).GetUserProjectProperty("XNASelectedDeployDevice");
            }
            if (defaultDevice == null)
            {
                string[] availableDevices = this.GetAvailableDevices(out defaultDevice);
                if ((availableDevices == null) || (availableDevices.Length <= 0))
                {
                    return defaultDevice;
                }
                foreach (KeyPair pair in this.registeredProjects[this.currentProject])
                {
                    this.currentDeviceSelection[pair] = defaultDevice;
                }
            }
            return defaultDevice;
        }

        public string GetDeployDevice(string platformName, string runtimeProfile)
        {
            PlatformDevices devices;
            string str;
            string str2;
            KeyPair key = new KeyPair(platformName, runtimeProfile);
            if (!this.registeredPlatforms.TryGetValue(key, out devices))
            {
                return null;
            }
            if (!this.currentDeviceSelection.TryGetValue(key, out str) || (str == XnaProjectResource.DefaultDeviceDisplayName))
            {
                return devices.Platform.DefaultTitlePlayer;
            }
            if (!devices.Devices.TryGetValue(str, out str2))
            {
                return null;
            }
            return str2;
        }

        private KeyPair GetProjectRuntimeProfile(object projectUnknown)
        {
            IHierarchyObjectProvider provider = projectUnknown as IHierarchyObjectProvider;
            if (provider == null)
            {
                return null;
            }
            IXnaCodeProject project = provider.GetObject(typeof(IXnaCodeProject)) as IXnaCodeProject;
            if (project == null)
            {
                return null;
            }
            string buildProperty = project.GetBuildProperty("XnaPlatform", null);
            return new KeyPair(buildProperty, project.GetBuildProperty("XnaFrameworkRuntimeProfile", null));
        }

        public int OnCmdUIContextChanged(uint dwCmdUICookie, int fActive)
        {
            return 0;
        }

        public int OnElementValueChanged(uint elementid, object varValueOld, object varValueNew)
        {
            if (elementid == 3)
            {
                this.currentPlatform = null;
                foreach (object obj2 in this.registeredProjects.Keys)
                {
                    if (ComUtilities.IsSameComObject(obj2, varValueNew))
                    {
                        this.currentProject = obj2;
                        List<KeyPair> list = this.registeredProjects[obj2];
                        if (list.Count == 1)
                        {
                            this.currentPlatform = list[0];
                        }
                        else
                        {
                            this.currentPlatform = this.GetProjectRuntimeProfile(obj2);
                            if (this.currentPlatform == null)
                            {
                                this.currentPlatform = list[0];
                            }
                        }
                        if (obj2 is XnaProject)
                        {
                            XnaPackage.Instance.EnableCentralDeviceDropdown(((XnaProject) obj2).SupportsCentralDeviceDropdown);
                        }
                        else
                        {
                            XnaPackage.Instance.EnableCentralDeviceDropdown(false);
                        }
                        IVsUIShell globalService = Package.GetGlobalService(typeof(SVsUIShell)) as IVsUIShell;
                        if (globalService != null)
                        {
                            ErrorHandler.ThrowOnFailure(globalService.UpdateCommandUI(1));
                        }
                        break;
                    }
                }
            }
            return 0;
        }

        public int OnSelectionChanged(IVsHierarchy pHierOld, uint itemidOld, IVsMultiItemSelect pMISOld, ISelectionContainer pSCOld, IVsHierarchy pHierNew, uint itemidNew, IVsMultiItemSelect pMISNew, ISelectionContainer pSCNew)
        {
            return 0;
        }

        public void RegisterProject(object projectUnknown, string platform, string runtimeProfile)
        {
            if ((platform != null) && (runtimeProfile != null))
            {
                IPlatform platform2 = PlatformManager.GetPlatform(platform, runtimeProfile);
                if (platform2 != null)
                {
                    PlatformDevices devices;
                    KeyPair item = new KeyPair(platform, runtimeProfile);
                    if (!this.registeredProjects.ContainsKey(projectUnknown))
                    {
                        this.registeredProjects.Add(projectUnknown, new List<KeyPair> { item });
                    }
                    else
                    {
                        this.registeredProjects[projectUnknown].Add(item);
                    }
                    IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(Microsoft.Xna.GameStudio.ProjectUtilities.GetVsHierarchy(projectUnknown) as IHierarchyObjectProvider);
                    this.platformAdapters[item] = hierarchyObject.Adapter;
                    if (!this.registeredPlatforms.TryGetValue(item, out devices))
                    {
                        devices = new PlatformDevices(platform2);
                        this.registeredPlatforms.Add(item, devices);
                    }
                    else
                    {
                        devices.Platform = platform2;
                    }
                    if (this.registeredProjects.Count == 1)
                    {
                        Guid guidDeviceManagementToolbarVisibility = CmdIDConstants.guidDeviceManagementToolbarVisibility;
                        int cmdUIContextCookie = this.selectionMonitorService.GetCmdUIContextCookie(ref guidDeviceManagementToolbarVisibility, out this.visibilityCookie);
                        if (ErrorHandler.Succeeded(cmdUIContextCookie))
                        {
                            ErrorHandler.ThrowOnFailure(this.selectionMonitorService.SetCmdUIContext(this.visibilityCookie, 1));
                        }
                        else
                        {
                            IVsShell globalService = Package.GetGlobalService(typeof(SVsShell)) as IVsShell;
                            if (globalService != null)
                            {
                                object obj2;
                                ErrorHandler.ThrowOnFailure(globalService.GetProperty(-9022, out obj2));
                                if ((obj2 is bool) && ((bool) obj2))
                                {
                                    cmdUIContextCookie = 0;
                                }
                            }
                            ErrorHandler.ThrowOnFailure(cmdUIContextCookie);
                        }
                    }
                }
            }
        }

        internal void SetCurrentSelection(string deviceName)
        {
            this.SetCurrentSelection(deviceName, deviceName);
        }

        internal void SetCurrentSelection(string friendlyName, string deviceName)
        {
            if ((this.currentPlatform != null) && (this.currentProject != null))
            {
                List<KeyPair> list = this.registeredProjects[this.currentProject];
                foreach (KeyPair pair in list)
                {
                    this.currentDeviceSelection[pair] = deviceName;
                    PlatformDevices devices = this.registeredPlatforms[pair];
                    if (((devices != null) && (devices.Devices != null)) && !devices.Devices.ContainsKey(friendlyName))
                    {
                        devices.Devices.Add(friendlyName, deviceName);
                    }
                    if (this.currentProject is XnaProject)
                    {
                        (this.currentProject as XnaProject).SetUserProjectProperty("XNASelectedDeployDevice", friendlyName);
                    }
                }
            }
        }

        internal void ShowAddDeviceDialog()
        {
            if (this.currentPlatform != null)
            {
                ITitlePlayer player = this.registeredPlatforms[this.currentPlatform].Platform.ShowAddTitlePlayerDialog();
                if ((player != null) && (player.Platform == this.currentPlatform.Platform))
                {
                    this.SetCurrentSelection(player.ToString(), player.Name);
                }
            }
        }

        public void UnregisterProject(object projectUnknown)
        {
            if (this.registeredProjects.ContainsKey(projectUnknown))
            {
                this.registeredProjects.Remove(projectUnknown);
                if ((this.registeredProjects.Count == 0) && (this.visibilityCookie != 0))
                {
                    ErrorHandler.ThrowOnFailure(this.selectionMonitorService.SetCmdUIContext(this.visibilityCookie, 0));
                    this.currentPlatform = null;
                    this.visibilityCookie = 0;
                }
            }
        }

        internal bool ContainsValidDevices
        {
            get
            {
                return (this.currentPlatform != null);
            }
        }

        internal bool CurrentPlatformSupportsDeviceCenter
        {
            get
            {
                if (this.currentPlatform == null)
                {
                    return false;
                }
                return this.platformAdapters[this.currentPlatform].SupportsDeviceCenter;
            }
        }

        private class KeyPair : IEquatable<DeploymentRegistrationService.KeyPair>
        {
            private string combinedValue;
            public readonly string Platform;
            public readonly string RuntimeToken;

            public KeyPair(string platform, string runtimeToken)
            {
                if (platform == null)
                {
                    throw new ArgumentNullException("platform");
                }
                if (runtimeToken == null)
                {
                    throw new ArgumentNullException("runtimeToken");
                }
                this.Platform = platform;
                this.RuntimeToken = runtimeToken;
                this.combinedValue = platform + ";" + runtimeToken;
            }

            public bool Equals(DeploymentRegistrationService.KeyPair other)
            {
                if (other == null)
                {
                    return false;
                }
                return ((this.Platform == other.Platform) && (this.RuntimeToken == other.RuntimeToken));
            }

            public override int GetHashCode()
            {
                return this.combinedValue.GetHashCode();
            }
        }

        private class PlatformDevices
        {
            public readonly Dictionary<string, string> Devices = new Dictionary<string, string>();
            public IPlatform Platform;

            public PlatformDevices(IPlatform platform)
            {
                this.Platform = platform;
            }
        }
    }
}

