﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.CodeAnalysis;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;

    internal class HierarchyItem
    {
        private List<int> disabledGuidProperties;
        private List<int> disabledProperties;
        private IVsUIHierarchy innerIVsUIHierarchy;
        private Dictionary<int, GetGuidPropertyDelegate> overrideGetGuidProperties;
        private Dictionary<int, GetPropertyDelegate> overrideGetProperties;
        private Dictionary<int, Guid> overrideGuidProperties;
        private Dictionary<int, object> overrideProperties;
        private Dictionary<int, SetPropertyDelegate> overrideSetProperties;

        internal HierarchyItem(IVsUIHierarchy innerIVsUIHierarchy)
        {
            this.innerIVsUIHierarchy = innerIVsUIHierarchy;
            this.disabledProperties = new List<int>();
            this.overrideProperties = new Dictionary<int, object>();
            this.overrideGetProperties = new Dictionary<int, GetPropertyDelegate>();
            this.overrideSetProperties = new Dictionary<int, SetPropertyDelegate>();
            this.disabledGuidProperties = new List<int>();
            this.overrideGuidProperties = new Dictionary<int, Guid>();
            this.overrideGetGuidProperties = new Dictionary<int, GetGuidPropertyDelegate>();
        }

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        protected void DisableGuidProperty(int propid)
        {
            DisableProperty(propid, this.disabledGuidProperties);
        }

        protected void DisableProperty(int propid)
        {
            DisableProperty(propid, this.disabledProperties);
        }

        private static void DisableProperty(int propid, List<int> sortedList)
        {
            if (sortedList == null)
            {
                throw new ArgumentNullException("sortedList");
            }
            int num = sortedList.BinarySearch(propid);
            if (num < 0)
            {
                int index = ~num;
                sortedList.Insert(index, propid);
            }
        }

        internal virtual int ExecCommand(uint itemid, ref Guid pguidCmdGroup, uint nCmdID, uint nCmdexecopt, IntPtr pvaIn, IntPtr pvaOut)
        {
            if (this.innerIVsUIHierarchy != null)
            {
                return this.innerIVsUIHierarchy.ExecCommand(itemid, ref pguidCmdGroup, nCmdID, nCmdexecopt, pvaIn, pvaOut);
            }
            return -2147467263;
        }

        internal virtual int GetGuidProperty(uint itemid, int propid, out Guid pguid)
        {
            pguid = Guid.Empty;
            if (this.disabledGuidProperties.BinarySearch(propid) < 0)
            {
                if (this.overrideGuidProperties.TryGetValue(propid, out pguid))
                {
                    return 0;
                }
                GetGuidPropertyDelegate delegate2 = null;
                if (this.overrideGetGuidProperties.TryGetValue(propid, out delegate2))
                {
                    return delegate2(itemid, propid, out pguid);
                }
                if (this.innerIVsUIHierarchy != null)
                {
                    return this.innerIVsUIHierarchy.GetGuidProperty(itemid, propid, out pguid);
                }
            }
            return -2147467263;
        }

        internal virtual int GetProperty(uint itemid, int propid, out object pvar)
        {
            pvar = null;
            if (this.disabledProperties.BinarySearch(propid) < 0)
            {
                if (this.overrideProperties.TryGetValue(propid, out pvar))
                {
                    return 0;
                }
                GetPropertyDelegate delegate2 = null;
                if (this.overrideGetProperties.TryGetValue(propid, out delegate2))
                {
                    return delegate2(itemid, propid, out pvar);
                }
                if (this.innerIVsUIHierarchy != null)
                {
                    return this.innerIVsUIHierarchy.GetProperty(itemid, propid, out pvar);
                }
            }
            return -2147467263;
        }

        protected void OverrideGuidProperty(int propid, ref Guid guidValue)
        {
            this.ResetGuidProperty(propid);
            this.overrideGuidProperties[propid] = guidValue;
        }

        protected void OverrideGuidProperty(int propid, GetGuidPropertyDelegate getHandler)
        {
            if (getHandler == null)
            {
                throw new ArgumentNullException("getHandler");
            }
            this.ResetGuidProperty(propid);
            this.overrideGetGuidProperties[propid] = getHandler;
        }

        protected void OverrideProperty(int propid, GetPropertyDelegate getHandler)
        {
            if (getHandler == null)
            {
                throw new ArgumentNullException("getHandler");
            }
            this.OverrideProperty(propid, getHandler, null);
        }

        protected void OverrideProperty(int propid, object value)
        {
            this.ResetProperty(propid);
            this.overrideProperties[propid] = value;
        }

        protected void OverrideProperty(int propid, GetPropertyDelegate getHandler, SetPropertyDelegate setHandler)
        {
            if (getHandler == null)
            {
                throw new ArgumentNullException("getHandler");
            }
            this.ResetProperty(propid);
            this.overrideGetProperties[propid] = getHandler;
            this.overrideSetProperties[propid] = setHandler;
        }

        internal virtual int QueryStatusCommand(uint itemid, ref Guid pguidCmdGroup, uint cCmds, OLECMD[] prgCmds, IntPtr pCmdText)
        {
            if (this.innerIVsUIHierarchy != null)
            {
                return this.innerIVsUIHierarchy.QueryStatusCommand(itemid, ref pguidCmdGroup, cCmds, prgCmds, pCmdText);
            }
            return -2147467263;
        }

        protected void ResetGuidProperty(int propid)
        {
            ResetProperty(propid, this.disabledGuidProperties);
            if (this.overrideGuidProperties.ContainsKey(propid))
            {
                this.overrideGuidProperties.Remove(propid);
            }
            if (this.overrideGetGuidProperties.ContainsKey(propid))
            {
                this.overrideGetGuidProperties.Remove(propid);
            }
        }

        protected void ResetProperty(int propid)
        {
            ResetProperty(propid, this.disabledProperties);
            if (this.overrideProperties.ContainsKey(propid))
            {
                this.overrideProperties.Remove(propid);
            }
            if (this.overrideGetProperties.ContainsKey(propid))
            {
                this.overrideGetProperties.Remove(propid);
            }
            if (this.overrideSetProperties.ContainsKey(propid))
            {
                this.overrideSetProperties.Remove(propid);
            }
        }

        private static void ResetProperty(int propid, List<int> sortedList)
        {
            if (sortedList == null)
            {
                throw new ArgumentNullException("sortedList");
            }
            int index = sortedList.BinarySearch(propid);
            if (index >= 0)
            {
                sortedList.RemoveAt(index);
            }
        }

        internal virtual int SetGuidProperty(uint itemid, int propid, ref Guid rguid)
        {
            if (this.disabledGuidProperties.BinarySearch(propid) < 0)
            {
                if (this.overrideGuidProperties.ContainsKey(propid))
                {
                    this.overrideGuidProperties[propid] = rguid;
                    return 0;
                }
                if (this.innerIVsUIHierarchy != null)
                {
                    return this.innerIVsUIHierarchy.SetGuidProperty(itemid, propid, ref rguid);
                }
            }
            return -2147467263;
        }

        internal virtual int SetProperty(uint itemid, int propid, object var)
        {
            if (this.disabledProperties.BinarySearch(propid) < 0)
            {
                if (this.overrideProperties.ContainsKey(propid))
                {
                    this.overrideProperties[propid] = var;
                    return 0;
                }
                SetPropertyDelegate delegate2 = null;
                if (this.overrideSetProperties.TryGetValue(propid, out delegate2))
                {
                    if (delegate2 == null)
                    {
                        return -2147467263;
                    }
                    return delegate2(itemid, propid, var);
                }
                if (this.innerIVsUIHierarchy != null)
                {
                    return this.innerIVsUIHierarchy.SetProperty(itemid, propid, var);
                }
            }
            return -2147467263;
        }

        protected IVsHierarchy InnerHierarchy
        {
            get
            {
                return this.innerIVsUIHierarchy;
            }
        }

        protected delegate int GetGuidPropertyDelegate(uint itemid, int propid, out Guid value);

        protected delegate int GetPropertyDelegate(uint itemid, int propid, out object value);

        protected delegate int SetPropertyDelegate(uint itemid, int propid, object value);
    }
}

