﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.ManagedInterfaces.ProjectDesigner;
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Drawing;
    using System.Globalization;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    public abstract class ProjectDesignerPropertyPage : IPropertyPage, IVsProjectDesignerPage, IDisposable
    {
        private IVsProjectDesignerPageSite designerSite;
        private string editedPropertyName;
        private System.Type editedPropertyType;
        private string f1HelpKeyword;
        private bool inApply;
        private bool isDirty;
        private string name;
        private FlowLayoutPanel pageViewContainer;
        private bool pendingDispose;
        private IPropertyPageSite propertyPageSite;
        private ISolutionBuildStatusProvider solutionBuildStatusProvider;

        internal ProjectDesignerPropertyPage(string name, string f1HelpKeyword)
        {
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (f1HelpKeyword == null)
            {
                throw new ArgumentNullException("f1HelpKeyword");
            }
            this.name = name;
            this.f1HelpKeyword = f1HelpKeyword;
        }

        public virtual void Activate(IntPtr hWndParent, Microsoft.VisualStudio.OLE.Interop.RECT[] pRect, int bModal)
        {
            if (pRect == null)
            {
                throw new ArgumentNullException("pRect");
            }
            if (this.pageViewContainer == null)
            {
                Microsoft.VisualStudio.OLE.Interop.RECT rect = pRect[0];
                Rectangle rectangle = Rectangle.FromLTRB(rect.left, rect.top, rect.right, rect.bottom);
                this.pageViewContainer = new FlowLayoutPanel();
                this.pageViewContainer.SuspendLayout();
                this.pageViewContainer.FlowDirection = FlowDirection.TopDown;
                this.pageViewContainer.WrapContents = true;
                this.pageViewContainer.AutoScroll = true;
                this.pageViewContainer.SetBounds(rectangle.X, rectangle.Y, rectangle.Width, rectangle.Height);
                this.pageViewContainer.Padding = Padding.Empty;
                this.pageViewContainer.Controls.Add(this.PageView);
                this.pageViewContainer.Location = new Point(rect.left, rect.top);
                this.pageViewContainer.TabIndex = 0;
                this.pageViewContainer.Parent = Control.FromHandle(hWndParent);
                this.pageViewContainer.ResumeLayout();
                this.pageViewContainer.PerformLayout();
                this.pageViewContainer.GetNextControl(null, true).Focus();
                this.SetPropertyPageEnabled();
            }
        }

        public virtual int Apply()
        {
            int num = 0;
            bool inApply = this.inApply;
            this.inApply = true;
            try
            {
                if (!this.isDirty || string.IsNullOrEmpty(this.editedPropertyName))
                {
                    return num;
                }
                DesignerTransaction transaction = null;
                PropertyDescriptor propertyDescriptor = TypeDescriptor.CreateProperty(base.GetType(), this.editedPropertyName, this.editedPropertyType, new Attribute[0]);
                try
                {
                    if (this.designerSite != null)
                    {
                        transaction = this.designerSite.GetTransaction(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.ProjectDesignerTransaction, new object[] { this.editedPropertyName }));
                        this.designerSite.OnPropertyChanging(this.editedPropertyName, propertyDescriptor);
                    }
                    this.OnCommitPropertyEdit();
                    this.OnStatusChange(false);
                    if (this.designerSite != null)
                    {
                        this.designerSite.OnPropertyChanged(this.editedPropertyName, propertyDescriptor, null, null);
                        transaction.Commit();
                    }
                }
                finally
                {
                    IDisposable disposable = transaction;
                    if (disposable != null)
                    {
                        disposable.Dispose();
                    }
                    this.isDirty = false;
                }
            }
            finally
            {
                this.inApply = inApply;
                if (this.pendingDispose && !this.inApply)
                {
                    this.Dispose();
                }
            }
            return num;
        }

        public virtual void Deactivate()
        {
            this.Dispose();
        }

        public void Dispose()
        {
            if (this.inApply)
            {
                this.pendingDispose = true;
            }
            else
            {
                this.Dispose(true);
                GC.SuppressFinalize(this);
            }
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (this.pageViewContainer != null)
                {
                    this.pageViewContainer.Dispose();
                }
                if (this.solutionBuildStatusProvider != null)
                {
                    this.solutionBuildStatusProvider.BuildStatusChanged -= new EventHandler(this.OnBuildStatusChanged);
                }
            }
            this.pageViewContainer = null;
            this.solutionBuildStatusProvider = null;
        }

        ~ProjectDesignerPropertyPage()
        {
            this.Dispose(false);
        }

        public virtual bool FinishPendingValidations()
        {
            return true;
        }

        public virtual void GetPageInfo(PROPPAGEINFO[] pPageInfo)
        {
            if ((pPageInfo == null) || (pPageInfo.Length == 0))
            {
                throw new ArgumentNullException("pPageInfo");
            }
            PROPPAGEINFO proppageinfo = new PROPPAGEINFO {
                cb = (uint) Marshal.SizeOf(typeof(PROPPAGEINFO)),
                dwHelpContext = 0,
                pszDocString = null,
                pszHelpFile = null,
                pszTitle = this.name
            };
            if (this.pageViewContainer != null)
            {
                Rectangle displayRectangle = this.pageViewContainer.DisplayRectangle;
                proppageinfo.SIZE.cx = displayRectangle.Width;
                proppageinfo.SIZE.cy = displayRectangle.Height;
            }
            pPageInfo[0] = proppageinfo;
        }

        public virtual object GetProperty(string propertyName)
        {
            throw new NotImplementedException();
        }

        public virtual bool GetPropertyMultipleValues(string propertyName, out object[] objects, out object[] values)
        {
            objects = null;
            values = null;
            return false;
        }

        public virtual void Help(string pszHelpDir)
        {
        }

        public virtual int IsPageDirty()
        {
            if (this.isDirty)
            {
                return 0;
            }
            return 1;
        }

        public virtual void Move(Microsoft.VisualStudio.OLE.Interop.RECT[] pRect)
        {
            Microsoft.VisualStudio.OLE.Interop.RECT rect = pRect[0];
            Rectangle rectangle = Rectangle.FromLTRB(rect.left, rect.top, rect.right, rect.bottom);
            this.pageViewContainer.SetBounds(rectangle.X, rectangle.Y, rectangle.Width, rectangle.Height);
        }

        public virtual void OnActivated(bool activated)
        {
            if (activated)
            {
                this.PageView.Focus();
            }
        }

        private void OnBuildStatusChanged(object sender, EventArgs args)
        {
            this.SetPropertyPageEnabled();
        }

        protected abstract void OnCommitPropertyEdit();
        protected void OnPropertyEdited(string propertyName, System.Type propertyType)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            this.editedPropertyName = propertyName;
            this.editedPropertyType = propertyType;
            try
            {
                this.OnStatusChange(true);
            }
            finally
            {
                this.editedPropertyName = null;
                this.editedPropertyType = null;
            }
        }

        private void OnStatusChange(bool isStatusDirty)
        {
            if (this.isDirty != isStatusDirty)
            {
                this.isDirty = isStatusDirty;
                uint dwFlags = isStatusDirty ? 3 : 4;
                this.propertyPageSite.OnStatusChange(dwFlags);
            }
        }

        public virtual void SetObjects(uint cObjects, object[] ppunk)
        {
            if (((this.solutionBuildStatusProvider == null) && (ppunk != null)) && (cObjects <= ppunk.Length))
            {
                for (int i = 0; i < cObjects; i++)
                {
                    uint num2;
                    IVsCfgBrowseObject obj2 = ppunk[i] as IVsCfgBrowseObject;
                    IVsHierarchy pHier = null;
                    if (obj2 != null)
                    {
                        ErrorHandler.ThrowOnFailure(obj2.GetProjectItem(out pHier, out num2));
                    }
                    else
                    {
                        IVsBrowseObject obj3 = ppunk[i] as IVsBrowseObject;
                        if (obj3 != null)
                        {
                            ErrorHandler.ThrowOnFailure(obj3.GetProjectItem(out pHier, out num2));
                        }
                    }
                    if (pHier != null)
                    {
                        this.solutionBuildStatusProvider = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<ISolutionBuildStatusProvider>(pHier as IHierarchyObjectProvider);
                        this.solutionBuildStatusProvider.BuildStatusChanged += new EventHandler(this.OnBuildStatusChanged);
                        return;
                    }
                }
            }
        }

        public virtual void SetPageSite(IPropertyPageSite pPageSite)
        {
            this.propertyPageSite = pPageSite;
        }

        public virtual void SetProperty(string propertyName, object value)
        {
            throw new NotImplementedException();
        }

        public virtual void SetPropertyMultipleValues(string propertyName, object[] objects, object[] values)
        {
        }

        protected virtual void SetPropertyPageEnabled()
        {
            if (this.pageViewContainer != null)
            {
                bool flag = false;
                if (this.solutionBuildStatusProvider != null)
                {
                    flag = !this.solutionBuildStatusProvider.IsBuilding;
                }
                this.pageViewContainer.Enabled = flag;
            }
        }

        public virtual void SetSite(IVsProjectDesignerPageSite site)
        {
            this.designerSite = site;
        }

        public virtual void Show(uint nCmdShow)
        {
            switch (nCmdShow)
            {
                case 0:
                    this.pageViewContainer.Hide();
                    return;

                case 1:
                case 5:
                {
                    this.pageViewContainer.Show();
                    System.IServiceProvider propertyPageSite = this.propertyPageSite as System.IServiceProvider;
                    if (propertyPageSite == null)
                    {
                        break;
                    }
                    IHelpService service = propertyPageSite.GetService(typeof(IHelpService)) as IHelpService;
                    if (service == null)
                    {
                        break;
                    }
                    service.AddContextAttribute("Keyword", this.f1HelpKeyword, HelpKeywordType.F1Keyword);
                    return;
                }
                default:
                    throw new ArgumentException(XnaProjectResource.ShowArgException, "nCmdShow");
            }
        }

        public virtual bool SupportsMultipleValueUndo(string propertyName)
        {
            return false;
        }

        public virtual int TranslateAccelerator(Microsoft.VisualStudio.OLE.Interop.MSG[] pMsg)
        {
            return 0;
        }

        protected abstract Control PageView { get; }
    }
}

