﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject.Debugging;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Globalization;
    using System.Runtime.InteropServices;

    internal class ProjectFlavorCfg : Component, IVsProjectFlavorCfg
    {
        private IVsCfg baseCfg;
        private IVsBuildPropertyStorage buildPropertyStorage;
        private Dictionary<Guid, CfgTypeObject> cfgTypeObjects;
        private IVsHierarchy hierarchy;
        private IVsProjectFlavorCfg innerCfg;
        private bool isClosed;
        private string xnaFrameworkVersion;

        internal ProjectFlavorCfg(IVsCfg baseCfg, IVsProjectFlavorCfg innerCfg, string xnaFrameworkVersion)
        {
            if (baseCfg == null)
            {
                throw new ArgumentNullException("baseCfg");
            }
            if (xnaFrameworkVersion == null)
            {
                throw new ArgumentNullException("xnaFrameworkVersion");
            }
            this.baseCfg = baseCfg;
            IVsCfgBrowseObject obj2 = baseCfg as IVsCfgBrowseObject;
            if (obj2 != null)
            {
                uint maxValue = uint.MaxValue;
                if (ErrorHandler.Succeeded(obj2.GetProjectItem(out this.hierarchy, out maxValue)) && (this.hierarchy != null))
                {
                    this.buildPropertyStorage = this.hierarchy as IVsBuildPropertyStorage;
                    IVsProject hierarchy = this.hierarchy as IVsProject;
                    if ((this.buildPropertyStorage != null) && (hierarchy != null))
                    {
                        this.buildPropertyStorage = new VsBuildPropertyStorageSccWrapper(this.buildPropertyStorage, hierarchy);
                    }
                }
            }
            this.innerCfg = innerCfg;
            this.xnaFrameworkVersion = xnaFrameworkVersion;
        }

        protected void AddCfgType(Type cfgType, object cfgObject)
        {
            if (cfgType == null)
            {
                throw new ArgumentNullException("cfgType");
            }
            if ((cfgObject != null) && (!cfgType.IsInterface || !cfgType.IsInstanceOfType(cfgObject)))
            {
                throw new ArgumentException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.NotInstanceOfType, new object[] { "cfgObject", "cfgType" }));
            }
            if (this.cfgTypeObjects == null)
            {
                this.cfgTypeObjects = new Dictionary<Guid, CfgTypeObject>();
            }
            Guid gUID = cfgType.GUID;
            CfgTypeObject obj2 = new CfgTypeObject(cfgType, cfgObject);
            this.cfgTypeObjects[gUID] = obj2;
        }

        public virtual int Close()
        {
            if (!this.isClosed)
            {
                this.baseCfg = null;
                IVsProjectFlavorCfg innerCfg = this.innerCfg;
                this.innerCfg = null;
                this.isClosed = true;
                if (innerCfg != null)
                {
                    return innerCfg.Close();
                }
                base.Dispose();
            }
            return 0;
        }

        public virtual int get_CfgType(ref Guid iidCfg, out IntPtr ppCfg)
        {
            CfgTypeObject obj2 = null;
            if ((this.cfgTypeObjects != null) && this.cfgTypeObjects.TryGetValue(iidCfg, out obj2))
            {
                if ((obj2 == null) || (obj2.Object == null))
                {
                    ppCfg = IntPtr.Zero;
                    return -2147467262;
                }
                ppCfg = Marshal.GetComInterfaceForObject(obj2.Object, obj2.Type);
                return 0;
            }
            if (this.innerCfg == null)
            {
                ppCfg = IntPtr.Zero;
                return -2147467262;
            }
            return this.innerCfg.get_CfgType(ref iidCfg, out ppCfg);
        }

        internal string GetConfigurationName()
        {
            string str;
            ErrorHandler.ThrowOnFailure(this.baseCfg.get_DisplayName(out str));
            return str;
        }

        internal string GetConfigurationProperty(string propertyName)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            string pbstrPropValue = null;
            if (this.buildPropertyStorage != null)
            {
                string configurationName = this.GetConfigurationName();
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.GetPropertyValue(propertyName, configurationName, storage, out pbstrPropValue), new int[] { -2147170504 });
            }
            return pbstrPropValue;
        }

        internal object GetInnerCfgType(Type t)
        {
            if (t == null)
            {
                throw new ArgumentNullException("t");
            }
            object objectForIUnknown = null;
            if (this.innerCfg != null)
            {
                IntPtr zero = IntPtr.Zero;
                try
                {
                    Guid gUID = t.GUID;
                    ErrorHandler.ThrowOnFailure(this.innerCfg.get_CfgType(ref gUID, out zero), new int[] { -2147467262 });
                    if (zero != IntPtr.Zero)
                    {
                        objectForIUnknown = Marshal.GetObjectForIUnknown(zero);
                    }
                }
                finally
                {
                    if (zero != IntPtr.Zero)
                    {
                        Marshal.Release(zero);
                    }
                }
            }
            return objectForIUnknown;
        }

        protected void RemoveCfgTypeObject(Type cfgType)
        {
            if (cfgType == null)
            {
                throw new ArgumentNullException("cfgType");
            }
            Guid gUID = cfgType.GUID;
            if ((this.cfgTypeObjects == null) || !this.cfgTypeObjects.ContainsKey(gUID))
            {
                throw new InvalidOperationException();
            }
            this.cfgTypeObjects.Remove(gUID);
        }

        internal void SetConfigurationProperty(string propertyName, string propertyValue)
        {
            if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName");
            }
            if (this.buildPropertyStorage != null)
            {
                string configurationName = this.GetConfigurationName();
                uint storage = 1;
                ErrorHandler.ThrowOnFailure(this.buildPropertyStorage.SetPropertyValue(propertyName, configurationName, storage, propertyValue));
            }
        }

        internal IVsCfg BaseCfg
        {
            get
            {
                return this.baseCfg;
            }
        }

        internal string XnaFrameworkRuntimeProfile
        {
            get
            {
                return this.GetConfigurationProperty("XnaFrameworkRuntimeProfile");
            }
        }

        internal string XnaFrameworkVersion
        {
            get
            {
                return this.xnaFrameworkVersion;
            }
        }
    }
}

