﻿namespace Microsoft.Xna.GameStudio
{
    using EnvDTE;
    using Microsoft.Build.Evaluation;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.VisualStudio.VSHelp;
    using Microsoft.VisualStudio.VSHelp80;
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.ContentProject;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Diagnostics;
    using System.Drawing;
    using System.Globalization;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Text;
    using System.Threading;
    using System.Windows.Forms;
    using VSLangProj;

    internal static class ProjectUtilities
    {
        private const int ILD_NORMAL = 0;

        internal static bool Checkout(string[] files)
        {
            return Checkout(files, false);
        }

        internal static bool Checkout(string[] files, bool noPrompt)
        {
            uint num;
            uint num2;
            if (files == null)
            {
                throw new ArgumentNullException("files");
            }
            IVsQueryEditQuerySave2 globalService = Package.GetGlobalService(typeof(SVsQueryEditQuerySave)) as IVsQueryEditQuerySave2;
            if (globalService == null)
            {
                return true;
            }
            tagVSQueryEditFlags flags = tagVSQueryEditFlags.QEF_DisallowInMemoryEdits;
            if (noPrompt)
            {
                flags |= tagVSQueryEditFlags.QEF_ForceEdit_NoPrompting;
            }
            if (ErrorHandler.Failed(globalService.QueryEditFiles((uint) flags, files.Length, files, new uint[files.Length], new VSQEQS_FILE_ATTRIBUTE_DATA[files.Length], out num, out num2)))
            {
                return false;
            }
            return (num == 0);
        }

        internal static void CopyDirectoryContents(string oldLocation, string newLocation)
        {
            if (string.IsNullOrEmpty(oldLocation))
            {
                throw new ArgumentNullException("oldLocation");
            }
            if (string.IsNullOrEmpty(newLocation))
            {
                throw new ArgumentNullException("newLocation");
            }
            if (!Directory.Exists(newLocation))
            {
                Directory.CreateDirectory(newLocation);
            }
            foreach (string str in Directory.GetFiles(oldLocation))
            {
                File.Copy(Path.Combine(oldLocation, str), Path.Combine(newLocation, Path.GetFileName(str)));
            }
            foreach (string str2 in Directory.GetDirectories(oldLocation))
            {
                CopyDirectoryContents(Path.Combine(oldLocation, str2), Path.Combine(newLocation, Path.GetFileName(str2)));
            }
        }

        internal static string CreateSemicolonDelimitedListOfStringFromGuids(Guid[] guids)
        {
            if ((guids == null) || (guids.Length == 0))
            {
                return string.Empty;
            }
            if (guids.Length == 1)
            {
                return guids[0].ToString("B");
            }
            StringBuilder builder = new StringBuilder(0x27 * guids.Length);
            builder.Append(guids[0].ToString("B"));
            for (int i = 1; i < guids.Length; i++)
            {
                builder.Append(";");
                builder.Append(guids[i].ToString("B"));
            }
            return builder.ToString();
        }

        public static string EnsureTrailingBackSlash(string str)
        {
            if ((str != null) && !str.EndsWith(@"\", StringComparison.Ordinal))
            {
                str = str + @"\";
            }
            return str;
        }

        internal static IEnumerable<uint> EnumerateHierarchyItemsFlat(IVsHierarchy hierarchy, uint targetId = 0xfffffffe)
        {
            object iteratorVariable0;
            ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(targetId, -1001, out iteratorVariable0));
            for (uint i = (uint) ((int) iteratorVariable0); i != uint.MaxValue; i = (uint) ((int) iteratorVariable0))
            {
                IEnumerator<uint> enumerator = EnumerateHierarchyItemsFlat(hierarchy, i).GetEnumerator();
                while (enumerator.MoveNext())
                {
                    uint current = enumerator.Current;
                    yield return current;
                }
                yield return i;
                ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(i, -1002, out iteratorVariable0));
            }
        }

        internal static IEnumerable<IVsHierarchy> EnumerateProjects(__VSENUMPROJFLAGS projectFlags)
        {
            IVsSolution2 globalService = Package.GetGlobalService(typeof(SVsSolution)) as IVsSolution2;
            if (globalService == null)
            {
                goto Label_00F9;
            }
            IEnumHierarchies ppenum = null;
            Guid empty = Guid.Empty;
            ErrorHandler.ThrowOnFailure(globalService.GetProjectEnum((uint) projectFlags, ref empty, out ppenum));
            IVsHierarchy[] rgelt = new IVsHierarchy[1];
            uint pceltFetched = 0;
            int hr = 0;
        Label_PostSwitchInIterator:;
            if (ErrorHandler.Succeeded(hr = ppenum.Next((uint) rgelt.Length, rgelt, out pceltFetched)) && (pceltFetched > 0))
            {
                yield return rgelt[0];
                goto Label_PostSwitchInIterator;
            }
            ErrorHandler.ThrowOnFailure(hr);
        Label_00F9:;
        }

        internal static Guid GetAssemblyGuid(EnvDTE.Project project)
        {
            try
            {
                string str = project.Properties.Item("AssemblyGuid").Value as string;
                if (string.IsNullOrEmpty(str))
                {
                    return Guid.Empty;
                }
                return new Guid(str);
            }
            catch (FormatException)
            {
                return Guid.Empty;
            }
        }

        internal static EnvDTE.Project GetDteProject(IVsHierarchy hierarchy)
        {
            object obj2;
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(0xfffffffe, -2027, out obj2));
            if (obj2 == null)
            {
                throw new InvalidOperationException(XnaProjectResource.NoProjectAutomation);
            }
            return (obj2 as EnvDTE.Project);
        }

        internal static string GetFullPathFromProjectRelativePath(IVsHierarchy hierarchy, string relativePath)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            if (relativePath == null)
            {
                throw new ArgumentNullException("relativePath");
            }
            string fullPath = null;
            object pvar = null;
            ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(0xfffffffe, -2021, out pvar));
            string str2 = (string) pvar;
            Exception innerException = null;
            try
            {
                fullPath = Path.GetFullPath(Path.Combine(str2, relativePath));
            }
            catch (ArgumentException exception2)
            {
                innerException = exception2;
            }
            catch (NotSupportedException exception3)
            {
                innerException = exception3;
            }
            catch (PathTooLongException exception4)
            {
                innerException = exception4;
            }
            if (innerException != null)
            {
                throw new ArgumentException(XnaProjectResource.InvalidItemPath, "relativePath", innerException);
            }
            return fullPath;
        }

        internal static T GetHierarchyObject<T>(IHierarchyObjectProvider hierarchy) where T: class
        {
            if (hierarchy == null)
            {
                return default(T);
            }
            return (hierarchy.GetObject(typeof(T)) as T);
        }

        public static ImageList GetImageList(object imageListAsPointer)
        {
            if (imageListAsPointer == null)
            {
                throw new ArgumentNullException("imageListAsPointer");
            }
            ImageList list = new ImageList {
                ColorDepth = ColorDepth.Depth24Bit,
                ImageSize = new Size(0x10, 0x10)
            };
            IntPtr handle = new IntPtr((int) imageListAsPointer);
            HandleRef himl = new HandleRef(null, handle);
            int num = ImageList_GetImageCount(himl);
            if (num > 0)
            {
                Bitmap image = new Bitmap(0x10 * num, 0x10);
                Graphics graphics = Graphics.FromImage(image);
                IntPtr zero = IntPtr.Zero;
                try
                {
                    zero = graphics.GetHdc();
                    HandleRef hdcDst = new HandleRef(null, zero);
                    for (int i = 0; i < num; i++)
                    {
                        ImageList_Draw(himl, i, hdcDst, i * 0x10, 0, 0);
                    }
                }
                finally
                {
                    if ((graphics != null) && (zero != IntPtr.Zero))
                    {
                        graphics.ReleaseHdc(zero);
                    }
                }
                list.Images.AddStrip(image);
            }
            return list;
        }

        internal static IEnumerable<Microsoft.Build.Evaluation.ProjectItem> GetMsBuildItemsByName(IVsProject project, string itemGroupName)
        {
            if (itemGroupName == null)
            {
                throw new ArgumentNullException("itemGroupName");
            }
            if (project != null)
            {
                Microsoft.Build.Evaluation.Project msBuildProject = GetMsBuildProject(project);
                if (msBuildProject != null)
                {
                    return msBuildProject.GetItems(itemGroupName);
                }
            }
            return null;
        }

        internal static Microsoft.Build.Evaluation.Project GetMsBuildProject(IVsProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string pbstrMkDocument = null;
            ErrorHandler.ThrowOnFailure(project.GetMkDocument(0xfffffffe, out pbstrMkDocument));
            using (IEnumerator<Microsoft.Build.Evaluation.Project> enumerator = ProjectCollection.GlobalProjectCollection.GetLoadedProjects(pbstrMkDocument).GetEnumerator())
            {
                while (enumerator.MoveNext())
                {
                    return enumerator.Current;
                }
            }
            return ProjectCollection.GlobalProjectCollection.LoadProject(pbstrMkDocument);
        }

        private static IVsHierarchy GetOwningHierarchy(object contextObject)
        {
            IVsBrowseObject obj2 = contextObject as IVsBrowseObject;
            if (obj2 != null)
            {
                IVsHierarchy pHier = null;
                uint maxValue = uint.MaxValue;
                if (ErrorHandler.Succeeded(obj2.GetProjectItem(out pHier, out maxValue)))
                {
                    return pHier;
                }
            }
            return null;
        }

        internal static ContentPipelineManager GetPipelineManagerFromContext(ITypeDescriptorContext context)
        {
            ContentPipelineManager objA = null;
            if (context == null)
            {
                return objA;
            }
            Array instance = context.Instance as Array;
            if (instance != null)
            {
                foreach (object obj2 in instance)
                {
                    ContentPipelineManager pipelineManagerFromContext = GetPipelineManagerFromContext(obj2);
                    if (objA == null)
                    {
                        objA = pipelineManagerFromContext;
                    }
                    if (pipelineManagerFromContext == null)
                    {
                        return null;
                    }
                    if (!object.ReferenceEquals(objA, pipelineManagerFromContext))
                    {
                        return null;
                    }
                }
                return objA;
            }
            return GetPipelineManagerFromContext(context.Instance);
        }

        internal static ContentPipelineManager GetPipelineManagerFromContext(object contextObject)
        {
            IVsHierarchy owningHierarchy = GetOwningHierarchy(contextObject);
            if (owningHierarchy != null)
            {
                return GetHierarchyObject<ContentPipelineManager>(owningHierarchy as IHierarchyObjectProvider);
            }
            return null;
        }

        internal static List<string> GetProjectFilePaths(IEnumerable<IVsHierarchy> projects)
        {
            if (projects == null)
            {
                throw new ArgumentNullException("projects");
            }
            List<string> list = new List<string>();
            foreach (IVsProject project in projects)
            {
                string str;
                ErrorHandler.ThrowOnFailure(project.GetMkDocument(0xfffffffe, out str));
                list.Add(str);
            }
            return list;
        }

        internal static string GetProjectName(IVsHierarchy hierarchy)
        {
            object obj2;
            if (hierarchy == null)
            {
                return null;
            }
            ErrorHandler.ThrowOnFailure(hierarchy.GetProperty(0xfffffffe, -2003, out obj2));
            return (obj2 as string);
        }

        internal static uint GetReferenceItemid(IXnaCodeProject codeProject, Reference reference)
        {
            object obj2;
            if (codeProject == null)
            {
                throw new ArgumentNullException("codeProject");
            }
            if (reference == null)
            {
                throw new ArgumentNullException("reference");
            }
            uint maxValue = uint.MaxValue;
            uint itemid = codeProject.HierarchyItemManager.ReferencesFolderNode.Itemid;
            IVsHierarchy innerIVsHierarchy = codeProject.InnerIVsHierarchy;
            if (ErrorHandler.Succeeded(innerIVsHierarchy.GetProperty(itemid, -1001, out obj2)))
            {
                maxValue = (uint) ((int) obj2);
            }
            while (maxValue != uint.MaxValue)
            {
                object obj3;
                if (ErrorHandler.Succeeded(innerIVsHierarchy.GetProperty(maxValue, -2027, out obj3)))
                {
                    Reference objB = obj3 as Reference;
                    if (object.ReferenceEquals(reference, objB))
                    {
                        return maxValue;
                    }
                    ErrorHandler.ThrowOnFailure(innerIVsHierarchy.GetProperty(maxValue, -1002, out obj2));
                    maxValue = (uint) ((int) obj2);
                }
            }
            return maxValue;
        }

        internal static T GetRequiredService<T>(IServiceProvider serviceProvider) where T: class
        {
            return GetRequiredService<T>(serviceProvider, typeof(T));
        }

        internal static T GetRequiredService<T>(IServiceProvider serviceProvider, System.Type serviceType) where T: class
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (serviceType == null)
            {
                throw new ArgumentNullException("serviceType");
            }
            T service = serviceProvider.GetService(serviceType) as T;
            if (service == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(T).FullName }));
            }
            return service;
        }

        public static string GetSilverlightServiceReferenceAssemblyPath()
        {
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Microsoft SDKs\WindowsPhone\v7.0\Install Path"))
            {
                if (key != null)
                {
                    string str = (string) key.GetValue("Install Path");
                    if (!string.IsNullOrEmpty(str))
                    {
                        string path = Path.Combine(str, @"Tools\Microsoft.Silverlight.Phone.ServiceReference.dll");
                        if (File.Exists(path))
                        {
                            return path;
                        }
                    }
                }
            }
            return null;
        }

        internal static string GetTempDirectory()
        {
            bool flag = true;
            string path = "";
            while (flag)
            {
                path = Path.Combine(Path.GetTempPath(), Path.GetRandomFileName());
                flag = Directory.Exists(path) || File.Exists(path);
            }
            return path;
        }

        public static IVsHierarchy GetVsHierarchy(object unknown)
        {
            IVsHierarchy typedObjectForIUnknown;
            IntPtr zero = IntPtr.Zero;
            try
            {
                zero = ComUtilities.QueryInterfaceIUnknown(unknown);
                typedObjectForIUnknown = (IVsHierarchy) Marshal.GetTypedObjectForIUnknown(zero, typeof(IVsHierarchy));
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.Release(zero);
                }
            }
            return typedObjectForIUnknown;
        }

        internal static VSProject GetVSProject(IVsHierarchy hierarchy)
        {
            EnvDTE.Project dteProject = GetDteProject(hierarchy);
            if (dteProject != null)
            {
                return (dteProject.Object as VSProject);
            }
            return null;
        }

        [DllImport("comctl32.dll", CharSet=CharSet.Auto)]
        private static extern bool ImageList_Draw(HandleRef himl, int i, HandleRef hdcDst, int x, int y, int fStyle);
        [DllImport("comctl32.dll", CharSet=CharSet.Auto)]
        private static extern int ImageList_GetImageCount(HandleRef himl);
        internal static bool IsDesignMode()
        {
            bool flag = false;
            bool flag2 = false;
            bool flag3 = false;
            IVsMonitorSelection selectionService = null;
            selectionService = Package.GetGlobalService(typeof(SVsShellMonitorSelection)) as IVsMonitorSelection;
            if (selectionService != null)
            {
                flag = IsUIContextActive(selectionService, VSConstants.UICONTEXT_DesignMode);
                flag2 = IsUIContextActive(selectionService, VSConstants.UICONTEXT_SolutionBuilding);
                flag3 = IsUIContextActive(selectionService, VSConstants.UICONTEXT_Debugging);
            }
            return ((flag && !flag2) && !flag3);
        }

        internal static bool IsInAutomationFunction(IServiceProvider serviceProvider)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            IVsExtensibility3 requiredService = GetRequiredService<IVsExtensibility3>(serviceProvider, typeof(IVsExtensibility));
            int pfInAutoFunc = 0;
            ErrorHandler.ThrowOnFailure(requiredService.IsInAutomationFunction(out pfInAutoFunc));
            return (pfInAutoFunc != 0);
        }

        internal static bool IsNestedProjectNode(IVsHierarchy hierarchy, uint itemid)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            Guid empty = Guid.Empty;
            return (ErrorHandler.Succeeded(hierarchy.GetGuidProperty(itemid, -1004, out empty)) && (empty == VSConstants.GUID_ItemType_SubProject));
        }

        private static bool IsUIContextActive(IVsMonitorSelection selectionService, Guid uiContextGuid)
        {
            if (selectionService == null)
            {
                throw new ArgumentNullException("selectionService");
            }
            int pfActive = 0;
            uint pdwCmdUICookie = 0;
            ErrorHandler.ThrowOnFailure(selectionService.GetCmdUIContextCookie(ref uiContextGuid, out pdwCmdUICookie));
            ErrorHandler.ThrowOnFailure(selectionService.IsCmdUIContextActive(pdwCmdUICookie, out pfActive));
            return (pfActive != 0);
        }

        internal static void QueryEditQuerySaveProjectFile(IVsProject project)
        {
            IVsQueryEditQuerySave2 globalService = Package.GetGlobalService(typeof(SVsQueryEditQuerySave)) as IVsQueryEditQuerySave2;
            if (globalService != null)
            {
                string pbstrMkDocument = null;
                ErrorHandler.ThrowOnFailure(project.GetMkDocument(0xfffffffe, out pbstrMkDocument));
                string[] rgpszMkDocuments = new string[] { pbstrMkDocument };
                uint pfEditVerdict = 0;
                uint prgfMoreInfo = 0;
                ErrorHandler.ThrowOnFailure(globalService.QueryEditFiles(0, rgpszMkDocuments.Length, rgpszMkDocuments, new uint[rgpszMkDocuments.Length], new VSQEQS_FILE_ATTRIBUTE_DATA[rgpszMkDocuments.Length], out pfEditVerdict, out prgfMoreInfo));
                if (pfEditVerdict != 0)
                {
                    throw CheckoutException.Canceled;
                }
            }
        }

        internal static int ShowErrorMessage(IVsUIShell uiShell, string title, string text, string helpContext)
        {
            int num;
            if (uiShell == null)
            {
                throw new ArgumentNullException("uiShell");
            }
            Guid empty = Guid.Empty;
            uiShell.ShowMessageBox(0, ref empty, title, text, helpContext, 0, OLEMSGBUTTON.OLEMSGBUTTON_OK, OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST, OLEMSGICON.OLEMSGICON_CRITICAL, 0, out num);
            return num;
        }

        public static void ShowHelp(string keyword)
        {
            Help2 globalService = Package.GetGlobalService(typeof(SVsHelp)) as Help2;
            if (globalService != null)
            {
                globalService.DisplayTopicFromF1Keyword(keyword);
            }
        }


    }
}

