﻿namespace Microsoft.Xna.GameStudio
{
    using EnvDTE;
    using EnvDTE80;
    using Microsoft.Build.Evaluation;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.ContentProject;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using Microsoft.Xna.GameStudio.Wizards;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Xml;
    using VSLangProj;

    internal class SolutionEventsListener : IVsSolutionEvents, IVsSolutionEventsProjectUpgrade, IDisposable
    {
        internal EventHandler<SolutionEventArgs> AfterLoadProject;
        internal EventHandler<SolutionEventArgs> AfterOpenProject;
        private static readonly int[] allowedUpgradeFailures = new int[] { -2147170504 };
        internal EventHandler<SolutionEventArgs> BeforeCloseProject;
        internal EventHandler<SolutionEventArgs> BeforeUnloadProject;
        private static readonly Version currentVersion = new Version("4.0.0.0");
        private uint eventsCookie;
        private IServiceProvider serviceProvider;
        private IVsSolution solution;

        internal SolutionEventsListener(IServiceProvider serviceProvider)
        {
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            this.serviceProvider = serviceProvider;
            this.solution = (IVsSolution) serviceProvider.GetService(typeof(IVsSolution));
            ErrorHandler.ThrowOnFailure(this.solution.AdviseSolutionEvents(this, out this.eventsCookie));
        }

        private void AddReferenceCleanMetadata(VSProject langProj, IXnaCodeProject codeProject, bool requireWeakReference, string reference)
        {
            if (string.IsNullOrEmpty(reference))
            {
                throw new ArgumentNullException("reference");
            }
            if (langProj == null)
            {
                throw new ArgumentNullException("langProj");
            }
            if (requireWeakReference)
            {
                reference = "*" + reference;
            }
            Reference reference2 = langProj.References.Add(reference);
            if ((reference2 != null) && (codeProject != null))
            {
                uint referenceItemid = Microsoft.Xna.GameStudio.ProjectUtilities.GetReferenceItemid(codeProject, reference2);
                if (referenceItemid != uint.MaxValue)
                {
                    ErrorHandler.ThrowOnFailure(codeProject.InnerVsBuildPropertyStorage.SetItemAttribute(referenceItemid, "RequiredTargetFramework", null));
                    ErrorHandler.ThrowOnFailure(codeProject.InnerVsBuildPropertyStorage.SetItemAttribute(referenceItemid, "Private", "False"));
                }
            }
        }

        private static IXnaCodeProject AsCodeProject(IVsHierarchy hierarchy)
        {
            return Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(hierarchy as IHierarchyObjectProvider);
        }

        private static IXnaContentProject AsContentProject(IVsHierarchy hierarchy)
        {
            return Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaContentProject>(hierarchy as IHierarchyObjectProvider);
        }

        private bool BackupContentProject(string solutionRootPath, string contentProjRelPath, string backupRoot, IVsUpgradeLogger upgradeLogger)
        {
            DirectoryInfo destinationInfo = new DirectoryInfo(Path.GetFullPath(Path.Combine(backupRoot, Path.GetDirectoryName(contentProjRelPath))));
            if (destinationInfo.Exists)
            {
                return false;
            }
            DirectoryInfo sourceInfo = new DirectoryInfo(solutionRootPath + Path.GetDirectoryName(contentProjRelPath));
            this.RecursiveCopy(sourceInfo, destinationInfo, upgradeLogger, Path.GetFileName(contentProjRelPath));
            return true;
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (disposing)
            {
                if ((this.solution != null) && (this.eventsCookie != 0))
                {
                    this.solution.UnadviseSolutionEvents(this.eventsCookie);
                    this.eventsCookie = 0;
                }
                this.solution = null;
            }
        }

        ~SolutionEventsListener()
        {
            this.Dispose(false);
        }

        private static bool HasXnaFrameworkVersion(IVsHierarchy hierarchy)
        {
            string str;
            IVsBuildPropertyStorage storage = hierarchy as IVsBuildPropertyStorage;
            if (storage == null)
            {
                return false;
            }
            return (ErrorHandler.Succeeded(storage.GetPropertyValue("XnaFrameworkVersion", null, 1, out str)) && !string.IsNullOrEmpty(str));
        }

        public int OnAfterCloseSolution(object pUnkReserved)
        {
            return 0;
        }

        public int OnAfterLoadProject(IVsHierarchy pStubHierarchy, IVsHierarchy pRealHierarchy)
        {
            if (this.AfterLoadProject != null)
            {
                this.AfterLoadProject(this, new SolutionEventArgs(pRealHierarchy));
            }
            return 0;
        }

        public int OnAfterOpenProject(IVsHierarchy pHierarchy, int fAdded)
        {
            if (this.AfterOpenProject != null)
            {
                this.AfterOpenProject(this, new SolutionEventArgs(pHierarchy));
            }
            OnProjectAdded(pHierarchy);
            return 0;
        }

        public int OnAfterOpenSolution(object pUnkReserved, int fNewSolution)
        {
            return 0;
        }

        public int OnAfterUpgradeProject(IVsHierarchy pHierarchy, uint fUpgradeFlag, string bstrCopyLocation, SYSTEMTIME stUpgradeTime, IVsUpgradeLogger pLogger)
        {
            if (pHierarchy == null)
            {
                throw new ArgumentNullException("pHierarchy");
            }
            if (pLogger == null)
            {
                throw new ArgumentNullException("pLogger");
            }
            VSProject vSProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(pHierarchy);
            if (vSProject == null)
            {
                return -2147213339;
            }
            bool isProjectUpgradeInProgress = XnaPackage.Instance.IsProjectUpgradeInProgress;
            try
            {
                if (XnaPackage.Instance != null)
                {
                    XnaPackage.Instance.IsProjectUpgradeInProgress = true;
                }
                IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(pHierarchy as IHierarchyObjectProvider);
                IVsBuildPropertyStorage props = pHierarchy as IVsBuildPropertyStorage;
                if ((props == null) && (hierarchyObject != null))
                {
                    props = hierarchyObject.InnerVsBuildPropertyStorage;
                }
                string fileName = vSProject.Project.FileName;
                string name = vSProject.Project.Name;
                bool flag2 = false;
                string pbstrPropValue = string.Empty;
                if (hierarchyObject != null)
                {
                    string str4;
                    string str5;
                    ErrorHandler.ThrowOnFailure(hierarchyObject.InnerVsBuildPropertyStorage.GetPropertyValue("XnaPlatform", null, 1, out str4), allowedUpgradeFailures);
                    if (!string.IsNullOrEmpty(str4) && str4.Equals("Zune", StringComparison.InvariantCultureIgnoreCase))
                    {
                        if (!string.IsNullOrEmpty(vSProject.Project.DTE.Solution.FullName) && (vSProject.Project.DTE.Solution.Projects.Count > 1))
                        {
                            vSProject.Project.DTE.Solution.Remove(vSProject.Project);
                        }
                        return 0;
                    }
                    ErrorHandler.ThrowOnFailure(hierarchyObject.InnerVsBuildPropertyStorage.GetPropertyValue("XnaUpgrade", null, 1, out str5), allowedUpgradeFailures);
                    flag2 = str5 == "1";
                    ErrorHandler.ThrowOnFailure(hierarchyObject.InnerVsBuildPropertyStorage.GetPropertyValue("XnaProfile", null, 1, out pbstrPropValue), allowedUpgradeFailures);
                }
                bool flag3 = false;
                if (props != null)
                {
                    string str6;
                    flag3 = (props.GetPropertyValue("XnaFrameworkVersion", null, 1, out str6) == 0) && ((str6 == "v3.0") || (str6 == "v3.1"));
                    if (flag3)
                    {
                        ErrorHandler.ThrowOnFailure(pLogger.LogMessage(0, name, fileName, XnaProjectResource.UpgradedContentPipelineReferences));
                        ErrorHandler.ThrowOnFailure(props.SetPropertyValue("XnaFrameworkVersion", null, 1, "v4.0"));
                        ErrorHandler.ThrowOnFailure(props.SetPropertyValue("TargetFrameworkVersion", null, 1, "v4.0"));
                    }
                }
                if (flag3 || flag2)
                {
                    bool requireWeakReference = (hierarchyObject != null) && (hierarchyObject.Adapter.XnaPlatform != "Windows");
                    if (flag2)
                    {
                        this.AddReferenceCleanMetadata(vSProject, hierarchyObject, false, "System.Core");
                        this.AddReferenceCleanMetadata(vSProject, hierarchyObject, false, "System.Xml.Linq");
                        this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Graphics");
                        this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.GamerServices");
                        this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Input.Touch");
                        if (!string.IsNullOrEmpty(pbstrPropValue) && pbstrPropValue.Equals("HiDef"))
                        {
                            this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Avatar");
                            this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Net");
                            this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Storage");
                            this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Video");
                            this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, "Microsoft.Xna.Framework.Xact");
                        }
                    }
                    List<Reference> list = new List<Reference>();
                    foreach (Reference reference in vSProject.References)
                    {
                        Version version = new Version(reference.Version);
                        if (reference.Name.StartsWith("Microsoft.Xna") && (version < currentVersion))
                        {
                            list.Add(reference);
                        }
                    }
                    foreach (Reference reference2 in list)
                    {
                        string identity = null;
                        if (hierarchyObject != null)
                        {
                            string[] sdkPaths = hierarchyObject.SdkPaths;
                            if (sdkPaths.Length == 0)
                            {
                                sdkPaths = SdkPaths.GetAllSdkPaths(SdkPathGroup.Windows, hierarchyObject.Adapter.XnaFrameworkVersion, null);
                            }
                            foreach (string str8 in sdkPaths)
                            {
                                string path = Path.Combine(str8, reference2.Identity) + ".dll";
                                if (File.Exists(path))
                                {
                                    identity = path;
                                    break;
                                }
                            }
                        }
                        if (string.IsNullOrEmpty(identity))
                        {
                            if (!requireWeakReference)
                            {
                                if (reference2.Name.StartsWith("Microsoft.Xna.Framework") && !reference2.Name.StartsWith("Microsoft.Xna.Framework.Tools"))
                                {
                                    identity = string.Format(CultureInfo.InvariantCulture, "{0}, Version=4.0.0.0, Culture=neutral, PublicKeyToken={1}", new object[] { reference2.Identity, "842cf8be1de50553" });
                                }
                                else
                                {
                                    identity = string.Format(CultureInfo.InvariantCulture, "{0}, Version=4.0.0.0, Culture=neutral, PublicKeyToken={1}", new object[] { reference2.Identity, "6d5c3888ef60e27d" });
                                }
                            }
                            else
                            {
                                identity = reference2.Identity;
                            }
                        }
                        reference2.Remove();
                        try
                        {
                            this.AddReferenceCleanMetadata(vSProject, hierarchyObject, requireWeakReference, identity);
                        }
                        catch (COMException)
                        {
                            ErrorHandler.ThrowOnFailure(pLogger.LogMessage(2, name, fileName, XnaProjectResource.UpgradeFailedReplaceReference));
                        }
                        vSProject.Refresh();
                    }
                    ErrorHandler.ThrowOnFailure(pLogger.LogMessage(0, name, fileName, XnaProjectResource.UpgradedFrameworkReferences));
                }
                if (flag2)
                {
                    if (hierarchyObject.Adapter.XnaPlatform == "Windows")
                    {
                        ProjectWizard.AddDependentBootstrapperPackages(vSProject.Project);
                    }
                    UpdateConfigurationProperties(hierarchyObject, props, vSProject);
                    XmlDocument document = new XmlDocument();
                    document.Load(fileName);
                    string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(fileName);
                    XmlNamespaceManager nsmgr = new XmlNamespaceManager(document.NameTable);
                    nsmgr.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
                    foreach (XmlNode node in document.SelectNodes("//msbuild:NestedContentProject[@Include]", nsmgr))
                    {
                        XmlNode node4;
                        IntPtr zero = IntPtr.Zero;
                        string str11 = node.Attributes["Include"].Value;
                        string fullPath = Path.GetFullPath(Path.Combine(Path.GetDirectoryName(fileName), str11));
                        if (!string.IsNullOrEmpty(bstrCopyLocation))
                        {
                            string solutionDir = hierarchyObject.GetSolutionDir();
                            if (!fullPath.StartsWith(solutionDir, StringComparison.OrdinalIgnoreCase))
                            {
                                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(1, fileNameWithoutExtension, fullPath, string.Format(XnaProjectResource.UpgradedContentOutsideSolution, fullPath)));
                                continue;
                            }
                            string str14 = Path.GetFullPath(fileName).Substring(solutionDir.Length);
                            string contentProjRelPath = fullPath.Substring(solutionDir.Length);
                            string backupRoot = bstrCopyLocation;
                            string directoryName = Path.GetDirectoryName(str14);
                            if (!string.IsNullOrWhiteSpace(directoryName))
                            {
                                int length = bstrCopyLocation.LastIndexOf(directoryName, StringComparison.OrdinalIgnoreCase);
                                if (length >= 0)
                                {
                                    backupRoot = bstrCopyLocation.Substring(0, length);
                                }
                            }
                            this.BackupContentProject(solutionDir, contentProjRelPath, backupRoot, pLogger);
                        }
                        if (!hierarchyObject.IsEntryProject)
                        {
                            XmlDocument document2 = new XmlDocument();
                            document2.Load(fullPath);
                            XmlNamespaceManager manager2 = new XmlNamespaceManager(document2.NameTable);
                            manager2.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
                            if (document2.SelectSingleNode("//msbuild:Compile", manager2) == null)
                            {
                                pLogger.LogMessage(0, fileNameWithoutExtension, str11, string.Format(XnaProjectResource.UpgradedLibraryRemovedEmptyContentReference, fileNameWithoutExtension, fullPath));
                                continue;
                            }
                        }
                        string str18 = Path.GetFullPath(Path.Combine(Path.GetDirectoryName(fullPath), Path.GetFileName(Path.GetDirectoryName(Path.GetDirectoryName(fullPath))) + Path.GetFileName(fullPath)));
                        if (!File.Exists(str18))
                        {
                            if (!File.Exists(fullPath))
                            {
                                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(1, fileNameWithoutExtension, str11, XnaProjectResource.UpgradedContentProjectMissing));
                                continue;
                            }
                            File.Move(fullPath, str18);
                        }
                        fullPath = str18;
                        XmlDocument document3 = new XmlDocument();
                        document3.Load(fullPath);
                        XmlNamespaceManager manager3 = new XmlNamespaceManager(document3.NameTable);
                        manager3.AddNamespace("msbuild", "http://schemas.microsoft.com/developer/msbuild/2003");
                        Guid empty = Guid.Empty;
                        XmlNode node3 = document3.SelectSingleNode("//msbuild:ProjectGuid", manager3);
                        if ((node3 != null) && !string.IsNullOrEmpty(node3.InnerText))
                        {
                            if (Guid.TryParse(node3.InnerText, ref empty))
                            {
                                goto Label_07CB;
                            }
                            ErrorHandler.ThrowOnFailure(pLogger.LogMessage(2, fileNameWithoutExtension, str11, XnaProjectResource.UpgradeNestedContentFailure));
                        }
                        else
                        {
                            ErrorHandler.ThrowOnFailure(pLogger.LogMessage(2, fileNameWithoutExtension, str11, XnaProjectResource.UpgradeNestedContentFailure));
                        }
                        continue;
                    Label_07CB:
                        node4 = node.SelectSingleNode("msbuild:Project", nsmgr);
                        if ((node4 != null) && !string.IsNullOrEmpty(node4.InnerText))
                        {
                            Guid guid2 = Guid.Empty;
                            if (Guid.TryParse(node4.InnerText, ref guid2) && !guid2.Equals(empty))
                            {
                                ErrorHandler.ThrowOnFailure(pLogger.LogMessage(2, fileNameWithoutExtension, str11, string.Format(XnaProjectResource.UpgradeFailedAddingContentReference, fullPath, str11)));
                                continue;
                            }
                        }
                        IVsHierarchy ppHierarchy = null;
                        this.solution.GetProjectOfGuid(ref empty, out ppHierarchy);
                        if (ppHierarchy == null)
                        {
                            Guid iidProject = VSConstants.IID_IUnknown;
                            Guid rguidProjectType = Guid.Empty;
                            this.solution.CreateProject(ref rguidProjectType, str18, null, null, 2, ref iidProject, out zero);
                            ppHierarchy = (IVsHierarchy) Marshal.GetObjectForIUnknown(zero);
                        }
                        Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(ppHierarchy);
                        Reference reference3 = vSProject.References.AddProject(Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(ppHierarchy).Project);
                        if (reference3 != null)
                        {
                            uint referenceItemid = Microsoft.Xna.GameStudio.ProjectUtilities.GetReferenceItemid(hierarchyObject, reference3);
                            if (referenceItemid != uint.MaxValue)
                            {
                                ErrorHandler.ThrowOnFailure(hierarchyObject.InnerVsBuildPropertyStorage.SetItemAttribute(referenceItemid, "XnaReferenceType", "Content"));
                                HierarchyItemManager hierarchyItemManager = hierarchyObject.HierarchyItemManager;
                                bool flag5 = hierarchyItemManager.IsContentReferencesFolderNodeVisible();
                                hierarchyItemManager.OnHierarchyChanged();
                                IVsHierarchyEvents vsHierarchy = hierarchyObject.GetVsHierarchy() as IVsHierarchyEvents;
                                if (vsHierarchy != null)
                                {
                                    vsHierarchy.OnInvalidateItems(hierarchyItemManager.ReferencesFolderNode.Itemid);
                                    ContentReferencesFolderNode contentReferencesFolderNode = hierarchyItemManager.ContentReferencesFolderNode;
                                    if (flag5)
                                    {
                                        ContentReferenceNode managedItem = hierarchyItemManager.GetManagedItem(referenceItemid) as ContentReferenceNode;
                                        if (managedItem != null)
                                        {
                                            vsHierarchy.OnItemAdded(contentReferencesFolderNode.GetItemId(), managedItem.PrevContentReferenceItem, referenceItemid);
                                        }
                                    }
                                    else
                                    {
                                        uint itemidSiblingPrev = hierarchyItemManager.IsNestedProjectFirstChildOfRoot ? uint.MaxValue : hierarchyItemManager.ItemBeforeContentReferencesFolder;
                                        vsHierarchy.OnItemAdded(0xfffffffe, itemidSiblingPrev, contentReferencesFolderNode.GetItemId());
                                    }
                                }
                            }
                        }
                    }
                }
                if (hierarchyObject != null)
                {
                    ErrorHandler.ThrowOnFailure(hierarchyObject.InnerVsBuildPropertyStorage.RemoveProperty("XnaUpgrade", null, 1));
                }
            }
            finally
            {
                if (XnaPackage.Instance != null)
                {
                    XnaPackage.Instance.IsProjectUpgradeInProgress = isProjectUpgradeInProgress;
                }
            }
            return 0;
        }

        public int OnBeforeCloseProject(IVsHierarchy pHierarchy, int fRemoved)
        {
            if (this.BeforeCloseProject != null)
            {
                this.BeforeCloseProject(this, new SolutionEventArgs(pHierarchy));
            }
            return 0;
        }

        public int OnBeforeCloseSolution(object pUnkReserved)
        {
            return 0;
        }

        public int OnBeforeUnloadProject(IVsHierarchy pRealHierarchy, IVsHierarchy pStubHierarchy)
        {
            if (this.BeforeUnloadProject != null)
            {
                this.BeforeUnloadProject(this, new SolutionEventArgs(pRealHierarchy));
            }
            return 0;
        }

        private static void OnContentPipelineExtensionProjectAdded(EnvDTE.Project newProject, SolutionConfigurations configurations)
        {
            foreach (SolutionConfiguration configuration in configurations)
            {
                foreach (SolutionContext context in configuration.SolutionContexts)
                {
                    if (context.ProjectName == newProject.UniqueName)
                    {
                        context.ShouldBuild = true;
                    }
                }
            }
        }

        private static void OnProjectAdded(IVsHierarchy hierarchy)
        {
            if (HasXnaFrameworkVersion(hierarchy))
            {
                EnvDTE.Project dteProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetDteProject(hierarchy);
                if (dteProject != null)
                {
                    SolutionConfigurations solutionConfigurations = dteProject.DTE.Solution.SolutionBuild.SolutionConfigurations;
                    IXnaCodeProject newCodeProject = AsCodeProject(hierarchy);
                    if (newCodeProject != null)
                    {
                        OnXnaCodeProjectAdded(newCodeProject, solutionConfigurations);
                    }
                    else if (AsContentProject(hierarchy) == null)
                    {
                        OnContentPipelineExtensionProjectAdded(dteProject, solutionConfigurations);
                    }
                }
            }
        }

        public int OnQueryCloseProject(IVsHierarchy pHierarchy, int fRemoving, ref int pfCancel)
        {
            return 0;
        }

        public int OnQueryCloseSolution(object pUnkReserved, ref int pfCancel)
        {
            return 0;
        }

        public int OnQueryUnloadProject(IVsHierarchy pRealHierarchy, ref int pfCancel)
        {
            return 0;
        }

        private static void OnXnaCodeProjectAdded(IXnaCodeProject newCodeProject, SolutionConfigurations configurations)
        {
            List<string> list = new List<string>();
            string platformConfigurationName = newCodeProject.Adapter.PlatformConfigurationName;
            int num = 0;
            foreach (IVsHierarchy hierarchy in Microsoft.Xna.GameStudio.ProjectUtilities.EnumerateProjects(__VSENUMPROJFLAGS.EPF_LOADEDINSOLUTION))
            {
                if (HasXnaFrameworkVersion(hierarchy))
                {
                    IXnaCodeProject project = AsCodeProject(hierarchy);
                    if ((project != null) && (project.Adapter.PlatformConfigurationName == platformConfigurationName))
                    {
                        num++;
                        if (num <= 1)
                        {
                            continue;
                        }
                        return;
                    }
                    if (AsContentProject(hierarchy) == null)
                    {
                        EnvDTE.Project dteProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetDteProject(hierarchy);
                        if (dteProject != null)
                        {
                            string uniqueName = dteProject.UniqueName;
                            list.Add(uniqueName);
                        }
                    }
                }
            }
            foreach (SolutionConfiguration2 configuration in configurations)
            {
                if (configuration.PlatformName == platformConfigurationName)
                {
                    foreach (SolutionContext context in configuration.SolutionContexts)
                    {
                        if (list.Contains(context.ProjectName))
                        {
                            context.ShouldBuild = true;
                        }
                    }
                }
            }
        }

        private void RecursiveCopy(DirectoryInfo sourceInfo, DirectoryInfo destinationInfo, IVsUpgradeLogger pLogger, string contentProjName)
        {
            if (sourceInfo.Exists)
            {
                if (!destinationInfo.Exists)
                {
                    destinationInfo.Create();
                }
                foreach (DirectoryInfo info in sourceInfo.GetDirectories())
                {
                    if ((!info.Name.Equals("obj", StringComparison.OrdinalIgnoreCase) && !info.Name.Equals("bin", StringComparison.OrdinalIgnoreCase)) && !info.FullName.Equals(destinationInfo.FullName, StringComparison.OrdinalIgnoreCase))
                    {
                        DirectoryInfo info2 = new DirectoryInfo(Path.Combine(destinationInfo.FullName, info.Name));
                        this.RecursiveCopy(info, info2, pLogger, contentProjName);
                    }
                }
                foreach (FileInfo info3 in sourceInfo.GetFiles())
                {
                    string destFileName = Path.Combine(destinationInfo.FullName, info3.Name);
                    info3.CopyTo(destFileName, true);
                    ErrorHandler.ThrowOnFailure(pLogger.LogMessage(0, contentProjName, info3.Name, XnaProjectResource.UpgradedContent));
                }
            }
        }

        private static void UpdateConfigurationProperties(IXnaCodeProject xnaProject, IVsBuildPropertyStorage props, VSProject langProj)
        {
            if (xnaProject == null)
            {
                throw new ArgumentNullException("xnaProject");
            }
            if (props == null)
            {
                throw new ArgumentNullException("props");
            }
            bool flag = false;
            IVsHierarchy ppHier = null;
            uint pitemid = 0;
            IntPtr zero = IntPtr.Zero;
            uint pdwCookie = 0;
            int hr = 0;
            Microsoft.Build.Evaluation.Project msBuildProject = Microsoft.Xna.GameStudio.ProjectUtilities.GetMsBuildProject(xnaProject.InnerVsProject);
            IVsRunningDocumentTable vsService = (IVsRunningDocumentTable) xnaProject.GetVsService(typeof(SVsRunningDocumentTable));
            ErrorHandler.ThrowOnFailure(vsService.FindAndLockDocument(0x2002, msBuildProject.FullPath, out ppHier, out pitemid, out zero, out pdwCookie));
            try
            {
                hr = ErrorHandler.ThrowOnFailure(vsService.NotifyOnBeforeSave(pdwCookie));
                List<string> list = msBuildProject.ConditionedProperties["Platform"];
                List<string> list2 = msBuildProject.ConditionedProperties["Configuration"];
                if ((list != null) && (list2 != null))
                {
                    foreach (string str in list)
                    {
                        foreach (string str2 in list2)
                        {
                            string configurationName = str2 + "|" + str;
                            switch (str)
                            {
                                case "x86":
                                {
                                    string buildProperty = xnaProject.GetBuildProperty("DefineConstants", configurationName);
                                    if (!buildProperty.Contains("WINDOWS"))
                                    {
                                        if (buildProperty.Length > 0)
                                        {
                                            buildProperty = buildProperty + ";";
                                        }
                                        buildProperty = buildProperty + "WINDOWS";
                                        ErrorHandler.ThrowOnFailure(props.SetPropertyValue("DefineConstants", configurationName, 1, buildProperty));
                                        flag = true;
                                        EnvDTE.Configuration activeConfiguration = langProj.Project.ConfigurationManager.ActiveConfiguration;
                                        if (((activeConfiguration != null) && (activeConfiguration.PlatformName == str)) && (activeConfiguration.ConfigurationName == str2))
                                        {
                                            activeConfiguration.Properties.Item("DefineConstants").Value = buildProperty;
                                        }
                                    }
                                    if (str2 == "Release")
                                    {
                                        ErrorHandler.ThrowOnFailure(props.SetPropertyValue("XnaCompressContent", configurationName, 1, bool.TrueString));
                                        flag = true;
                                    }
                                    break;
                                }
                                case "Xbox 360":
                                    ErrorHandler.ThrowOnFailure(props.SetPropertyValue("XnaCompressContent", configurationName, 1, bool.TrueString));
                                    flag = true;
                                    break;
                            }
                        }
                    }
                    if (flag)
                    {
                        msBuildProject.MarkDirty();
                        hr = vsService.NotifyDocumentChanged(pdwCookie, 0x10000);
                    }
                }
            }
            finally
            {
                ComUtilities.SmartRelease(ref zero);
                ErrorHandler.ThrowOnFailure(vsService.UnlockDocument(2, pdwCookie));
                ErrorHandler.ThrowOnFailure(hr);
            }
        }

        internal class SolutionEventArgs : EventArgs
        {
            private IVsHierarchy hierarchy;

            internal SolutionEventArgs(IVsHierarchy hierarchy)
            {
                if (hierarchy == null)
                {
                    throw new ArgumentNullException("hierarchy");
                }
                this.hierarchy = hierarchy;
            }

            internal IVsHierarchy Hierarchy
            {
                get
                {
                    return this.hierarchy;
                }
            }
        }
    }
}

