﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio.OLE.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using System;
    using System.Collections.Generic;
    using System.Runtime.CompilerServices;

    internal class SyncRenameComponent : IOleComponent
    {
        private Dictionary<string, List<RenameInfo>> directoryRenames;
        private Dictionary<string, List<RenameInfo>> fileRenames;
        private CrossPlatformGroupManager groupManager;

        internal SyncRenameComponent(CrossPlatformGroupManager groupManager)
        {
            if (groupManager == null)
            {
                throw new ArgumentNullException("groupManager");
            }
            this.groupManager = groupManager;
            this.fileRenames = new Dictionary<string, List<RenameInfo>>();
            this.directoryRenames = new Dictionary<string, List<RenameInfo>>();
        }

        public int FContinueMessageLoop(uint uReason, IntPtr pvLoopData, MSG[] pMsgPeeked)
        {
            return 1;
        }

        public int FDoIdle(uint grfidlef)
        {
            if (this.fileRenames.Count > 0)
            {
                this.SyncRenamedProjectItems(this.fileRenames, new SyncItemRename(SyncRenameComponent.SyncRenamedFile));
            }
            if (this.directoryRenames.Count > 0)
            {
                this.SyncRenamedProjectItems(this.directoryRenames, new SyncItemRename(SyncRenameComponent.SyncRenamedDirectory));
            }
            return 0;
        }

        public int FPreTranslateMessage(MSG[] pMsg)
        {
            return 0;
        }

        public int FQueryTerminate(int fPromptUser)
        {
            return 1;
        }

        public int FReserved1(uint dwReserved, uint message, IntPtr wParam, IntPtr lParam)
        {
            return 1;
        }

        public IntPtr HwndGetWindow(uint dwWhich, uint dwReserved)
        {
            return IntPtr.Zero;
        }

        internal void NotifyDirectoryRenamed(string groupID, string sourcePlatform, string oldName, string newName)
        {
            if (!string.IsNullOrEmpty(groupID))
            {
                this.ReportRename(this.directoryRenames, groupID, new RenameInfo(sourcePlatform, oldName, newName));
            }
        }

        internal void NotifyFileRenamed(string groupID, string sourcePlatform, string oldName, string newName)
        {
            if (!string.IsNullOrEmpty(groupID))
            {
                this.ReportRename(this.fileRenames, groupID, new RenameInfo(sourcePlatform, oldName, newName));
            }
        }

        public void OnActivationChange(IOleComponent pic, int fSameComponent, OLECRINFO[] pcrinfo, int fHostIsActivating, OLECHOSTINFO[] pchostinfo, uint dwReserved)
        {
        }

        public void OnAppActivate(int fActive, uint dwOtherThreadID)
        {
        }

        public void OnEnterState(uint uStateID, int fEnter)
        {
        }

        public void OnLoseActivation()
        {
        }

        private void ReportRename(Dictionary<string, List<RenameInfo>> groupRenames, string groupID, RenameInfo renameInfo)
        {
            List<RenameInfo> list;
            if (groupRenames == null)
            {
                throw new ArgumentNullException("groupRenames");
            }
            if (groupID == null)
            {
                throw new ArgumentNullException("groupID");
            }
            if (renameInfo == null)
            {
                throw new ArgumentNullException("renameInfo");
            }
            if (!groupRenames.TryGetValue(groupID, out list))
            {
                list = new List<RenameInfo>();
                groupRenames.Add(groupID, list);
            }
            list.Add(renameInfo);
        }

        private static void SyncRenamedDirectory(IXnaCodeProject project, string oldName, string newName)
        {
            project.SyncDirectoryRename(oldName, newName);
        }

        private static void SyncRenamedFile(IXnaCodeProject project, string oldName, string newName)
        {
            project.SyncFileRename(oldName, newName);
        }

        private void SyncRenamedProjectItems(Dictionary<string, List<RenameInfo>> projectItemRenames, SyncItemRename syncRenamedItem)
        {
            foreach (KeyValuePair<string, List<RenameInfo>> pair in projectItemRenames)
            {
                string key = pair.Key;
                List<RenameInfo> list = pair.Value;
                foreach (KeyValuePair<string, IXnaCodeProject> pair2 in this.groupManager.GetProjects(key))
                {
                    foreach (RenameInfo info in list)
                    {
                        if (pair2.Key != info.SourcePlatform)
                        {
                            syncRenamedItem(pair2.Value, info.OldName, info.NewName);
                        }
                    }
                }
            }
            projectItemRenames.Clear();
        }

        public void Terminate()
        {
        }

        private class RenameInfo
        {
            public readonly string NewName;
            public readonly string OldName;
            public readonly string SourcePlatform;

            internal RenameInfo(string sourcePlatform, string oldName, string newName)
            {
                this.SourcePlatform = sourcePlatform;
                this.OldName = oldName;
                this.NewName = newName;
            }
        }

        private delegate void SyncItemRename(IXnaCodeProject project, string oldName, string newName);
    }
}

