﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.Globalization;
    using System.IO;
    using System.Security.AccessControl;
    using System.Security.Principal;
    using System.Windows.Forms;
    using System.Xml;

    internal class UpgradeSolutionCommand : OleMenuCommand
    {
        private const string capabilitiesRootElementPath = ".//Capabilities";
        private const string capabilityElementName = "Capability";
        private bool isUpgradePossible;
        private const string nameAttributeName = "Name";
        private const string namedCapabilityElementPath = ".//Capabilities/Capability[@Name='{0}']";
        private bool needsRefresh;
        private static readonly List<string> newRefreshCapabilities = new List<string> { "ID_CAP_ISV_CAMERA", "ID_CAP_CONTACTS", "ID_CAP_APPOINTMENTS" };

        internal UpgradeSolutionCommand(SolutionEventsListener solutionEventsListener, CommandID commandID) : base(new EventHandler(UpgradeSolutionCommand.OnUpgradeSolution), commandID)
        {
            if (solutionEventsListener == null)
            {
                throw new ArgumentNullException("solutionEventsListener");
            }
            base.BeforeQueryStatus += new EventHandler(this.OnBeforeQueryStatus);
            solutionEventsListener.AfterLoadProject = (EventHandler<SolutionEventsListener.SolutionEventArgs>) Delegate.Combine(solutionEventsListener.AfterLoadProject, new EventHandler<SolutionEventsListener.SolutionEventArgs>(this.OnProjectAdded));
            solutionEventsListener.AfterOpenProject = (EventHandler<SolutionEventsListener.SolutionEventArgs>) Delegate.Combine(solutionEventsListener.AfterOpenProject, new EventHandler<SolutionEventsListener.SolutionEventArgs>(this.OnProjectAdded));
            solutionEventsListener.BeforeCloseProject = (EventHandler<SolutionEventsListener.SolutionEventArgs>) Delegate.Combine(solutionEventsListener.BeforeCloseProject, new EventHandler<SolutionEventsListener.SolutionEventArgs>(this.OnProjectRemoved));
            solutionEventsListener.BeforeUnloadProject = (EventHandler<SolutionEventsListener.SolutionEventArgs>) Delegate.Combine(solutionEventsListener.BeforeUnloadProject, new EventHandler<SolutionEventsListener.SolutionEventArgs>(this.OnProjectRemoved));
            this.needsRefresh = true;
        }

        private static bool AreAnyFilesReadOnly(List<string> filesToCheckout, List<string> warnings)
        {
            if (filesToCheckout == null)
            {
                throw new ArgumentNullException("filesToCheckout");
            }
            if (warnings == null)
            {
                throw new ArgumentNullException("warnings");
            }
            bool flag = false;
            foreach (string str in filesToCheckout)
            {
                FileInfo info = new FileInfo(str);
                if (info.IsReadOnly)
                {
                    warnings.Add(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.WarningFileIsReadOnly, new object[] { str }));
                    flag = true;
                }
                else
                {
                    foreach (FileSystemAccessRule rule in info.GetAccessControl().GetAccessRules(true, true, typeof(NTAccount)))
                    {
                        if ((rule.AccessControlType == AccessControlType.Deny) && (rule.FileSystemRights == FileSystemRights.Write))
                        {
                            warnings.Add(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.WarningFileIsNotWritable, new object[] { str }));
                            flag = true;
                            break;
                        }
                    }
                }
            }
            return flag;
        }

        private static bool CanUpgradeRefreshProject(IVsHierarchy hierarchy)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(hierarchy as IHierarchyObjectProvider);
            if (hierarchyObject != null)
            {
                IProjectPlatformAdapter adapter = hierarchyObject.Adapter;
                if (adapter != null)
                {
                    IXnaRefreshLevel level = adapter as IXnaRefreshLevel;
                    if (level != null)
                    {
                        IVsBuildPropertyStorage storage = hierarchy as IVsBuildPropertyStorage;
                        if (storage != null)
                        {
                            string pbstrPropValue = null;
                            uint num = 1;
                            if (ErrorHandler.Failed(storage.GetPropertyValue("XnaRefreshLevel", null, num, out pbstrPropValue)))
                            {
                                pbstrPropValue = string.Empty;
                            }
                            return level.CanUpgradeRefreshLevel(pbstrPropValue);
                        }
                    }
                }
            }
            return false;
        }

        private static void FindAllUpgradableHierarchies(List<IVsHierarchy> codeProjectsToUpgrade)
        {
            if (codeProjectsToUpgrade == null)
            {
                throw new ArgumentNullException("codeProjectsToUpgrade");
            }
            foreach (IVsHierarchy hierarchy in Microsoft.Xna.GameStudio.ProjectUtilities.EnumerateProjects(__VSENUMPROJFLAGS.EPF_LOADEDINSOLUTION))
            {
                if (CanUpgradeRefreshProject(hierarchy))
                {
                    codeProjectsToUpgrade.Add(hierarchy);
                }
            }
        }

        private static string FindWindowsPhone7Manifest(IVsHierarchy projectHierarchy)
        {
            string pbstrMkDocument = string.Empty;
            IVsProject project = projectHierarchy as IVsProject;
            if (project != null)
            {
                foreach (uint num in Microsoft.Xna.GameStudio.ProjectUtilities.EnumerateHierarchyItemsFlat(projectHierarchy, 0xfffffffe))
                {
                    project.GetMkDocument(num, out pbstrMkDocument);
                    if (!string.IsNullOrWhiteSpace(pbstrMkDocument) && string.Equals(Path.GetFileName(pbstrMkDocument), "WMAppManifest.xml", StringComparison.OrdinalIgnoreCase))
                    {
                        return pbstrMkDocument;
                    }
                }
            }
            return null;
        }

        private void OnBeforeQueryStatus(object sender, EventArgs args)
        {
            this.Refresh();
            OleMenuCommand command = sender as OleMenuCommand;
            if (command != null)
            {
                command.Supported = true;
                command.Visible = this.isUpgradePossible;
                command.Enabled = this.isUpgradePossible && Microsoft.Xna.GameStudio.ProjectUtilities.IsDesignMode();
            }
        }

        private void OnProjectAdded(object sender, SolutionEventsListener.SolutionEventArgs args)
        {
            if (!this.isUpgradePossible)
            {
                this.needsRefresh = true;
            }
        }

        private void OnProjectRemoved(object sender, SolutionEventsListener.SolutionEventArgs args)
        {
            if (this.isUpgradePossible)
            {
                this.needsRefresh = true;
            }
        }

        private static void OnUpgradeSolution(object sender, EventArgs args)
        {
            IVsUIShell globalService = Package.GetGlobalService(typeof(SVsUIShell)) as IVsUIShell;
            VsDialogOwner owner = new VsDialogOwner(globalService);
            List<IVsHierarchy> codeProjectsToUpgrade = new List<IVsHierarchy>();
            FindAllUpgradableHierarchies(codeProjectsToUpgrade);
            using (UpgradeSolutionForm form = new UpgradeSolutionForm(codeProjectsToUpgrade))
            {
                ErrorHandler.ThrowOnFailure(globalService.EnableModeless(0));
                try
                {
                    if (form.ShowDialog(owner) == DialogResult.Cancel)
                    {
                        return;
                    }
                }
                finally
                {
                    ErrorHandler.ThrowOnFailure(globalService.EnableModeless(1));
                }
            }
            IVsSolution solution = (IVsSolution) Package.GetGlobalService(typeof(SVsSolution));
            int hr = solution.SaveSolutionElement(1, null, 0);
            if (hr != -2147467260)
            {
                ErrorHandler.ThrowOnFailure(hr);
                if (hr == 1)
                {
                    Microsoft.Xna.GameStudio.ProjectUtilities.ShowErrorMessage(globalService, null, XnaProjectResource.UpgradeCannotContinueDirtyFiles, "Microsoft.Xna.GameStudio.UpgradeSolution");
                }
                else
                {
                    List<IVsHierarchy> projects = new List<IVsHierarchy>(codeProjectsToUpgrade);
                    List<string> projectFilePaths = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectFilePaths(projects);
                    List<string> list4 = new List<string>();
                    foreach (IVsHierarchy hierarchy in codeProjectsToUpgrade)
                    {
                        if (Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(hierarchy) != null)
                        {
                            IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(hierarchy as IHierarchyObjectProvider);
                            if (!(hierarchy is IVsBuildPropertyStorage) && (hierarchyObject != null))
                            {
                                IVsBuildPropertyStorage innerVsBuildPropertyStorage = hierarchyObject.InnerVsBuildPropertyStorage;
                            }
                            string pbstrPropValue = string.Empty;
                            ErrorHandler.ThrowOnFailure(hierarchyObject.InnerVsBuildPropertyStorage.GetPropertyValue("XnaPlatform", null, 1, out pbstrPropValue));
                            if (!string.IsNullOrEmpty(pbstrPropValue) && pbstrPropValue.Equals("Windows Phone", StringComparison.OrdinalIgnoreCase))
                            {
                                string item = FindWindowsPhone7Manifest(hierarchy);
                                if (!string.IsNullOrWhiteSpace(item))
                                {
                                    projectFilePaths.Add(item);
                                    list4.Add(item);
                                }
                            }
                        }
                    }
                    if (!Microsoft.Xna.GameStudio.ProjectUtilities.Checkout(projectFilePaths.ToArray()))
                    {
                        Microsoft.Xna.GameStudio.ProjectUtilities.ShowErrorMessage(globalService, null, XnaProjectResource.UpgradeFailedCheckout, "Microsoft.Xna.GameStudio.UpgradeSolution");
                    }
                    else
                    {
                        List<string> warnings = new List<string>();
                        bool foundReadOnlyFiles = AreAnyFilesReadOnly(projectFilePaths, warnings);
                        if (!foundReadOnlyFiles)
                        {
                            string str4;
                            string str5;
                            string str6;
                            foreach (IVsHierarchy hierarchy2 in codeProjectsToUpgrade)
                            {
                                UpgradeXnaCodeProject(hierarchy2, warnings);
                            }
                            foreach (string str3 in list4)
                            {
                                UpgradeWindowsPhone7Manifest(str3, warnings);
                            }
                            ErrorHandler.ThrowOnFailure(solution.GetSolutionInfo(out str4, out str5, out str6));
                            if (ErrorHandler.Failed(solution.SaveSolutionElement(0, null, 0)))
                            {
                                warnings.Add(XnaProjectResource.UpgradeFailedChangesNotSaved);
                            }
                            if (ErrorHandler.Succeeded(solution.CloseSolutionElement(0x10000, null, 0)))
                            {
                                if (ErrorHandler.Failed(solution.OpenSolutionFile(1, str5)))
                                {
                                    string str7;
                                    if (ErrorHandler.Succeeded(globalService.GetErrorInfo(out str7)))
                                    {
                                        warnings.Add(str7);
                                    }
                                    warnings.Add(XnaProjectResource.UpgradeCompletedSolutionNotReloaded);
                                }
                            }
                            else
                            {
                                warnings.Add(XnaProjectResource.UpgradeFailedSolutionNotReloaded);
                                warnings.Add(XnaProjectResource.UpgradeFailedIncompleteChanges);
                            }
                        }
                        ReportWarnings(globalService, warnings, foundReadOnlyFiles);
                    }
                }
            }
        }

        private void Refresh()
        {
            if (this.needsRefresh)
            {
                this.isUpgradePossible = false;
                foreach (IVsHierarchy hierarchy in Microsoft.Xna.GameStudio.ProjectUtilities.EnumerateProjects(__VSENUMPROJFLAGS.EPF_LOADEDINSOLUTION))
                {
                    if (CanUpgradeRefreshProject(hierarchy))
                    {
                        this.isUpgradePossible = true;
                        break;
                    }
                }
                this.needsRefresh = false;
            }
        }

        private static void ReportWarnings(IVsUIShell uiShell, List<string> warnings, bool foundReadOnlyFiles)
        {
            if (uiShell == null)
            {
                throw new ArgumentNullException("uiShell");
            }
            if (warnings == null)
            {
                throw new ArgumentNullException("warnings");
            }
            if (warnings.Count > 0)
            {
                GameStudioWindowPane.WriteLine(XnaProjectResource.UpgradeEncounteredIssues, new object[0]);
                foreach (string str in warnings)
                {
                    GameStudioWindowPane.WriteLine("    " + XnaProjectResource.Warning, new object[] { str });
                }
                string upgradeSolutionProducedWarnings = XnaProjectResource.UpgradeSolutionProducedWarnings;
                if (foundReadOnlyFiles)
                {
                    upgradeSolutionProducedWarnings = XnaProjectResource.UpgradeCanceledReadOnlyFiles;
                    GameStudioWindowPane.WriteLine(XnaProjectResource.UpgradeCanceled, new object[0]);
                }
                Microsoft.Xna.GameStudio.ProjectUtilities.ShowErrorMessage(uiShell, null, upgradeSolutionProducedWarnings, "Microsoft.Xna.GameStudio.UpgradeSolution");
            }
        }

        private static void UpgradeWindowsPhone7Manifest(string filePath, IList<string> warnings)
        {
            XmlDocument document = new XmlDocument();
            XmlElement newChild = null;
            XmlAttribute attribute = null;
            document.Load(filePath);
            XmlNode node = document.SelectSingleNode(".//Capabilities");
            if (node != null)
            {
                bool flag = false;
                foreach (string str in newRefreshCapabilities)
                {
                    if (document.SelectSingleNode(string.Format(".//Capabilities/Capability[@Name='{0}']", str)) == null)
                    {
                        newChild = document.CreateElement("Capability");
                        attribute = document.CreateAttribute("Name");
                        attribute.Value = str;
                        newChild.Attributes.Append(attribute);
                        if (!flag)
                        {
                            node.AppendChild(document.CreateComment(XnaProjectResource.WPUpgradeCommentCapabilities));
                            flag = true;
                        }
                        node.AppendChild(newChild);
                    }
                }
                if (flag)
                {
                    document.Save(filePath);
                }
            }
            else
            {
                warnings.Add(XnaProjectResource.WPWarningNoCapabilities);
            }
        }

        private static bool UpgradeXnaCodeProject(IVsHierarchy hierarchy, IList<string> warnings)
        {
            UpgradeXnaRefresh(hierarchy, warnings);
            return true;
        }

        private static void UpgradeXnaRefresh(IVsHierarchy hierarchy, IList<string> warnings)
        {
            if (hierarchy == null)
            {
                throw new ArgumentNullException("hierarchy");
            }
            IXnaCodeProject hierarchyObject = Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(hierarchy as IHierarchyObjectProvider);
            if (hierarchyObject != null)
            {
                IProjectPlatformAdapter adapter = hierarchyObject.Adapter;
                if (adapter != null)
                {
                    IXnaRefreshLevel level = adapter as IXnaRefreshLevel;
                    if (level == null)
                    {
                        throw new InvalidOperationException("can't really upgrade after all");
                    }
                    level.UpgradeRefreshLevel(hierarchy);
                }
            }
        }
    }
}

