﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    internal static class VsBrowseDialog
    {
        private const int MAX_FILE_LENGTH = 0x400;

        private static string MakeFilterString(string s)
        {
            if (s == null)
            {
                return null;
            }
            int length = s.Length;
            char[] destination = new char[length + 1];
            s.CopyTo(0, destination, 0, length);
            for (int i = 0; i < length; i++)
            {
                if (destination[i] == '|')
                {
                    destination[i] = '\0';
                }
            }
            return new string(destination);
        }

        internal static DialogResult ShowOpenFileDialog(IServiceProvider serviceProvider, IntPtr hwndOwner, string filePathIn, string filter, int filterIndex, string title, int helpTopic, out string filePathOut)
        {
            DialogResult cancel;
            filePathOut = null;
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (hwndOwner == IntPtr.Zero)
            {
                throw new ArgumentNullException("hwndOwner");
            }
            if (filter == null)
            {
                throw new ArgumentNullException("filter");
            }
            if (title == null)
            {
                throw new ArgumentNullException("title");
            }
            IVsUIShell service = (IVsUIShell) serviceProvider.GetService(typeof(IVsUIShell));
            string currentDirectory = null;
            if ((filePathIn == null) || (filePathIn.Trim().Length == 0))
            {
                currentDirectory = Directory.GetCurrentDirectory();
            }
            else
            {
                currentDirectory = Path.GetFullPath(Path.GetDirectoryName(filePathIn));
            }
            int num = 0x400;
            IntPtr zero = IntPtr.Zero;
            VSOPENFILENAMEW vsopenfilenamew = new VSOPENFILENAMEW();
            try
            {
                char[] source = new char[num + 1];
                zero = Marshal.AllocCoTaskMem(source.Length * 2);
                Marshal.Copy(source, 0, zero, source.Length);
                vsopenfilenamew.lStructSize = (uint) Marshal.SizeOf(typeof(VSOPENFILENAMEW));
                vsopenfilenamew.hwndOwner = hwndOwner;
                vsopenfilenamew.pwzFilter = MakeFilterString(filter);
                vsopenfilenamew.nFilterIndex = (uint) filterIndex;
                vsopenfilenamew.pwzFileName = zero;
                vsopenfilenamew.nMaxFileName = (uint) num;
                vsopenfilenamew.pwzDlgTitle = title;
                vsopenfilenamew.dwHelpTopic = (uint) helpTopic;
                vsopenfilenamew.pwzInitialDir = currentDirectory;
                VSOPENFILENAMEW[] pOpenFileName = new VSOPENFILENAMEW[] { vsopenfilenamew };
                ErrorHandler.ThrowOnFailure(service.GetOpenFileNameViaDlg(pOpenFileName));
                Marshal.Copy(zero, source, 0, source.Length);
                int index = 0;
                while ((index < source.Length) && (source[index] != '\0'))
                {
                    index++;
                }
                filePathOut = new string(source, 0, index);
                if (index > 0)
                {
                    return DialogResult.OK;
                }
                cancel = DialogResult.Cancel;
            }
            catch (COMException exception)
            {
                if (exception.ErrorCode != -2147221492)
                {
                    throw;
                }
                cancel = DialogResult.Cancel;
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.FreeCoTaskMem(zero);
                    zero = IntPtr.Zero;
                }
            }
            return cancel;
        }

        internal static DialogResult ShowOpenFolderDialog(IServiceProvider serviceProvider, IntPtr hwndOwner, string initialDirectory, string title, int helpTopic, out string filePathOut)
        {
            DialogResult cancel;
            filePathOut = null;
            if (serviceProvider == null)
            {
                throw new ArgumentNullException("serviceProvider");
            }
            if (hwndOwner == IntPtr.Zero)
            {
                throw new ArgumentNullException("hwndOwner");
            }
            if (title == null)
            {
                throw new ArgumentNullException("title");
            }
            IVsUIShell service = (IVsUIShell) serviceProvider.GetService(typeof(IVsUIShell));
            if (initialDirectory == null)
            {
                initialDirectory = Directory.GetCurrentDirectory();
            }
            int num = Math.Max(0x400, initialDirectory.Length);
            IntPtr zero = IntPtr.Zero;
            VSBROWSEINFOW vsbrowseinfow = new VSBROWSEINFOW();
            try
            {
                char[] source = new char[num + 1];
                zero = Marshal.AllocCoTaskMem(source.Length * 2);
                Marshal.Copy(source, 0, zero, source.Length);
                vsbrowseinfow.lStructSize = (uint) Marshal.SizeOf(typeof(VSBROWSEINFOW));
                vsbrowseinfow.hwndOwner = hwndOwner;
                vsbrowseinfow.pwzDirName = zero;
                vsbrowseinfow.nMaxDirName = (uint) num;
                vsbrowseinfow.pwzDlgTitle = title;
                vsbrowseinfow.dwHelpTopic = (uint) helpTopic;
                vsbrowseinfow.pwzInitialDir = initialDirectory;
                VSBROWSEINFOW[] pBrowse = new VSBROWSEINFOW[] { vsbrowseinfow };
                ErrorHandler.ThrowOnFailure(service.GetDirectoryViaBrowseDlg(pBrowse));
                Marshal.Copy(zero, source, 0, source.Length);
                int index = 0;
                while ((index < source.Length) && (source[index] != '\0'))
                {
                    index++;
                }
                filePathOut = new string(source, 0, index);
                if (index > 0)
                {
                    return DialogResult.OK;
                }
                cancel = DialogResult.Cancel;
            }
            catch (COMException exception)
            {
                if (exception.ErrorCode != -2147221492)
                {
                    throw;
                }
                cancel = DialogResult.Cancel;
            }
            finally
            {
                if (zero != IntPtr.Zero)
                {
                    Marshal.FreeCoTaskMem(zero);
                    zero = IntPtr.Zero;
                }
            }
            return cancel;
        }
    }
}

