﻿namespace Microsoft.Xna.GameStudio.WindowsPhone
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.PlatformTools;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;
    using System.Threading;

    internal class WindowsPhoneDeployableProjectCfg : IVsDeployableProjectCfg
    {
        private volatile EventSinkCollection adviseSink;
        private bool deployInProgress;
        private volatile Thread deploymentThread;
        private readonly XnaErrorProvider errorProvider;
        private List<string> errors = new List<string>();
        private readonly string f1HelpKeyword;
        private readonly WindowsPhoneFlavorCfg parentCfg;
        private readonly object processLock = new object();
        private readonly object sinkLock = new object();

        public WindowsPhoneDeployableProjectCfg(WindowsPhoneFlavorCfg cfg, string f1HelpKeyword)
        {
            this.parentCfg = cfg;
            this.f1HelpKeyword = f1HelpKeyword;
            this.errorProvider = this.parentCfg.Project.ErrorProvider;
        }

        public int AdviseDeployStatusCallback(IVsDeployStatusCallback pIVsDeployStatusCallback, out uint pdwCookie)
        {
            if (pIVsDeployStatusCallback == null)
            {
                throw new ArgumentNullException("pIVsDeployStatusCallback");
            }
            lock (this.sinkLock)
            {
                if (this.adviseSink == null)
                {
                    this.adviseSink = new EventSinkCollection();
                }
                pdwCookie = this.adviseSink.Add(pIVsDeployStatusCallback);
            }
            return 0;
        }

        public int Commit(uint dwReserved)
        {
            return 0;
        }

        private void DeploymentThread(object passedParameters)
        {
            bool flag = false;
            bool flag2 = false;
            StartParameters parameters = passedParameters as StartParameters;
            try
            {
                this.errors.Clear();
                WriteLine(parameters.OutputPane, string.Format(XnaProjectResource.WPDeploying, parameters.XapPath), new object[0]);
                parameters.Container.SendFile(parameters.XapPath, parameters.CachePath);
                flag = true;
            }
            catch (ThreadAbortException)
            {
                flag2 = true;
                flag = false;
                Thread.ResetAbort();
            }
            catch (Exception exception)
            {
                flag = false;
                if ((exception.InnerException != null) && !string.IsNullOrWhiteSpace(exception.InnerException.Message))
                {
                    WriteLine(parameters.OutputPane, exception.InnerException.Message, new object[0]);
                    this.errors.Add(exception.InnerException.Message);
                }
                else
                {
                    WriteLine(parameters.OutputPane, exception.Message, new object[0]);
                    this.errors.Add(exception.Message);
                }
            }
            finally
            {
                this.deploymentThread = null;
                if (flag2)
                {
                    this.OnEndDeploy(DeployEndReason.Canceled, parameters.OutputPane);
                }
                else
                {
                    this.OnEndDeploy(flag ? DeployEndReason.Success : DeployEndReason.Failure, parameters.OutputPane);
                }
            }
        }

        private void OnEndDeploy(DeployEndReason reason, IVsOutputWindowPane pOutputPane)
        {
            try
            {
                if (reason == DeployEndReason.Failure)
                {
                    foreach (string str in this.errors)
                    {
                        this.errorProvider.ReportError(str, this.f1HelpKeyword);
                    }
                    if (this.errors.Count == 0)
                    {
                        this.errorProvider.ReportError(XnaProjectResource.DeployFailedUnknownError, this.f1HelpKeyword);
                    }
                }
                if (reason == DeployEndReason.Canceled)
                {
                    this.errorProvider.ReportError(XnaProjectResource.WPDeployCanceled, this.f1HelpKeyword);
                }
                else
                {
                    WriteLine(pOutputPane, XnaProjectResource.WPDeployComplete, new object[0]);
                }
                foreach (IVsDeployStatusCallback callback in this.adviseSink)
                {
                    try
                    {
                        callback.OnEndDeploy((reason == DeployEndReason.Success) ? 1 : 0);
                    }
                    catch
                    {
                    }
                }
            }
            finally
            {
                this.deployInProgress = false;
            }
        }

        public int QueryStartDeploy(uint dwOptions, int[] pfSupported, int[] pfReady)
        {
            if ((pfSupported != null) && (pfSupported.Length > 0))
            {
                pfSupported[0] = this.parentCfg.Project.IsEntryProject ? 1 : 0;
            }
            if ((pfReady != null) && (pfReady.Length > 0))
            {
                pfReady[0] = this.deployInProgress ? 0 : 1;
            }
            return 0;
        }

        public int QueryStatusDeploy(out int pfDeployDone)
        {
            pfDeployDone = this.deployInProgress ? 0 : 1;
            return 0;
        }

        public int Rollback(uint dwReserved)
        {
            return 0;
        }

        public int StartDeploy(IVsOutputWindowPane pIVsOutputWindowPane, uint dwOptions)
        {
            if (pIVsOutputWindowPane == null)
            {
                throw new ArgumentNullException("pIVsOutputWindowPane");
            }
            this.errorProvider.Tasks.Clear();
            bool flag = false;
            try
            {
                string targetConsole = this.parentCfg.TargetConsole;
                if (string.IsNullOrEmpty(targetConsole))
                {
                    throw new InvalidOperationException(XnaProjectResource.DeployNoTargetConsole);
                }
                Guid assemblyGuid = Microsoft.Xna.GameStudio.ProjectUtilities.GetAssemblyGuid(Microsoft.Xna.GameStudio.ProjectUtilities.GetDteProject(this.parentCfg.Project.GetVsHierarchy()));
                if (assemblyGuid == Guid.Empty)
                {
                    throw new InvalidOperationException(XnaProjectResource.Xbox360DeployFailedMissingTitleID);
                }
                string titleId = assemblyGuid.ToString("D", CultureInfo.InvariantCulture);
                string configurationProperty = this.parentCfg.GetConfigurationProperty("MSBuildProjectDirectory");
                string str4 = this.parentCfg.GetConfigurationProperty("IntermediateOutputPath");
                if (string.IsNullOrWhiteSpace(configurationProperty) || string.IsNullOrWhiteSpace(str4))
                {
                    throw new InvalidOperationException(XnaProjectResource.DeployNoIntDir);
                }
                string str5 = this.parentCfg.GetConfigurationProperty("OutDir");
                if (string.IsNullOrWhiteSpace(str5))
                {
                    throw new InvalidOperationException(XnaProjectResource.DeployNoOutDir);
                }
                string fullPath = Path.GetFullPath(Path.Combine(configurationProperty, str5));
                string str7 = this.parentCfg.GetConfigurationProperty("XapFilename");
                if (string.IsNullOrWhiteSpace(str7))
                {
                    throw new InvalidOperationException(XnaProjectResource.DeployNoXapFilename);
                }
                string path = Path.Combine(fullPath, str7);
                if (!File.Exists(path))
                {
                    throw new InvalidOperationException(string.Format(XnaProjectResource.DeployNoXapFile, path));
                }
                string str9 = this.parentCfg.GetConfigurationProperty("Thumbnail");
                if (!string.IsNullOrWhiteSpace(str9))
                {
                    str9 = Path.GetFullPath(Path.Combine(configurationProperty, str9));
                }
                string cachePath = Path.GetFullPath(Path.Combine(configurationProperty, Path.Combine(str4, "XapCacheFile.xml")));
                string str11 = this.parentCfg.GetConfigurationProperty("ForceCleanDeploy");
                bool flag2 = "true".Equals(str11, StringComparison.OrdinalIgnoreCase);
                IPlatform platform = PlatformManager.GetPlatform(this.parentCfg.Project.Adapter.XnaPlatform, this.parentCfg.XnaFrameworkRuntimeProfile);
                if (platform == null)
                {
                    throw new InvalidOperationException(XnaProjectResource.DeployNoPlatformAvailable);
                }
                ITitlePlayer titlePlayer = platform.GetTitlePlayer(targetConsole);
                if (titlePlayer == null)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.DeployNoTargetAvailable, new object[] { targetConsole }));
                }
                if (titlePlayer.GetRuntimeSupport(this.parentCfg.XnaFrameworkRuntimeProfile) != RuntimeSupport.Supported)
                {
                    throw new InvalidOperationException(string.Format(CultureInfo.CurrentUICulture, XnaProjectResource.WPErrorIncompatibleRuntime, new object[] { targetConsole }));
                }
                IDeployService service = titlePlayer.GetService(typeof(IDeployService)) as IDeployService;
                string containerId = service.GetContainerId(titleId);
                IContainer container = service.OpenContainer(containerId, titleId);
                if (container != null)
                {
                    if (!flag2)
                    {
                        container.Close();
                    }
                    container.SetThumbnail(!string.IsNullOrEmpty(str9) ? str9 : string.Empty);
                    Thread thread = new Thread(new ParameterizedThreadStart(this.DeploymentThread));
                    if (Interlocked.CompareExchange<Thread>(ref this.deploymentThread, thread, null) != null)
                    {
                        throw new InvalidOperationException();
                    }
                    StartParameters parameter = new StartParameters(container, path, cachePath, pIVsOutputWindowPane);
                    lock (this.processLock)
                    {
                        try
                        {
                            this.deployInProgress = true;
                            flag = true;
                            this.deploymentThread.Start(parameter);
                        }
                        catch (Exception)
                        {
                            throw new Exception(XnaProjectResource.UnableToStartWindowsPhoneTool);
                        }
                    }
                }
            }
            catch (Exception exception)
            {
                flag = false;
                this.deployInProgress = false;
                this.deploymentThread = null;
                this.errorProvider.ReportError(exception, this.f1HelpKeyword);
                throw exception;
            }
            if (!flag)
            {
                return -2147467259;
            }
            return 0;
        }

        public int StopDeploy(int fSync)
        {
            try
            {
                lock (this.processLock)
                {
                    if (this.deploymentThread != null)
                    {
                        this.deploymentThread.Abort();
                    }
                }
            }
            catch
            {
                return -2147467259;
            }
            return 0;
        }

        public int UnadviseDeployStatusCallback(uint dwCookie)
        {
            lock (this.sinkLock)
            {
                if (this.adviseSink == null)
                {
                    throw new InvalidOperationException();
                }
                this.adviseSink.RemoveAt(dwCookie);
            }
            return 0;
        }

        public int WaitDeploy(uint dwMilliseconds, int fTickWhenMessageQNotEmpty)
        {
            return 0;
        }

        private static void WriteLine(IVsOutputWindowPane outputWindow, string format, params object[] args)
        {
            if (format == null)
            {
                throw new ArgumentNullException("format");
            }
            if (outputWindow != null)
            {
                string str = format + "\n";
                if (args != null)
                {
                    str = string.Format(CultureInfo.InvariantCulture, str, args);
                }
                ErrorHandler.ThrowOnFailure(outputWindow.OutputStringThreadSafe(str));
            }
        }

        private enum DeployEndReason
        {
            Canceled = 3,
            Failure = 2,
            Success = 1
        }

        private class StartParameters
        {
            public StartParameters(IContainer container, string xapPath, string cachePath, IVsOutputWindowPane outputPane)
            {
                this.Container = container;
                this.XapPath = xapPath;
                this.CachePath = cachePath;
                this.OutputPane = outputPane;
            }

            public string CachePath { get; set; }

            public IContainer Container { get; set; }

            public IVsOutputWindowPane OutputPane { get; set; }

            public string XapPath { get; set; }
        }
    }
}

