﻿namespace Microsoft.Xna.GameStudio.Wizards
{
    using EnvDTE;
    using Microsoft.Build.Construction;
    using Microsoft.Build.Evaluation;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Xna.GameStudio;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.FlavoredProject;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Runtime.InteropServices;
    using System.Text;

    internal sealed class CrossPlatformProjectWizard
    {
        private Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> availableAdapters;
        private static readonly string[] BclAssemblyNames = new string[] { "mscorlib", "system", "system.core", "system.xml", "system.xml.linq" };
        private ConvertedProject convertedProject;
        private Stack<ConvertedProject> convertedProjects;
        private readonly XnaErrorProvider errorProvider;
        private List<Exception> errors;
        private CrossPlatformGroupManager groupManager;
        private string originalStartupType;
        private IVsSolution solution;
        private string sourceProjectPath;
        private IProjectPlatformAdapter targetAdapter;
        private string targetPlatform;
        private string targetProfile;
        private StringBuilder userFeedback;
        private IXnaCodeProject xnaProject;

        internal CrossPlatformProjectWizard(IXnaCodeProject xnaProject, string targetPlatform)
        {
            if (xnaProject == null)
            {
                throw new ArgumentNullException("xnaProject");
            }
            if (targetPlatform == null)
            {
                throw new ArgumentNullException("targetPlatform");
            }
            this.xnaProject = xnaProject;
            this.errorProvider = xnaProject.ErrorProvider;
            this.convertedProjects = new Stack<ConvertedProject>();
            IVsProject vsHierarchy = (IVsProject) this.xnaProject.GetVsHierarchy();
            ErrorHandler.ThrowOnFailure(vsHierarchy.GetMkDocument(0xfffffffe, out this.sourceProjectPath));
            Microsoft.Build.Evaluation.Project project = ProjectCollection.GlobalProjectCollection.LoadProject(this.sourceProjectPath);
            try
            {
                this.originalStartupType = project.GetPropertyValue("XnaOutputType");
                this.solution = (IVsSolution) xnaProject.GetVsService(typeof(SVsSolution));
                this.groupManager = (CrossPlatformGroupManager) this.xnaProject.GetVsService(typeof(CrossPlatformGroupManager));
                this.availableAdapters = (Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter>) this.xnaProject.GetVsService(typeof(Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter>));
                this.errors = new List<Exception>();
                this.userFeedback = new StringBuilder();
                this.SetTargetPlatformAndProfile(targetPlatform);
            }
            finally
            {
                ProjectCollection.GlobalProjectCollection.UnloadProject(project);
            }
        }

        private CrossPlatformProjectWizard(IXnaCodeProject xnaProject, string targetPlatform, Stack<ConvertedProject> convertedProjects) : this(xnaProject, targetPlatform)
        {
            if (convertedProjects == null)
            {
                throw new ArgumentNullException("convertedProjects");
            }
            this.convertedProjects = convertedProjects;
        }

        private bool AddAlreadyConvertedProjectReference(IXnaCodeProject xnaCodeProject, ProjectRootElement projectRoot)
        {
            if (xnaCodeProject == null)
            {
                throw new ArgumentNullException("xnaCodeProject");
            }
            if (projectRoot == null)
            {
                throw new ArgumentNullException("projectRoot");
            }
            string crossPlatformGroupID = xnaCodeProject.CrossPlatformGroupID;
            foreach (ConvertedProject project in this.convertedProjects)
            {
                if (string.Compare(crossPlatformGroupID, project.GroupID, StringComparison.OrdinalIgnoreCase) == 0)
                {
                    this.AddMSBuildProjectReference(project, projectRoot);
                    return true;
                }
            }
            return false;
        }

        private void AddMSBuildProjectReference(IXnaCodeProject xnaCodeProject, ProjectRootElement projectRoot)
        {
            if (xnaCodeProject == null)
            {
                throw new ArgumentNullException("xnaCodeProject");
            }
            this.AddMSBuildProjectReference(GetProjectDescriptor(xnaCodeProject), projectRoot);
        }

        private void AddMSBuildProjectReference(ConvertedProject targetProject, ProjectRootElement projectRoot)
        {
            string include = PackageUtilities.MakeRelative(this.convertedProject.MkDocument, targetProject.MkDocument);
            ProjectItemElement element = projectRoot.AddItem("ProjectReference", include);
            element.AddMetadata("Project", targetProject.ProjectGuid);
            element.AddMetadata("Name", Path.GetFileNameWithoutExtension(targetProject.MkDocument));
        }

        private ProjectItemElement AddReplacementReference(ProjectItemGroupElement newReferences, AssemblyName oldAssemblyName, string newAssemblyName)
        {
            ProjectItemElement element = newReferences.AddItem("Reference", newAssemblyName);
            if (oldAssemblyName.FullName != newAssemblyName)
            {
                this.userFeedback.AppendLine(string.Format(XnaProjectResource.XPlatWizardReferenceReplaced, oldAssemblyName.FullName, newAssemblyName));
            }
            return element;
        }

        private string CloneSourceProject()
        {
            if ((this.sourceProjectPath == null) || !File.Exists(this.sourceProjectPath))
            {
                throw new InvalidOperationException();
            }
            string directoryName = Path.GetDirectoryName(this.sourceProjectPath);
            string extension = Path.GetExtension(this.sourceProjectPath);
            string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(this.sourceProjectPath);
            string str4 = string.Format(CultureInfo.InvariantCulture, XnaProjectResource.CrossPlatformWizardTargetNameFormat, new object[] { this.targetPlatform, fileNameWithoutExtension });
            int num = 2;
            string path = Path.Combine(directoryName, str4 + extension);
            while (File.Exists(path))
            {
                path = Path.Combine(directoryName, string.Format(CultureInfo.InvariantCulture, XnaProjectResource.CrossPlatformWizardTargetNameFormatNumbered, new object[] { this.targetPlatform, num, fileNameWithoutExtension }) + extension);
                num++;
            }
            File.Copy(this.sourceProjectPath, path);
            FileAttributes attributes = File.GetAttributes(path);
            if ((attributes & FileAttributes.ReadOnly) != 0)
            {
                File.SetAttributes(path, attributes & ~FileAttributes.ReadOnly);
            }
            return path;
        }

        private bool ConvertClonedFileToTargetPlatform(string targetFilePath)
        {
            bool flag = false;
            if (targetFilePath == null)
            {
                throw new ArgumentNullException("targetFilePath");
            }
            if (!File.Exists(targetFilePath))
            {
                throw new InvalidOperationException();
            }
            ProjectRootElement projectRoot = ProjectRootElement.Open(targetFilePath, ProjectCollection.GlobalProjectCollection);
            string projectGuid = Guid.NewGuid().ToString("B", CultureInfo.InvariantCulture);
            string crossPlatformGroupID = this.xnaProject.CrossPlatformGroupID;
            this.convertedProject = new ConvertedProject(targetFilePath, projectGuid, crossPlatformGroupID);
            this.convertedProjects.Push(this.convertedProject);
            try
            {
                foreach (ProjectPropertyElement element2 in projectRoot.Properties)
                {
                    switch (element2.Name)
                    {
                        case "SccProjectName":
                        case "SccLocalPath":
                        case "SccAuxPath":
                        case "SccProvider":
                            element2.Parent.RemoveChild(element2);
                            break;

                        case "XnaPlatform":
                            element2.Value = this.targetPlatform;
                            break;

                        case "XnaProfile":
                            element2.Value = this.targetProfile;
                            break;

                        case "ProjectGuid":
                            element2.Value = projectGuid;
                            break;

                        case "Platform":
                            element2.Value = this.targetAdapter.PlatformConfigurationName;
                            break;
                    }
                }
                if (string.Equals(this.originalStartupType, "Library", StringComparison.OrdinalIgnoreCase))
                {
                    projectRoot.AddProperty("OutputType", "Library");
                }
                else if (this.targetPlatform == "Windows Phone")
                {
                    projectRoot.AddProperty("OutputType", "Library");
                }
                else if (this.targetPlatform == "Windows")
                {
                    projectRoot.AddProperty("OutputType", "WinExe");
                }
                else
                {
                    projectRoot.AddProperty("OutputType", "Exe");
                }
                RemoveExistingConfigurations(projectRoot);
                ProjectPropertyGroupElement child = projectRoot.CreatePropertyGroupElement();
                ProjectPropertyGroupElement element4 = projectRoot.CreatePropertyGroupElement();
                IEnumerator enumerator = projectRoot.PropertyGroups.GetEnumerator();
                enumerator.Reset();
                enumerator.MoveNext();
                projectRoot.InsertAfterChild(child, enumerator.Current as ProjectElement);
                projectRoot.InsertAfterChild(element4, enumerator.Current as ProjectElement);
                this.SetDefaultDebugConfigProperties(child);
                this.SetDefaultReleaseConfigProperties(element4);
                this.xnaProject.LanguageAdapter.ModifyProjectOnCopy(projectRoot, this.targetAdapter);
                List<ProjectItemElement> oldReferences = new List<ProjectItemElement>();
                List<ProjectItemElement> list2 = new List<ProjectItemElement>();
                List<ProjectItemElement> list3 = new List<ProjectItemElement>();
                List<ProjectItemElement> list4 = new List<ProjectItemElement>();
                List<ProjectItemElement> list5 = new List<ProjectItemElement>();
                foreach (ProjectItemElement element5 in projectRoot.Items)
                {
                    switch (element5.ItemType)
                    {
                        case "Reference":
                            oldReferences.Add(element5);
                            break;

                        case "COMReference":
                        case "COMFileReference":
                            list2.Add(element5);
                            break;

                        case "ProjectReference":
                            list3.Add(element5);
                            break;

                        case "WCFMetadata":
                        case "WCFMetadataStorage":
                            list5.Add(element5);
                            break;

                        default:
                            list4.Add(element5);
                            break;
                    }
                }
                foreach (ProjectItemElement element6 in list4.FindAll(p => p.HasMetadata))
                {
                    foreach (ProjectMetadataElement element7 in element6.Metadata)
                    {
                        if ((element7.Name == "XnaPlatformSpecific") && string.Equals(element7.Value, bool.TrueString, StringComparison.OrdinalIgnoreCase))
                        {
                            element6.Parent.RemoveChild(element6);
                            this.userFeedback.AppendLine(string.Format(XnaProjectResource.XplatWizardConfigFileRemoved, element6.Include));
                            break;
                        }
                    }
                }
                if (this.targetPlatform != "Windows")
                {
                    foreach (ProjectItemElement element8 in list2)
                    {
                        this.userFeedback.AppendLine(string.Format(XnaProjectResource.XplatWizardComReferenceRemoved, element8.Include));
                        element8.Parent.RemoveChild(element8);
                    }
                }
                if (list5.Count > 0)
                {
                    Predicate<ProjectItemElement> match = null;
                    this.userFeedback.AppendLine(string.Format(XnaProjectResource.XPlatWizardRemovedServiceReferences, this.targetPlatform));
                    List<string> referencePaths = new List<string>();
                    foreach (ProjectItemElement element9 in list5)
                    {
                        if (!string.IsNullOrEmpty(element9.Include))
                        {
                            referencePaths.Add(element9.Include);
                        }
                    }
                    if (referencePaths.Count > 0)
                    {
                        if (match == null)
                        {
                            match = p => this.IsItemInReferencedPaths(p.Include, referencePaths);
                        }
                        foreach (ProjectItemElement element10 in list4.FindAll(match))
                        {
                            element10.Parent.RemoveChild(element10);
                        }
                    }
                    foreach (ProjectItemElement element11 in list5)
                    {
                        element11.Parent.RemoveChild(element11);
                    }
                }
                this.ReplaceReferences(projectRoot, oldReferences);
                this.ReplaceProjectReferences(projectRoot, list3);
                List<string> errors = new List<string>();
                BasePlatformAdapter targetAdapter = this.targetAdapter as BasePlatformAdapter;
                if (targetAdapter != null)
                {
                    targetAdapter.SetLanguageName(this.xnaProject.LanguageAdapter.Name);
                }
                this.targetAdapter.AddPlatformSpecificComponents(projectRoot, errors);
                flag = !WizardUtilities.PresentProjectMigrationOptions(projectRoot, this.targetAdapter);
                foreach (string str3 in errors)
                {
                    this.errorProvider.ReportError(str3, null);
                    this.userFeedback.AppendLine(str3);
                }
                projectRoot.Save(targetFilePath);
            }
            finally
            {
                if (projectRoot != null)
                {
                    ProjectCollection.GlobalProjectCollection.TryUnloadProject(projectRoot);
                }
            }
            return !flag;
        }

        internal void Execute()
        {
            GetProjectDescriptor(this.xnaProject);
            string projectName = Microsoft.Xna.GameStudio.ProjectUtilities.GetProjectName(this.xnaProject.GetVsHierarchy());
            this.userFeedback.AppendFormat(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardStartCopyingFor, new object[] { projectName, this.targetPlatform });
            this.userFeedback.AppendLine();
            try
            {
                string targetFilePath = this.CloneSourceProject();
                if (this.ConvertClonedFileToTargetPlatform(targetFilePath))
                {
                    int num;
                    IntPtr zero = IntPtr.Zero;
                    try
                    {
                        Guid empty = Guid.Empty;
                        uint grfCreateFlags = 2;
                        Guid iidProject = VSConstants.IID_IUnknown;
                        num = this.solution.CreateProject(ref empty, targetFilePath, null, null, grfCreateFlags, ref iidProject, out zero);
                        if (ErrorHandler.Succeeded(num))
                        {
                            object obj2;
                            IVsHierarchy objectForIUnknown = (IVsHierarchy) Marshal.GetObjectForIUnknown(zero);
                            EnvDTE.ProjectItem webReferencesFolder = Microsoft.Xna.GameStudio.ProjectUtilities.GetVSProject(objectForIUnknown).WebReferencesFolder;
                            if (webReferencesFolder != null)
                            {
                                webReferencesFolder.Remove();
                                this.userFeedback.AppendLine(XnaProjectResource.XPlatWizardRemovedWebReferences);
                            }
                            ErrorHandler.ThrowOnFailure(objectForIUnknown.GetProperty(0xfffffffe, -2027, out obj2));
                            EnvDTE.Project project = obj2 as EnvDTE.Project;
                            ProjectWizard.AddDependentBootstrapperPackages(project);
                            Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>((IHierarchyObjectProvider) objectForIUnknown).CrossPlatformGroupID = this.xnaProject.CrossPlatformGroupID;
                        }
                    }
                    finally
                    {
                        ComUtilities.SmartRelease(ref zero);
                        this.convertedProjects.Pop();
                    }
                    ErrorHandler.ThrowOnFailure(num);
                }
            }
            finally
            {
                foreach (Exception exception in this.errors)
                {
                    this.userFeedback.AppendLine(exception.Message);
                }
                this.userFeedback.AppendFormat(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardDoneCopyingFor, new object[] { projectName, this.targetPlatform });
                this.userFeedback.AppendLine();
            }
        }

        private IXnaCodeProject GetCrossPlatformProject(IXnaCodeProject project)
        {
            if (project == null)
            {
                throw new ArgumentNullException("project");
            }
            string crossPlatformGroupID = project.CrossPlatformGroupID;
            if (string.IsNullOrEmpty(crossPlatformGroupID))
            {
                throw new InvalidOperationException(XnaProjectResource.InternalMissingXnaGroupID);
            }
            return this.groupManager.GetProject(crossPlatformGroupID, this.targetPlatform);
        }

        internal string GetFeedback()
        {
            return this.userFeedback.ToString();
        }

        private string GetPlatformDefines()
        {
            return this.targetAdapter.PlatformConstants;
        }

        private static ConvertedProject GetProjectDescriptor(IXnaCodeProject xnaCodeProject)
        {
            if (xnaCodeProject == null)
            {
                throw new ArgumentNullException("xnaCodeProject");
            }
            string buildProperty = xnaCodeProject.GetBuildProperty("ProjectGuid", null);
            string crossPlatformGroupID = xnaCodeProject.CrossPlatformGroupID;
            if (string.IsNullOrEmpty(crossPlatformGroupID))
            {
                crossPlatformGroupID = Guid.NewGuid().ToString();
                xnaCodeProject.CrossPlatformGroupID = crossPlatformGroupID;
            }
            string pbstrMkDocument = null;
            ErrorHandler.ThrowOnFailure(xnaCodeProject.InnerVsProject.GetMkDocument(0xfffffffe, out pbstrMkDocument));
            return new ConvertedProject(pbstrMkDocument, buildProperty, crossPlatformGroupID);
        }

        private static Guid GetProjectGuid(ProjectItemElement removedReference, Dictionary<string, string> metadataTable)
        {
            string str;
            if (removedReference == null)
            {
                throw new ArgumentNullException("removedReference");
            }
            if (metadataTable == null)
            {
                throw new ArgumentNullException("metadataTable");
            }
            metadataTable.TryGetValue("Project", out str);
            if (string.IsNullOrEmpty(str))
            {
                string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(removedReference.Include);
                throw new WizardException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardInvalidProjectReference, new object[] { fileNameWithoutExtension }));
            }
            Guid empty = Guid.Empty;
            try
            {
                empty = new Guid(str);
            }
            catch (FormatException exception)
            {
                string str3 = Path.GetFileNameWithoutExtension(removedReference.Include);
                throw new WizardException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardFailedProjectReferenceConversion, new object[] { str3, exception }));
            }
            catch (OverflowException exception2)
            {
                string str4 = Path.GetFileNameWithoutExtension(removedReference.Include);
                throw new WizardException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardFailedProjectReferenceConversion, new object[] { str4, exception2 }));
            }
            return empty;
        }

        private string GetProjectTypeGuids(string targetPlatform)
        {
            if (targetPlatform == null)
            {
                throw new ArgumentNullException("targetPlatform");
            }
            return ("{6D335F3A-9D43-41b4-9D22-F6F17C4BE596};" + this.xnaProject.LanguageAdapter.LanguageGuid.ToString("B"));
        }

        private IXnaCodeProject GetReferencedProject(ProjectItemElement removedReference, Dictionary<string, string> metadataTable)
        {
            IVsHierarchy hierarchy;
            if (removedReference == null)
            {
                throw new ArgumentNullException("removedReference");
            }
            Guid projectGuid = GetProjectGuid(removedReference, metadataTable);
            ErrorHandler.ThrowOnFailure(this.solution.GetProjectOfGuid(ref projectGuid, out hierarchy));
            return Microsoft.Xna.GameStudio.ProjectUtilities.GetHierarchyObject<IXnaCodeProject>(hierarchy as IHierarchyObjectProvider);
        }

        private bool IsItemInReferencedPaths(string sourcePath, IEnumerable<string> referencedPaths)
        {
            foreach (string str in referencedPaths)
            {
                if (sourcePath.StartsWith(str, StringComparison.OrdinalIgnoreCase))
                {
                    return true;
                }
            }
            return false;
        }

        private static void RemoveExistingConfigurations(ProjectRootElement projectRoot)
        {
            if (projectRoot == null)
            {
                throw new ArgumentNullException("projectRoot");
            }
            List<ProjectPropertyGroupElement> list = new List<ProjectPropertyGroupElement>();
            foreach (ProjectPropertyGroupElement element in projectRoot.PropertyGroups)
            {
                if (!string.IsNullOrEmpty(element.Condition))
                {
                    list.Add(element);
                }
            }
            foreach (ProjectPropertyGroupElement element2 in list)
            {
                element2.Parent.RemoveChild(element2);
            }
        }

        private void ReplaceProjectReferences(ProjectRootElement projectRoot, List<ProjectItemElement> oldReferences)
        {
            if (oldReferences.Count > 0)
            {
                foreach (ProjectItemElement element in oldReferences)
                {
                    element.Parent.RemoveChild(element);
                    Dictionary<string, string> metadata = new Dictionary<string, string>();
                    foreach (ProjectMetadataElement element2 in element.Metadata)
                    {
                        metadata.Add(element2.Name, element2.Value);
                    }
                    try
                    {
                        if (metadata.ContainsKey("XnaReferenceType"))
                        {
                            projectRoot.AddItem(element.ItemType, element.Include, metadata);
                        }
                        else
                        {
                            IXnaCodeProject referencedProject = this.GetReferencedProject(element, metadata);
                            if (referencedProject == null)
                            {
                                string fileNameWithoutExtension = Path.GetFileNameWithoutExtension(element.Include);
                                this.errors.Add(new WizardException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardProjectTypeNotSupported, new object[] { fileNameWithoutExtension })));
                            }
                            else
                            {
                                IXnaCodeProject crossPlatformProject = this.GetCrossPlatformProject(referencedProject);
                                if (crossPlatformProject != null)
                                {
                                    this.AddMSBuildProjectReference(crossPlatformProject, projectRoot);
                                }
                                else if (!this.AddAlreadyConvertedProjectReference(referencedProject, projectRoot))
                                {
                                    CrossPlatformProjectWizard wizard = new CrossPlatformProjectWizard(referencedProject, this.targetPlatform, this.convertedProjects);
                                    wizard.Execute();
                                    this.userFeedback.Append(wizard.GetFeedback());
                                    this.AddMSBuildProjectReference(wizard.ConvertedProjectReference, projectRoot);
                                }
                            }
                        }
                    }
                    catch (COMException exception)
                    {
                        string str2 = Path.GetFileNameWithoutExtension(element.Include);
                        this.errors.Add(new WizardException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.XPlatWizardProjectErrorDuringConversion, new object[] { str2, exception.Message }), exception));
                    }
                    catch (WizardException exception2)
                    {
                        this.errors.Add(exception2);
                    }
                }
            }
        }

        private void ReplaceReferences(ProjectRootElement projectRoot, List<ProjectItemElement> oldReferences)
        {
            foreach (ProjectItemElement element in oldReferences)
            {
                element.Parent.RemoveChild(element);
            }
            ProjectItemGroupElement newReferences = projectRoot.AddItemGroup();
            newReferences.AddItem("Reference", "mscorlib");
            string format = "{0}";
            if (this.targetPlatform == "Windows")
            {
                format = "{0}, Version=4.0.0.0, Culture=neutral, PublicKeyToken=842cf8be1de50553, processorArchitecture=x86";
            }
            foreach (ProjectItemElement element3 in oldReferences)
            {
                AssemblyName oldAssemblyName = new AssemblyName(element3.Include);
                if (!string.Equals(oldAssemblyName.Name, "mscorlib", StringComparison.OrdinalIgnoreCase))
                {
                    if (!string.IsNullOrEmpty(oldAssemblyName.Name) && oldAssemblyName.Name.StartsWith("microsoft.xna.framework", StringComparison.OrdinalIgnoreCase))
                    {
                        string newAssemblyName = string.Format(format, oldAssemblyName.Name);
                        this.AddReplacementReference(newReferences, oldAssemblyName, newAssemblyName);
                    }
                    else
                    {
                        this.AddReplacementReference(newReferences, oldAssemblyName, oldAssemblyName.Name);
                    }
                }
            }
        }

        private void SetCommonDefaultConfigProperties(ProjectPropertyGroupElement config, string configName)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }
            if (configName == null)
            {
                throw new ArgumentNullException("configName");
            }
            config.Condition = string.Format(CultureInfo.InvariantCulture, " '$(Configuration)|$(Platform)' == '{0}|{1}' ", new object[] { configName, this.targetAdapter.PlatformConfigurationName });
            config.AddProperty("OutputPath", string.Format(CultureInfo.InvariantCulture, @"bin\{0}\{1}", new object[] { this.targetAdapter.PlatformConfigurationName, configName }));
            config.AddProperty("ErrorReport", "prompt");
            config.AddProperty("WarningLevel", "4");
            config.AddProperty("NoStdLib", "true");
            config.AddProperty("UseVSHostingProcess", "false");
        }

        private void SetDefaultDebugConfigProperties(ProjectPropertyGroupElement config)
        {
            this.SetCommonDefaultConfigProperties(config, "Debug");
            config.AddProperty("DebugSymbols", "true");
            config.AddProperty("DebugType", "full");
            config.AddProperty("Optimize", "false");
            string platformDefines = this.GetPlatformDefines();
            if (string.IsNullOrEmpty(platformDefines))
            {
                config.AddProperty("DefineConstants", this.xnaProject.LanguageAdapter.TranslateProjectDefines("DEBUG;TRACE"));
            }
            else
            {
                config.AddProperty("DefineConstants", this.xnaProject.LanguageAdapter.TranslateProjectDefines("DEBUG;TRACE;" + platformDefines));
            }
        }

        private void SetDefaultReleaseConfigProperties(ProjectPropertyGroupElement config)
        {
            this.SetCommonDefaultConfigProperties(config, "Release");
            config.AddProperty("DebugType", "pdbonly");
            config.AddProperty("Optimize", "true");
            string platformDefines = this.GetPlatformDefines();
            if (string.IsNullOrEmpty(platformDefines))
            {
                config.AddProperty("DefineConstants", this.xnaProject.LanguageAdapter.TranslateProjectDefines("TRACE"));
            }
            else
            {
                config.AddProperty("DefineConstants", this.xnaProject.LanguageAdapter.TranslateProjectDefines("TRACE;" + platformDefines));
            }
        }

        private void SetTargetPlatformAndProfile(string requestedTargetPlatform)
        {
            if (string.IsNullOrEmpty(requestedTargetPlatform))
            {
                throw new ArgumentNullException("requestedTargetPlatform");
            }
            this.availableAdapters.TryGetValue(new KeyValuePair<string, string>(this.xnaProject.XnaFrameworkVersion, requestedTargetPlatform), out this.targetAdapter);
            if (this.targetAdapter == null)
            {
                throw new ArgumentException(XnaProjectResource.XPlatWizardInvalidTargetPlatform, "requestedTargetPlatform");
            }
            this.targetPlatform = requestedTargetPlatform;
            if ((this.targetPlatform == "Windows Phone") || (this.targetPlatform == "Windows"))
            {
                this.targetProfile = "Reach";
            }
            else
            {
                this.targetProfile = "HiDef";
            }
        }

        private ConvertedProject ConvertedProjectReference
        {
            get
            {
                return this.convertedProject;
            }
        }

        private class ConvertedProject
        {
            private string groupID;
            private string mkDocument;
            private string projectGuid;

            public ConvertedProject(string mkDocument, string projectGuid, string groupID)
            {
                if (mkDocument == null)
                {
                    throw new ArgumentNullException("mkDocument");
                }
                if (projectGuid == null)
                {
                    throw new ArgumentNullException("projectGuid");
                }
                if (groupID == null)
                {
                    throw new ArgumentNullException("groupID");
                }
                this.mkDocument = mkDocument;
                this.projectGuid = projectGuid;
                this.groupID = groupID;
            }

            public string GroupID
            {
                get
                {
                    return this.groupID;
                }
            }

            public string MkDocument
            {
                get
                {
                    return this.mkDocument;
                }
            }

            public string ProjectGuid
            {
                get
                {
                    return this.projectGuid;
                }
            }
        }
    }
}

