﻿namespace Microsoft.Xna.GameStudio
{
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using Microsoft.Win32;
    using Microsoft.Xna.GameStudio.CodeProject;
    using Microsoft.Xna.GameStudio.CodeProject.Debugging;
    using Microsoft.Xna.GameStudio.ContentProject;
    using Microsoft.Xna.GameStudio.RegistrationAttributes;
    using Microsoft.Xna.GameStudio.Resources;
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.Diagnostics;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Resources;
    using System.Runtime.CompilerServices;
    using System.Runtime.InteropServices;

    [ProvideManagedMenuResource("XnaPackage.ctmenu", 1), ProvideObject(typeof(ContentBuildPropertyPage)), ProvideObject(typeof(ContentCompressionPropertyPage)), ProvideObject(typeof(XnaPortSupplier)), ProvideObject(typeof(XnaGameStudioPropPage)), ProvideAutoLoad("{EF4F870B-7B85-4F29-9D15-CE1ABFBE733B}"), Guid("4098508B-4144-45D3-9801-736DC39D8E8A"), ComVisible(true), CLSCompliant(false), ClassInterface(ClassInterfaceType.None), SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable", Justification="This is a managed implementation of a COM interface. Dispose(bool) is invoked through Close."), PackageRegistration(UseManagedResourcesOnly=true), InstalledProductRegistration(true, null, null, null)]
    public sealed class XnaPackage : Package, IVsInstalledProduct
    {
        private Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> adapters;
        private CentralDeviceSelectorProxy centralDeviceSelector;
        private bool enableCentralDeviceSelector;
        private CrossPlatformGroupManager groupManagerService;
        private static string productDetails;
        private Dictionary<IVsProjectFactory, uint> projectFactories;
        private static ResourceManager resourceManager;
        private SolutionEventsListener solutionEventsListener;
        private DeploymentRegistrationService xnaDeployService;
        private static XnaPackage xnaPackage;

        public XnaPackage()
        {
            this.IsProjectUpgradeInProgress = false;
            xnaPackage = this;
        }

        internal static bool CheckExtenderSignature(object extender)
        {
            return CheckExtenderTypeSignature(extender.GetType());
        }

        internal static bool CheckExtenderTypeSignature(Type extenderType)
        {
            byte[] publicKey = typeof(XnaPackage).Assembly.GetName().GetPublicKey();
            byte[] buffer2 = extenderType.Assembly.GetName().GetPublicKey();
            if (publicKey.Length != buffer2.Length)
            {
                return false;
            }
            for (int i = 0; i < publicKey.Length; i++)
            {
                if (publicKey[i] != buffer2[i])
                {
                    return false;
                }
            }
            return true;
        }

        private void comboBoxCommand_BeforeQueryStatus(object sender, EventArgs e)
        {
            OleMenuCommand command = sender as OleMenuCommand;
            if (command != null)
            {
                command.Supported = true;
                command.Visible = !this.enableCentralDeviceSelector;
                command.Enabled = ((this.xnaDeployService != null) && this.xnaDeployService.ContainsValidDevices) && !this.enableCentralDeviceSelector;
            }
        }

        private void deviceToolbarButtonCommand_BeforeQueryStatus(object sender, EventArgs e)
        {
            OleMenuCommand command = sender as OleMenuCommand;
            if (command != null)
            {
                command.Supported = true;
                bool flag = ((this.xnaDeployService != null) && this.xnaDeployService.CurrentPlatformSupportsDeviceCenter) && this.xnaDeployService.ContainsValidDevices;
                command.Visible = flag;
                command.Enabled = flag;
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                if (disposing)
                {
                    if (this.xnaDeployService != null)
                    {
                        this.xnaDeployService.Dispose();
                        this.xnaDeployService = null;
                    }
                    if (this.groupManagerService != null)
                    {
                        this.groupManagerService.Dispose();
                        this.groupManagerService = null;
                    }
                    if (this.projectFactories != null)
                    {
                        IVsRegisterProjectTypes service = base.GetService(typeof(SVsRegisterProjectTypes)) as IVsRegisterProjectTypes;
                        foreach (KeyValuePair<IVsProjectFactory, uint> pair in this.projectFactories)
                        {
                            try
                            {
                                if (service != null)
                                {
                                    service.UnregisterProjectType(pair.Value);
                                }
                            }
                            finally
                            {
                                IDisposable key = pair.Key as IDisposable;
                                if (key != null)
                                {
                                    key.Dispose();
                                }
                            }
                        }
                    }
                    if (this.solutionEventsListener != null)
                    {
                        this.solutionEventsListener.Dispose();
                        this.solutionEventsListener = null;
                    }
                    if (this.centralDeviceSelector != null)
                    {
                        this.centralDeviceSelector.SelectionChanged -= new EventHandler(this.OnCentralComboSelection);
                        this.centralDeviceSelector.Disconnect();
                    }
                }
            }
            finally
            {
                this.xnaDeployService = null;
                this.projectFactories = null;
                base.Dispose(disposing);
            }
        }

        public void EnableCentralDeviceDropdown(bool enable)
        {
            if (this.centralDeviceSelector != null)
            {
                bool flag = this.enableCentralDeviceSelector != enable;
                this.enableCentralDeviceSelector = enable;
                if (enable)
                {
                    if (flag)
                    {
                        this.centralDeviceSelector.Enable();
                    }
                    if (this.xnaDeployService != null)
                    {
                        this.xnaDeployService.SetCurrentSelection(this.centralDeviceSelector.SelectedDeviceId);
                    }
                }
                else if (flag)
                {
                    this.centralDeviceSelector.Disable();
                }
            }
        }

        private object GetCrossPlatformGroupManager(IServiceContainer container, Type serviceType)
        {
            CrossPlatformGroupManager manager = null;
            if (container != this)
            {
                return manager;
            }
            if (this.groupManagerService == null)
            {
                this.groupManagerService = new CrossPlatformGroupManager(this);
            }
            return this.groupManagerService;
        }

        private object GetIVsUIHierarchyWindow(IServiceContainer container, Type serviceType)
        {
            IVsUIHierarchyWindow window = null;
            if (container == this)
            {
                IVsUIShell globalService = Package.GetGlobalService(typeof(SVsUIShell)) as IVsUIShell;
                if (globalService != null)
                {
                    Guid rguidPersistenceSlot = new Guid("{3AE79031-E1BC-11D0-8F78-00A0C9110057}");
                    IVsWindowFrame ppWindowFrame = null;
                    if (ErrorHandler.Succeeded(globalService.FindToolWindow(0, ref rguidPersistenceSlot, out ppWindowFrame)))
                    {
                        object pvar = null;
                        ErrorHandler.ThrowOnFailure(ppWindowFrame.GetProperty(-3001, out pvar));
                        window = pvar as IVsUIHierarchyWindow;
                    }
                }
            }
            return window;
        }

        private object GetXnaDeployService(IServiceContainer container, Type serviceType)
        {
            DeploymentRegistrationService xnaDeployService = null;
            if (container == this)
            {
                if (this.xnaDeployService == null)
                {
                    this.xnaDeployService = new DeploymentRegistrationService(this);
                }
                xnaDeployService = this.xnaDeployService;
                if (this.enableCentralDeviceSelector)
                {
                    this.xnaDeployService.SetCurrentSelection(this.centralDeviceSelector.SelectedDeviceId);
                }
            }
            return xnaDeployService;
        }

        public int IdBmpSplash(out uint pIdBmp)
        {
            pIdBmp = 0;
            return -2147467263;
        }

        public int IdIcoLogoForAboutbox(out uint pIdIco)
        {
            pIdIco = 400;
            return 0;
        }

        [SuppressMessage("Microsoft.Reliability", "CA2000:DisposeObjectsBeforeLosingScope")]
        protected override void Initialize()
        {
            base.Initialize();
            this.RegisterCodeProjectFactories();
            base.RegisterProjectFactory(new ContentProjectFactory(this));
            IServiceContainer container = this;
            container.AddService(typeof(ISolutionBuildStatusProvider), new SolutionBuildStatusListener(this));
            container.AddService(typeof(IVsUIHierarchyWindow), new ServiceCreatorCallback(this.GetIVsUIHierarchyWindow));
            container.AddService(typeof(CrossPlatformGroupManager), new ServiceCreatorCallback(this.GetCrossPlatformGroupManager));
            container.AddService(typeof(IXnaDeviceDeploymentRegistration), new ServiceCreatorCallback(this.GetXnaDeployService));
            container.AddService(typeof(Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter>), this.adapters);
            this.solutionEventsListener = new SolutionEventsListener(this);
            container.AddService(typeof(SolutionEventsListener), this.solutionEventsListener);
            try
            {
                this.centralDeviceSelector = new CentralDeviceSelectorProxy();
                if (this.centralDeviceSelector.IsConnectable && this.centralDeviceSelector.Connect(base.GetService(this.centralDeviceSelector.CentralDeviceSelectorServiceType)))
                {
                    this.centralDeviceSelector.SelectionChanged += new EventHandler(this.OnCentralComboSelection);
                }
            }
            catch
            {
                this.centralDeviceSelector = null;
                this.enableCentralDeviceSelector = false;
            }
            OleMenuCommandService service = base.GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (service != null)
            {
                OleMenuCommand command = new OleMenuCommand(new EventHandler(this.OnComboSelection), CmdIDConstants.comboBoxCommandID);
                command.BeforeQueryStatus += new EventHandler(this.comboBoxCommand_BeforeQueryStatus);
                command.ParametersDescription = "$";
                service.AddCommand(command);
                OleMenuCommand command2 = new OleMenuCommand(new EventHandler(this.OnComboFill), CmdIDConstants.comboBoxFillCommandID);
                service.AddCommand(command2);
                OleMenuCommand command3 = new OleMenuCommand(new EventHandler(this.OnLaunchDevCenter), CmdIDConstants.buttonLaunchDevCenterCommandID);
                command3.BeforeQueryStatus += new EventHandler(this.deviceToolbarButtonCommand_BeforeQueryStatus);
                service.AddCommand(command3);
                OleMenuCommand command4 = new OleMenuCommand(new EventHandler(this.OnAddDevice), CmdIDConstants.buttonAddDeviceID);
                command4.BeforeQueryStatus += new EventHandler(this.deviceToolbarButtonCommand_BeforeQueryStatus);
                service.AddCommand(command4);
                service.AddCommand(new UpgradeSolutionCommand(this.solutionEventsListener, CmdIDConstants.XnaUpgradeSolutionCmdID));
                service.AddCommand(new UpgradeSolutionCommand(this.solutionEventsListener, CmdIDConstants.XnaUpgradeSolutionCtxCmdID));
            }
        }

        public int OfficialName(out string pbstrName)
        {
            pbstrName = VSPackage.GetString("100", null);
            return 0;
        }

        private void OnAddDevice(object sender, EventArgs e)
        {
            this.xnaDeployService.ShowAddDeviceDialog();
        }

        private void OnCentralComboSelection(object sender, EventArgs e)
        {
            if (e == null)
            {
                throw new ArgumentNullException("e");
            }
            if (this.enableCentralDeviceSelector && ((this.xnaDeployService != null) && (e is CentralDeviceSelectorProxy.CentralDeviceEventArgs)))
            {
                this.xnaDeployService.SetCurrentSelection(((CentralDeviceSelectorProxy.CentralDeviceEventArgs) e).DeviceId);
            }
        }

        private void OnComboFill(object sender, EventArgs e)
        {
            if (e == EventArgs.Empty)
            {
                throw new ArgumentNullException("e");
            }
            if (!this.enableCentralDeviceSelector)
            {
                OleMenuCmdEventArgs args = e as OleMenuCmdEventArgs;
                if ((args != null) && (this.xnaDeployService != null))
                {
                    IntPtr outValue = args.OutValue;
                    if (outValue != IntPtr.Zero)
                    {
                        Marshal.GetNativeVariantForObject(this.xnaDeployService.GetAvailableDevices(), outValue);
                    }
                }
            }
        }

        private void OnComboSelection(object sender, EventArgs e)
        {
            if ((e == null) || (e == EventArgs.Empty))
            {
                throw new ArgumentNullException("e");
            }
            if (!this.enableCentralDeviceSelector)
            {
                OleMenuCmdEventArgs args = e as OleMenuCmdEventArgs;
                if ((args != null) && (this.xnaDeployService != null))
                {
                    string inValue = args.InValue as string;
                    IntPtr outValue = args.OutValue;
                    if (outValue != IntPtr.Zero)
                    {
                        Marshal.GetNativeVariantForObject(this.xnaDeployService.GetCurrentSelection(), outValue);
                    }
                    else if (inValue != null)
                    {
                        this.xnaDeployService.SetCurrentSelection(inValue);
                    }
                }
            }
        }

        private void OnLaunchDevCenter(object sender, EventArgs e)
        {
            string str = Registry.GetValue(@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\XNA\Game Studio\", "SharedComponentsPath", "") as string;
            if (!string.IsNullOrEmpty(str))
            {
                string path = Path.Combine(str, @"Device Center\XnaDeviceCenter.exe");
                if (File.Exists(path))
                {
                    Process.Start(path);
                }
            }
        }

        public int ProductDetails(out string pbstrProductDetails)
        {
            if (productDetails == null)
            {
                string fileVersion = null;
                string str2 = VSPackage.GetString("102", null);
                if (!string.IsNullOrEmpty(str2))
                {
                    Assembly executingAssembly = Assembly.GetExecutingAssembly();
                    if (executingAssembly != null)
                    {
                        string location = executingAssembly.Location;
                        if (!string.IsNullOrEmpty(location))
                        {
                            try
                            {
                                FileVersionInfo versionInfo = FileVersionInfo.GetVersionInfo(location);
                                if (versionInfo != null)
                                {
                                    fileVersion = versionInfo.FileVersion;
                                }
                            }
                            catch (FileNotFoundException)
                            {
                            }
                        }
                    }
                    if (fileVersion == null)
                    {
                        fileVersion = "";
                    }
                    productDetails = string.Format(CultureInfo.CurrentUICulture, str2, new object[] { fileVersion });
                }
                if (productDetails == null)
                {
                    productDetails = "";
                }
            }
            pbstrProductDetails = productDetails;
            return 0;
        }

        public int ProductID(out string pbstrPID)
        {
            pbstrPID = VSPackage.GetString("101", null);
            return 0;
        }

        private void RegisterCodeProjectFactories()
        {
            ILocalRegistry3 service = base.GetService(typeof(SLocalRegistry)) as ILocalRegistry3;
            if (service == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(SLocalRegistry).FullName }));
            }
            string pbstrRoot = string.Empty;
            ErrorHandler.ThrowOnFailure(service.GetLocalRegistryRoot(out pbstrRoot));
            IVsShell shell = base.GetService(typeof(SVsShell)) as IVsShell;
            object pvar = null;
            int subSku = 0;
            if (shell == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(SVsShell).FullName }));
            }
            shell.GetProperty(-9047, out pvar);
            subSku = (int) pvar;
            shell.GetProperty(-9046, out pvar);
            this.adapters = new Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter>(new XnaProjectFactory.AdapterKeyComparer());
            string name = string.Format(CultureInfo.InvariantCulture, @"{0}\Packages\{1}\CodeProjects", new object[] { pbstrRoot, typeof(XnaPackage).GUID.ToString("B") });
            using (RegistryKey key = Registry.LocalMachine.OpenSubKey(name))
            {
                foreach (string str3 in key.GetSubKeyNames())
                {
                    using (RegistryKey key2 = key.OpenSubKey(str3))
                    {
                        string assemblyName = (string) key2.GetValue("Assembly");
                        string typeName = (string) key2.GetValue("Class");
                        IProjectPlatformAdapter extender = (IProjectPlatformAdapter) AppDomain.CurrentDomain.CreateInstance(assemblyName, typeName).Unwrap();
                        if (!(extender is IProjectSkuSupport) || (((IProjectSkuSupport) extender).SkuSupport(pvar, subSku, pbstrRoot) == SkuSupportLevel.Supported))
                        {
                            if (!CheckExtenderSignature(extender))
                            {
                                throw new InvalidOperationException(XnaProjectResource.InvalidPlatformAdapter);
                            }
                            KeyValuePair<string, string> pair = new KeyValuePair<string, string>(extender.XnaFrameworkVersion, extender.XnaPlatform);
                            if (this.adapters.ContainsKey(pair))
                            {
                                throw new InvalidOperationException(XnaProjectResource.InvalidPlatformAdapter);
                            }
                            this.adapters.Add(pair, extender);
                        }
                    }
                }
                this.RegisterCodeProjectFactory(typeof(EntryPointAdapter).GUID, this.adapters);
            }
            this.RegisterLegacyProjectType(new ZunePlatformLegacyAdapter());
            this.RegisterLegacyProjectType(new Xbox360PlatformLegacyAdapter());
        }

        private void RegisterCodeProjectFactory(Guid projectTypeGuid, Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> customProjectExtenders)
        {
            uint num;
            if (customProjectExtenders == null)
            {
                throw new ArgumentNullException("customProjectExtenders");
            }
            IVsRegisterProjectTypes service = base.GetService(typeof(SVsRegisterProjectTypes)) as IVsRegisterProjectTypes;
            if (service == null)
            {
                throw new InvalidOperationException(string.Format(CultureInfo.InvariantCulture, XnaProjectResource.MissingService, new object[] { typeof(SVsRegisterProjectTypes).FullName }));
            }
            Guid rguidProjType = projectTypeGuid;
            IVsProjectFactory pVsPF = new XnaProjectFactory(this, customProjectExtenders);
            ErrorHandler.ThrowOnFailure(service.RegisterProjectType(ref rguidProjType, pVsPF, out num));
            if (this.projectFactories == null)
            {
                this.projectFactories = new Dictionary<IVsProjectFactory, uint>();
            }
            this.projectFactories[pVsPF] = num;
        }

        private void RegisterLegacyProjectType(IProjectPlatformAdapter legacyAdapter)
        {
            if (legacyAdapter == null)
            {
                throw new ArgumentNullException("legacyAdapter");
            }
            Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter> customProjectExtenders = new Dictionary<KeyValuePair<string, string>, IProjectPlatformAdapter>(new XnaProjectFactory.AdapterKeyComparer());
            customProjectExtenders.Add(new KeyValuePair<string, string>(legacyAdapter.XnaFrameworkVersion, legacyAdapter.XnaPlatform), legacyAdapter);
            this.RegisterCodeProjectFactory(legacyAdapter.ProjectTypeGuid, customProjectExtenders);
        }

        internal static XnaPackage Instance
        {
            get
            {
                return xnaPackage;
            }
        }

        public bool IsProjectUpgradeInProgress { get; set; }

        internal static ResourceManager VSPackage
        {
            get
            {
                if (resourceManager == null)
                {
                    resourceManager = new ResourceManager("VSPackage", typeof(XnaPackage).Assembly);
                }
                return resourceManager;
            }
        }
    }
}

