﻿namespace Microsoft.Xna.GameStudio.RegistrationAttributes
{
    using Microsoft.VisualStudio.Shell;
    using System;
    using System.Globalization;

    [AttributeUsage(AttributeTargets.Class, AllowMultiple=true, Inherited=true)]
    public sealed class CodeGeneratorAttribute : RegistrationAttribute
    {
        private string _contextGuid;
        private bool _generatesDesignTimeSource = true;
        private bool _generatesSharedDesignTimeSource;
        private Guid _generatorGuid;
        private string _generatorName;
        private string _generatorRegKeyName;
        private Type _generatorType;

        public CodeGeneratorAttribute(Type generatorType, string generatorName, string contextGuid)
        {
            if (generatorType == null)
            {
                throw new ArgumentNullException("generatorType");
            }
            if (generatorName == null)
            {
                throw new ArgumentNullException("generatorName");
            }
            if (contextGuid == null)
            {
                throw new ArgumentNullException("contextGuid");
            }
            this._contextGuid = contextGuid;
            this._generatorType = generatorType;
            this._generatorName = generatorName;
            this._generatorRegKeyName = generatorType.Name;
            this._generatorGuid = generatorType.GUID;
        }

        public override void Register(RegistrationAttribute.RegistrationContext context)
        {
            using (RegistrationAttribute.Key key = context.CreateKey(this.GeneratorRegKey))
            {
                key.SetValue(string.Empty, this.GeneratorName);
                key.SetValue("CLSID", this.GeneratorGuid.ToString("B"));
                if (this.GeneratesDesignTimeSource)
                {
                    key.SetValue("GeneratesDesignTimeSource", 1);
                }
                if (this.GeneratesSharedDesignTimeSource)
                {
                    key.SetValue("GeneratesSharedDesignTimeSource", 1);
                }
            }
        }

        public override void Unregister(RegistrationAttribute.RegistrationContext context)
        {
            context.RemoveKey(this.GeneratorRegKey);
        }

        public string ContextGuid
        {
            get
            {
                return this._contextGuid;
            }
        }

        public bool GeneratesDesignTimeSource
        {
            get
            {
                return this._generatesDesignTimeSource;
            }
            set
            {
                this._generatesDesignTimeSource = value;
            }
        }

        public bool GeneratesSharedDesignTimeSource
        {
            get
            {
                return this._generatesSharedDesignTimeSource;
            }
            set
            {
                this._generatesSharedDesignTimeSource = value;
            }
        }

        public Guid GeneratorGuid
        {
            get
            {
                return this._generatorGuid;
            }
        }

        public string GeneratorName
        {
            get
            {
                return this._generatorName;
            }
        }

        private string GeneratorRegKey
        {
            get
            {
                return string.Format(CultureInfo.InvariantCulture, @"Generators\{0}\{1}", new object[] { this.ContextGuid, this.GeneratorRegKeyName });
            }
        }

        public string GeneratorRegKeyName
        {
            get
            {
                return this._generatorRegKeyName;
            }
            set
            {
                this._generatorRegKeyName = value;
            }
        }

        public Type GeneratorType
        {
            get
            {
                return this._generatorType;
            }
        }
    }
}

