﻿namespace Microsoft.Xna.GameStudio.RegistrationAttributes
{
    using Microsoft.VisualStudio.Shell;
    using Microsoft.Xna.GameStudio.Properties;
    using System;
    using System.Globalization;
    using System.IO;

    [AttributeUsage(AttributeTargets.Class, AllowMultiple=true)]
    public sealed class ProjectFileFilterAttribute : RegistrationAttribute
    {
        private bool commonFindFilesFilter;
        private bool commonOpenFilesFilter;
        private string filterName;
        private int filterResourceId;
        private string filterValue;
        private bool notAddExistingItemFilter;
        private bool notOpenFileFilter;
        private string projectGuid;
        private Type projectType;
        private int sortPriority;

        public ProjectFileFilterAttribute(Type projectType, string filterName, int filterResourceId, int sortPriority)
        {
            if (projectType == null)
            {
                throw new ArgumentNullException("projectType");
            }
            if (filterName == null)
            {
                throw new ArgumentNullException("filterName");
            }
            if (filterName.Length == 0)
            {
                throw new ArgumentException(Microsoft.Xna.GameStudio.Properties.Resources.NoEmptyString, "filterName");
            }
            this.projectType = projectType;
            this.projectGuid = projectType.GUID.ToString("B").ToUpperInvariant();
            this.filterName = filterName;
            this.filterResourceId = filterResourceId;
            this.filterValue = "#" + filterResourceId.ToString(CultureInfo.InvariantCulture);
            this.sortPriority = sortPriority;
        }

        public ProjectFileFilterAttribute(Type projectType, string filterName, string filterValue, int sortPriority)
        {
            if (projectType == null)
            {
                throw new ArgumentNullException("projectType");
            }
            if (filterName == null)
            {
                throw new ArgumentNullException("filterName");
            }
            if (filterValue == null)
            {
                throw new ArgumentNullException("filterValue");
            }
            if (filterName.Length == 0)
            {
                throw new ArgumentException(Microsoft.Xna.GameStudio.Properties.Resources.NoEmptyString, "filterName");
            }
            if (filterValue.Length == 0)
            {
                throw new ArgumentException(Microsoft.Xna.GameStudio.Properties.Resources.NoEmptyString, "filterValue");
            }
            this.projectType = projectType;
            this.projectGuid = projectType.GUID.ToString("B").ToUpperInvariant();
            this.filterName = filterName;
            this.filterValue = filterValue;
            this.sortPriority = sortPriority;
        }

        public override void Register(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            using (RegistrationAttribute.Key key = context.CreateKey(this.ProjectFiltersKeyPath))
            {
                using (RegistrationAttribute.Key key2 = key.CreateSubkey(this.filterName))
                {
                    key2.SetValue(null, this.filterValue);
                    key2.SetValue("SortPriority", this.sortPriority);
                    if (this.commonFindFilesFilter)
                    {
                        key2.SetValue("CommonFindFilesFilter", 1);
                    }
                    if (this.commonOpenFilesFilter)
                    {
                        key2.SetValue("CommonOpenFilesFilter", 1);
                    }
                    if (this.notAddExistingItemFilter)
                    {
                        key2.SetValue("NotAddExistingItemFilter", 1);
                    }
                    if (this.notOpenFileFilter)
                    {
                        key2.SetValue("NotOpenFileFilter", 1);
                    }
                }
            }
        }

        public override void Unregister(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            string name = Path.Combine(this.ProjectFiltersKeyPath, this.filterName);
            context.RemoveKey(name);
            context.RemoveKeyIfEmpty(this.ProjectFiltersKeyPath);
        }

        public bool CommonFindFilesFilter
        {
            get
            {
                return this.commonFindFilesFilter;
            }
            set
            {
                this.commonFindFilesFilter = value;
            }
        }

        public bool CommonOpenFilesFilter
        {
            get
            {
                return this.commonOpenFilesFilter;
            }
            set
            {
                this.commonOpenFilesFilter = value;
            }
        }

        public string FilterName
        {
            get
            {
                return this.filterName;
            }
        }

        public int FilterResourceId
        {
            get
            {
                return this.filterResourceId;
            }
        }

        public string FilterValue
        {
            get
            {
                return this.filterValue;
            }
        }

        public bool NotAddExistingItemFilter
        {
            get
            {
                return this.notAddExistingItemFilter;
            }
            set
            {
                this.notAddExistingItemFilter = value;
            }
        }

        public bool NotOpenFileFilter
        {
            get
            {
                return this.notOpenFileFilter;
            }
            set
            {
                this.notOpenFileFilter = value;
            }
        }

        private string ProjectFiltersKeyPath
        {
            get
            {
                return (@"Projects\" + this.projectGuid + @"\Filters");
            }
        }

        public Type ProjectType
        {
            get
            {
                return this.projectType;
            }
        }

        public int SortPriority
        {
            get
            {
                return this.sortPriority;
            }
        }
    }
}

