﻿namespace Microsoft.Xna.GameStudio.RegistrationAttributes
{
    using Microsoft.VisualStudio.Shell;
    using System;
    using System.Diagnostics.CodeAnalysis;
    using System.Globalization;
    using System.IO;

    [AttributeUsage(AttributeTargets.Class, AllowMultiple=true)]
    public sealed class ProjectTypeAttribute : RegistrationAttribute
    {
        private string defaultProjectExtension;
        private string displayProjectFileExtensions;
        private string displayProjectTypeVsTemplate;
        private Type factoryType;
        private Guid folderGuid = Guid.Empty;
        private string languageVsTemplate;
        private string name;
        private bool newProjectRequireNewFolderVsTemplate;
        private Type packageType;
        private string possibleProjectExtensions;
        private string projectSubTypeVsTemplate;
        private string projectTemplatesDirectory;
        private bool showOnlySpecifiedTemplatesVsTemplate;
        private int sortPriority = 100;
        private string templateGroupIDsVsTemplate;
        private string templateIDsVsTemplate;

        public ProjectTypeAttribute(Type factoryType, Type packageType, string name, string displayProjectFileExtensionsResourceId, string defaultProjectExtension, string possibleProjectExtensions, string projectTemplatesDirectory)
        {
            if (factoryType == null)
            {
                throw new ArgumentNullException("factoryType");
            }
            if (packageType == null)
            {
                throw new ArgumentNullException("packageType");
            }
            this.factoryType = factoryType;
            this.packageType = packageType;
            this.name = name;
            this.displayProjectFileExtensions = displayProjectFileExtensionsResourceId;
            this.defaultProjectExtension = defaultProjectExtension;
            this.possibleProjectExtensions = possibleProjectExtensions;
            this.projectTemplatesDirectory = projectTemplatesDirectory;
        }

        private static string NewPrjTemplateRegKey(RegistrationAttribute.RegistrationContext context)
        {
            return string.Format(CultureInfo.InvariantCulture, @"NewProjectTemplates\TemplateDirs\{0}\/1", new object[] { context.ComponentType.GUID.ToString("B").ToUpperInvariant() });
        }

        public override void Register(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            context.Log.WriteLine("Project Type: {0}", this.FactoryType.Name);
            using (RegistrationAttribute.Key key = context.CreateKey(this.ProjectRegKey))
            {
                key.SetValue(string.Empty, this.FactoryType.Name);
                if (this.name != null)
                {
                    key.SetValue("DisplayName", this.name);
                }
                if (this.displayProjectFileExtensions != null)
                {
                    key.SetValue("DisplayProjectFileExtensions", this.displayProjectFileExtensions);
                }
                key.SetValue("Package", this.packageType.GUID.ToString("B").ToUpperInvariant());
                if (this.defaultProjectExtension != null)
                {
                    key.SetValue("DefaultProjectExtension", this.defaultProjectExtension);
                }
                if (this.possibleProjectExtensions != null)
                {
                    key.SetValue("PossibleProjectExtensions", this.possibleProjectExtensions);
                }
                if (this.projectTemplatesDirectory != null)
                {
                    if (!Path.IsPathRooted(this.projectTemplatesDirectory))
                    {
                        this.projectTemplatesDirectory = Path.Combine(context.ComponentPath, this.projectTemplatesDirectory);
                    }
                    key.SetValue("ProjectTemplatesDir", this.projectTemplatesDirectory);
                }
                if (this.languageVsTemplate != null)
                {
                    key.SetValue("Language(VsTemplate)", this.languageVsTemplate);
                }
                if (this.showOnlySpecifiedTemplatesVsTemplate)
                {
                    key.SetValue("ShowOnlySpecifiedTemplates(VsTemplate)", 1);
                }
                if (this.templateGroupIDsVsTemplate != null)
                {
                    key.SetValue("TemplateGroupIDs(VsTemplate)", this.templateGroupIDsVsTemplate);
                }
                if (this.templateIDsVsTemplate != null)
                {
                    key.SetValue("TemplateIDs(VsTemplate)", this.templateIDsVsTemplate);
                }
                if (this.displayProjectTypeVsTemplate != null)
                {
                    key.SetValue("DisplayProjectType(VsTemplate)", this.displayProjectTypeVsTemplate);
                }
                if (this.projectSubTypeVsTemplate != null)
                {
                    key.SetValue("ProjectSubType(VsTemplate)", this.projectSubTypeVsTemplate);
                }
                if (this.newProjectRequireNewFolderVsTemplate)
                {
                    key.SetValue("NewProjectRequireNewFolder(VsTemplate)", 1);
                }
            }
            if (!string.IsNullOrEmpty(this.projectTemplatesDirectory) || (this.folderGuid != Guid.Empty))
            {
                using (RegistrationAttribute.Key key2 = context.CreateKey(NewPrjTemplateRegKey(context)))
                {
                    string valueName = string.Empty;
                    if (this.name != null)
                    {
                        key2.SetValue(valueName, this.name);
                    }
                    key2.SetValue("SortPriority", this.sortPriority);
                    if (this.projectTemplatesDirectory != null)
                    {
                        key2.SetValue("TemplatesDir", this.projectTemplatesDirectory);
                    }
                    if (this.folderGuid != Guid.Empty)
                    {
                        key2.SetValue("Folder", this.FolderGuid);
                    }
                }
            }
        }

        public override void Unregister(RegistrationAttribute.RegistrationContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            context.RemoveKey(this.ProjectRegKey);
            context.RemoveKey(NewPrjTemplateRegKey(context));
        }

        public string DefaultProjectExtension
        {
            get
            {
                return this.defaultProjectExtension;
            }
        }

        public string DisplayProjectFileExtensionsResourceId
        {
            get
            {
                return this.displayProjectFileExtensions;
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public string DisplayProjectTypeVsTemplate
        {
            get
            {
                return this.displayProjectTypeVsTemplate;
            }
            set
            {
                this.displayProjectTypeVsTemplate = value;
            }
        }

        public Type FactoryType
        {
            get
            {
                return this.factoryType;
            }
        }

        public string FolderGuid
        {
            get
            {
                return this.folderGuid.ToString("B").ToUpperInvariant();
            }
            set
            {
                this.folderGuid = new Guid(value);
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public string LanguageVsTemplate
        {
            get
            {
                return this.languageVsTemplate;
            }
            set
            {
                this.languageVsTemplate = value;
            }
        }

        public string Name
        {
            get
            {
                return this.name;
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public bool NewProjectRequireNewFolderVsTemplate
        {
            get
            {
                return this.newProjectRequireNewFolderVsTemplate;
            }
            set
            {
                this.newProjectRequireNewFolderVsTemplate = value;
            }
        }

        public Type PackageType
        {
            get
            {
                return this.packageType;
            }
        }

        public string PossibleProjectExtensions
        {
            get
            {
                return this.possibleProjectExtensions;
            }
        }

        private string ProjectRegKey
        {
            get
            {
                return string.Format(CultureInfo.InvariantCulture, @"Projects\{0}", new object[] { this.FactoryType.GUID.ToString("B").ToUpperInvariant() });
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public string ProjectSubTypeVsTemplate
        {
            get
            {
                return this.projectSubTypeVsTemplate;
            }
            set
            {
                this.projectSubTypeVsTemplate = value;
            }
        }

        public string ProjectTemplatesDirectory
        {
            get
            {
                return this.projectTemplatesDirectory;
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public bool ShowOnlySpecifiedTemplatesVsTemplate
        {
            get
            {
                return this.showOnlySpecifiedTemplatesVsTemplate;
            }
            set
            {
                this.showOnlySpecifiedTemplatesVsTemplate = value;
            }
        }

        public int SortPriority
        {
            get
            {
                return this.sortPriority;
            }
            set
            {
                this.sortPriority = value;
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public string TemplateGroupIDsVsTemplate
        {
            get
            {
                return this.templateGroupIDsVsTemplate;
            }
            set
            {
                this.templateGroupIDsVsTemplate = value;
            }
        }

        [SuppressMessage("Microsoft.Naming", "CA1706:ShortAcronymsShouldBeUppercase", MessageId="Member")]
        public string TemplateIDsVsTemplate
        {
            get
            {
                return this.templateIDsVsTemplate;
            }
            set
            {
                this.templateIDsVsTemplate = value;
            }
        }
    }
}

